@@ -1,6 +1,6 @@
 /* StarPU --- Runtime system for heterogeneous multicore architectures.
  *
- * Copyright (C) 2010, 2011, 2012  Centre National de la Recherche Scientifique
+ * Copyright (C) 2010, 2011, 2012, 2013, 2015, 2016  CNRS
  *
  * StarPU is free software; you can redistribute it and/or modify
  * it under the terms of the GNU Lesser General Public License as published by
@@ -14,13 +14,18 @@
  * See the GNU Lesser General Public License in COPYING.LGPL for more details.
  */
 
+/*
+ * This examplifies how to use partitioning filters.  We here just split a 2D
+ * matrix into 2D slices (along the X axis), and run a dumb kernel on them.
+ */
+
 #include <starpu.h>
 
 #define NX    5
 #define NY    4
 #define PARTS 2
 
-#define FPRINTF(ofile, fmt, args ...) do { if (!getenv("STARPU_SSILENT")) {fprintf(ofile, fmt, ##args); }} while(0)
+#define FPRINTF(ofile, fmt, ...) do { if (!getenv("STARPU_SSILENT")) {fprintf(ofile, fmt, ## __VA_ARGS__); }} while(0)
 
 void cpu_func(void *buffers[], void *cl_arg)
 {
@@ -37,15 +42,17 @@ void cpu_func(void *buffers[], void *cl_arg)
         for(j=0; j<ny ; j++)
 	{
                 for(i=0; i<nx ; i++)
-                        val[(j*ld)+i] = *factor;
+                        val[(j*ld)+i] *= *factor;
         }
 }
 
 int main(int argc, char **argv)
 {
-	unsigned i, j, n=1;
+	unsigned j;
+	int n=1;
         int matrix[NX*NY];
-	int ret;
+	int ret, i;
+	int factor = 12;
 
         FPRINTF(stderr,"IN  Matrix: \n");
         for(j=0 ; j<NY ; j++)
@@ -53,7 +60,7 @@ int main(int argc, char **argv)
                 for(i=0 ; i<NX ; i++)
 		{
                         matrix[(j*NX)+i] = n++;
-                        FPRINTF(stderr, "%2d ", matrix[(j*NX)+i]);
+                        FPRINTF(stderr, "%4d ", matrix[(j*NX)+i]);
                 }
                 FPRINTF(stderr,"\n");
         }
@@ -62,10 +69,11 @@ int main(int argc, char **argv)
         starpu_data_handle_t handle;
         struct starpu_codelet cl =
 	{
-                .where = STARPU_CPU,
-                .cpu_funcs = {cpu_func, NULL},
+                .cpu_funcs = {cpu_func},
+                .cpu_funcs_name = {"cpu_func"},
                 .nbuffers = 1,
-		.modes = {STARPU_RW}
+		.modes = {STARPU_RW},
+		.name = "matrix_scal"
         };
 
         ret = starpu_init(NULL);
@@ -74,12 +82,12 @@ int main(int argc, char **argv)
 	STARPU_CHECK_RETURN_VALUE(ret, "starpu_init");
 
 	/* Declare data to StarPU */
-	starpu_matrix_data_register(&handle, 0, (uintptr_t)matrix, NX, NX, NY, sizeof(matrix[0]));
+	starpu_matrix_data_register(&handle, STARPU_MAIN_RAM, (uintptr_t)matrix, NX, NX, NY, sizeof(matrix[0]));
 
         /* Partition the matrix in PARTS sub-matrices */
 	struct starpu_data_filter f =
 	{
-		.filter_func = starpu_block_filter_func,
+		.filter_func = starpu_matrix_filter_block,
 		.nchildren = PARTS
 	};
 	starpu_data_partition(handle, &f);
@@ -88,7 +96,6 @@ int main(int argc, char **argv)
 	for (i=0; i<starpu_data_get_nb_children(handle); i++)
 	{
                 struct starpu_task *task = starpu_task_create();
-                int factor = i;
 		task->handles[0] = starpu_data_get_sub_data(handle, 1, i);
                 task->cl = &cl;
                 task->synchronous = 1;
@@ -101,23 +108,30 @@ int main(int argc, char **argv)
 	}
 
         /* Unpartition the data, unregister it from StarPU and shutdown */
-	starpu_data_unpartition(handle, 0);
+	starpu_data_unpartition(handle, STARPU_MAIN_RAM);
         starpu_data_unregister(handle);
 	starpu_shutdown();
 
         /* Print result matrix */
+	n=1;
         FPRINTF(stderr,"OUT Matrix: \n");
         for(j=0 ; j<NY ; j++)
 	{
                 for(i=0 ; i<NX ; i++)
 		{
-                        FPRINTF(stderr, "%2d ", matrix[(j*NX)+i]);
+                        FPRINTF(stderr, "%4d ", matrix[(j*NX)+i]);
+			if (matrix[(j*NX)+i] != (int) n*12)
+			{
+				FPRINTF(stderr, "Incorrect result %4d != %4d", matrix[(j*NX)+i], n*12);
+				ret=1;
+			}
+			n++;
                 }
                 FPRINTF(stderr,"\n");
         }
         FPRINTF(stderr,"\n");
 
-	return 0;
+	return ret;
 
 enodev:
 	starpu_shutdown();
