@@ -1,8 +1,8 @@
 /* StarPU --- Runtime system for heterogeneous multicore architectures.
  *
- * Copyright (C) 2010-2012  Université de Bordeaux 1
+ * Copyright (C) 2010-2012, 2014-2015, 2017  Université de Bordeaux
  * Copyright (C) 2010  Mehdi Juhoor <mjuhoor@gmail.com>
- * Copyright (C) 2010, 2011, 2012  Centre National de la Recherche Scientifique
+ * Copyright (C) 2010, 2011, 2012, 2016  CNRS
  *
  * StarPU is free software; you can redistribute it and/or modify
  * it under the terms of the GNU Lesser General Public License as published by
@@ -16,6 +16,9 @@
  * See the GNU Lesser General Public License in COPYING.LGPL for more details.
  */
 
+/* LU StarPU implementation using implicit task dependencies and partial
+ * pivoting */
+
 #include "xlu.h"
 #include "xlu_kernels.h"
 
@@ -39,6 +42,8 @@ static int create_task_pivot(starpu_data_handle_t *dataAp, unsigned nblocks,
 	/* which sub-data is manipulated ? */
 	task->handles[0] = get_block(dataAp, nblocks, k, i);
 
+	task->tag_id = PIVOT(k, i);
+
 	task->cl_arg = &piv_description[k];
 
 	/* this is an important task */
@@ -65,6 +70,8 @@ static int create_task_11_pivot(starpu_data_handle_t *dataAp, unsigned nblocks,
 	/* which sub-data is manipulated ? */
 	task->handles[0] = get_block(dataAp, nblocks, k, k);
 
+	task->tag_id = TAG11(k);
+
 	/* this is an important task */
 	if (!no_prio)
 		task->priority = STARPU_MAX_PRIO;
@@ -86,6 +93,8 @@ static int create_task_12(starpu_data_handle_t *dataAp, unsigned nblocks, unsign
 	task->handles[0] = get_block(dataAp, nblocks, k, k);
 	task->handles[1] = get_block(dataAp, nblocks, j, k);
 
+	task->tag_id = TAG12(k,j);
+
 	if (!no_prio && (j == k+1))
 		task->priority = STARPU_MAX_PRIO;
 
@@ -106,6 +115,8 @@ static int create_task_21(starpu_data_handle_t *dataAp, unsigned nblocks, unsign
 	task->handles[0] = get_block(dataAp, nblocks, k, k);
 	task->handles[1] = get_block(dataAp, nblocks, k, i);
 
+	task->tag_id = TAG21(k,i);
+
 	if (!no_prio && (i == k+1))
 		task->priority = STARPU_MAX_PRIO;
 
@@ -127,6 +138,8 @@ static int create_task_22(starpu_data_handle_t *dataAp, unsigned nblocks, unsign
 	task->handles[1] = get_block(dataAp, nblocks, j, k);
 	task->handles[2] = get_block(dataAp, nblocks, j, i);
 
+	task->tag_id = TAG22(k,i,j);
+
 	if (!no_prio &&  (i == k + 1) && (j == k +1) )
 		task->priority = STARPU_MAX_PRIO;
 
@@ -147,17 +160,23 @@ static int dw_codelet_facto_pivot(starpu_data_handle_t *dataAp,
 {
 	double start;
 	double end;
-	int ret;
 
 	/* create all the DAG nodes */
 	unsigned i,j,k;
 
+	if (bound)
+		starpu_bound_start(bounddeps, boundprio);
+
 	start = starpu_timing_now();
 
 	for (k = 0; k < nblocks; k++)
 	{
-	     ret = create_task_11_pivot(dataAp, nblocks, k, piv_description, get_block);
-	     if (ret == -ENODEV) return ret;
+		int ret;
+
+		starpu_iteration_push(k);
+
+		ret = create_task_11_pivot(dataAp, nblocks, k, piv_description, get_block);
+		if (ret == -ENODEV) return ret;
 
 		for (i = 0; i < nblocks; i++)
 		{
@@ -175,6 +194,7 @@ static int dw_codelet_facto_pivot(starpu_data_handle_t *dataAp,
 		     ret = create_task_21(dataAp, nblocks, k, i, get_block);
 		     if (ret == -ENODEV) return ret;
 		}
+		starpu_data_wont_use(get_block(dataAp, nblocks, k, k));
 
 		for (i = k+1; i<nblocks; i++)
 		     for (j = k+1; j<nblocks; j++)
@@ -182,6 +202,12 @@ static int dw_codelet_facto_pivot(starpu_data_handle_t *dataAp,
 			  ret = create_task_22(dataAp, nblocks, k, i, j, get_block);
 			  if (ret == -ENODEV) return ret;
 		     }
+		for (i = k+1; i<nblocks; i++)
+		{
+		    starpu_data_wont_use(get_block(dataAp, nblocks, k, i));
+		    starpu_data_wont_use(get_block(dataAp, nblocks, i, k));
+		}
+		starpu_iteration_pop();
 	}
 
 	/* stall the application until the end of computations */
@@ -189,6 +215,9 @@ static int dw_codelet_facto_pivot(starpu_data_handle_t *dataAp,
 
 	end = starpu_timing_now();
 
+	if (bound)
+		starpu_bound_stop();
+
 	*timing = end - start;
 	return 0;
 }
@@ -203,11 +232,15 @@ starpu_data_handle_t get_block_with_striding(starpu_data_handle_t *dataAp,
 
 int STARPU_LU(lu_decomposition_pivot)(TYPE *matA, unsigned *ipiv, unsigned size, unsigned ld, unsigned nblocks)
 {
+	if (starpu_mic_worker_get_count() || starpu_scc_worker_get_count())
+		/* These won't work with pivoting: we pass a pointer in cl_args */
+		return -ENODEV;
+
 	starpu_data_handle_t dataA;
 
 	/* monitor and partition the A matrix into blocks :
 	 * one block is now determined by 2 unsigned (i,j) */
-	starpu_matrix_data_register(&dataA, 0, (uintptr_t)matA, ld, size, size, sizeof(TYPE));
+	starpu_matrix_data_register(&dataA, STARPU_MAIN_RAM, (uintptr_t)matA, ld, size, size, sizeof(TYPE));
 
 	struct starpu_data_filter f =
 	{
@@ -238,16 +271,28 @@ int STARPU_LU(lu_decomposition_pivot)(TYPE *matA, unsigned *ipiv, unsigned size,
 
 	double timing;
 	int ret = dw_codelet_facto_pivot(&dataA, piv_description, nblocks, get_block_with_striding, &timing);
-
-	FPRINTF(stderr, "Computation took (in ms)\n");
-	FPRINTF(stderr, "%2.2f\n", timing/1000);
+	if (ret)
+		return ret;
 
 	unsigned n = starpu_matrix_get_nx(dataA);
 	double flop = (2.0f*n*n*n)/3.0f;
-	FPRINTF(stderr, "Synthetic GFlops : %2.2f\n", (flop/timing/1000.0f));
+
+	PRINTF("# size\tms\tGFlops");
+	if (bound)
+		PRINTF("\tTms\tTGFlops");
+	PRINTF("\n");
+	PRINTF("%u\t%.0f\t%.1f", n, timing/1000, flop/timing/1000.0f);
+	if (bound)
+	{
+		double min;
+		starpu_bound_compute(&min, NULL, 0);
+		PRINTF("\t%.0f\t%.1f", min, flop/min/1000000.0f);
+	}
+	PRINTF("\n");
+
 
 	/* gather all the data */
-	starpu_data_unpartition(dataA, 0);
+	starpu_data_unpartition(dataA, STARPU_MAIN_RAM);
 	starpu_data_unregister(dataA);
 
 	free(piv_description);
@@ -271,7 +316,7 @@ int STARPU_LU(lu_decomposition_pivot_no_stride)(TYPE **matA, unsigned *ipiv, uns
 	for (bj = 0; bj < nblocks; bj++)
 	for (bi = 0; bi < nblocks; bi++)
 	{
-		starpu_matrix_data_register(&dataAp[bi+nblocks*bj], 0,
+		starpu_matrix_data_register(&dataAp[bi+nblocks*bj], STARPU_MAIN_RAM,
 			(uintptr_t)matA[bi+nblocks*bj], size/nblocks,
 			size/nblocks, size/nblocks, sizeof(TYPE));
 	}
@@ -291,13 +336,25 @@ int STARPU_LU(lu_decomposition_pivot_no_stride)(TYPE **matA, unsigned *ipiv, uns
 
 	double timing;
 	int ret = dw_codelet_facto_pivot(dataAp, piv_description, nblocks, get_block_with_no_striding, &timing);
-
-	FPRINTF(stderr, "Computation took (in ms)\n");
-	FPRINTF(stderr, "%2.2f\n", timing/1000);
+	if (ret)
+		return ret;
 
 	unsigned n = starpu_matrix_get_nx(dataAp[0])*nblocks;
 	double flop = (2.0f*n*n*n)/3.0f;
-	FPRINTF(stderr, "Synthetic GFlops : %2.2f\n", (flop/timing/1000.0f));
+
+	PRINTF("# size\tms\tGFlops");
+	if (bound)
+		PRINTF("\tTms\tTGFlops");
+	PRINTF("\n");
+	PRINTF("%u\t%.0f\t%.1f", n, timing/1000, flop/timing/1000.0f);
+	if (bound)
+	{
+		double min;
+		starpu_bound_compute(&min, NULL, 0);
+		PRINTF("\t%.0f\t%.1f", min, flop/min/1000000.0f);
+	}
+	PRINTF("\n");
+
 
 	for (bj = 0; bj < nblocks; bj++)
 	for (bi = 0; bi < nblocks; bi++)
