@@ -1,7 +1,7 @@
 /* StarPU --- Runtime system for heterogeneous multicore architectures.
  *
- * Copyright (C) 2010, 2011  Université de Bordeaux 1
- * Copyright (C) 2010, 2011, 2012  Centre National de la Recherche Scientifique
+ * Copyright (C) 2010, 2011, 2014-2015, 2017  Université de Bordeaux
+ * Copyright (C) 2010, 2011, 2012, 2013, 2016  CNRS
  *
  * StarPU is free software; you can redistribute it and/or modify
  * it under the terms of the GNU Lesser General Public License as published by
@@ -15,11 +15,13 @@
  * See the GNU Lesser General Public License in COPYING.LGPL for more details.
  */
 
+/*
+ * This computes the Mandelbrot set: the output image is split in horizontal
+ * stripes, which are computed in parallel.  We also make the same computation
+ * several times, so that OpenGL interaction allows to browse through the set.
+ */
+
 #include <starpu.h>
-#ifdef STARPU_USE_OPENCL
-#include <starpu_opencl.h>
-#endif
-#include <sys/time.h>
 #include <math.h>
 #include <limits.h>
 #ifdef STARPU_HAVE_X11
@@ -28,6 +30,16 @@
 int use_x11 = 1;
 #endif
 
+#ifdef STARPU_HAVE_HELGRIND_H
+#include <valgrind/helgrind.h>
+#endif
+#ifndef ANNOTATE_HAPPENS_BEFORE
+#define ANNOTATE_HAPPENS_BEFORE(obj) ((void)0)
+#endif
+#ifndef ANNOTATE_HAPPENS_AFTER
+#define ANNOTATE_HAPPENS_AFTER(obj) ((void)0)
+#endif
+
 int demo = 0;
 static double demozoom = 0.05;
 
@@ -68,23 +80,23 @@ static void init_x11(int width, int height, unsigned *buffer)
 {
 	/* Attempt to open the display */
 	dpy = XOpenDisplay(NULL);
-	
+
 	/* Failure */
 	if (!dpy)
 		exit(0);
-	
+
 	unsigned long white = WhitePixel(dpy,DefaultScreen(dpy));
 	unsigned long black = BlackPixel(dpy,DefaultScreen(dpy));
 
 	win = XCreateSimpleWindow(dpy, DefaultRootWindow(dpy), 0, 0,
 					width, height, 0, black, white);
-	
+
 	/* We want to be notified when the window appears */
 	XSelectInput(dpy, win, StructureNotifyMask);
-	
+
 	/* Make it appear */
 	XMapWindow(dpy, win);
-	
+
 	XTextProperty tp;
 	char name[128] = "Mandelbrot - StarPU";
 	char *n = name;
@@ -94,7 +106,7 @@ static void init_x11(int width, int height, unsigned *buffer)
 
 	/* Wait for the MapNotify event */
 	XFlush(dpy);
-	
+
 	int depth = DefaultDepth(dpy, DefaultScreen(dpy));
 	Visual *visual = DefaultVisual(dpy, DefaultScreen(dpy));
 
@@ -102,13 +114,13 @@ static void init_x11(int width, int height, unsigned *buffer)
 	bitmap = XCreateImage(dpy, visual, depth,
 		ZPixmap, 0, (char *)buffer,
 		width, height, 32, 0);
-	
+
 	/* Init GC */
 	gc = XCreateGC(dpy, win, 0, NULL);
 	XSetForeground(dpy, gc, black);
-	
+
 	XSelectInput(dpy, win, ExposureMask | KeyPressMask | StructureNotifyMask);
-	
+
 	Atom wmDeleteMessage;
 	wmDeleteMessage = XInternAtom(dpy, "WM_DELETE_WINDOW", False);
 	XSetWMProtocols(dpy, win, &wmDeleteMessage, 1);
@@ -123,13 +135,13 @@ static void init_x11(int width, int height, unsigned *buffer)
 static int handle_events(void)
 {
 	XEvent event;
+
 	XNextEvent(dpy, &event);
-
-	KeySym key;
-	char text[255];
-
 	if (event.type == KeyPress)
 	{
+		KeySym key;
+		char text[255];
+
 		XLookupString(&event.xkey,text,255,&key,0);
 		if (key == Left)
 		{
@@ -244,12 +256,13 @@ static void compute_block_opencl(void *descr[], void *cl_arg)
 
 	cl_kernel kernel;
 	cl_command_queue queue;
-	cl_event event;
+	cl_int err;
 
-	int id = starpu_worker_get_id();
+	int id = starpu_worker_get_id_check();
 	int devid = starpu_worker_get_devid(id);
 
-	starpu_opencl_load_kernel(&kernel, &queue, &opencl_programs, "mandelbrot_kernel", devid);
+	err = starpu_opencl_load_kernel(&kernel, &queue, &opencl_programs, "mandelbrot_kernel", devid);
+	if (err != CL_SUCCESS) STARPU_OPENCL_REPORT_ERROR(err);
 
 	clSetKernelArg(kernel, 0, sizeof(data), &data);
 	clSetKernelArg(kernel, 1, sizeof(leftX), &leftX);
@@ -264,10 +277,8 @@ static void compute_block_opencl(void *descr[], void *cl_arg)
 	unsigned dim = 16;
 	size_t local[2] = {dim, 1};
 	size_t global[2] = {width, block_size};
-	clEnqueueNDRangeKernel(queue, kernel, 2, NULL, global, local, 0, NULL, &event);
-	clFinish(queue);
-	starpu_opencl_collect_stats(event);
-	clReleaseEvent(event);
+	err = clEnqueueNDRangeKernel(queue, kernel, 2, NULL, global, local, 0, NULL, NULL);
+	if (err != CL_SUCCESS) STARPU_OPENCL_REPORT_ERROR(err);
 	starpu_opencl_release_kernel(kernel);
 }
 #endif
@@ -278,11 +289,10 @@ static void compute_block_opencl(void *descr[], void *cl_arg)
 
 static void compute_block(void *descr[], void *cl_arg)
 {
-	int ix, iy;
-
 	int iby, block_size;
 	double stepX, stepY;
 	int *pcnt; /* unused for sequential tasks */
+
 	starpu_codelet_unpack_args(cl_arg, &iby, &block_size, &stepX, &stepY, &pcnt);
 
 	unsigned *data = (unsigned *)STARPU_VECTOR_GET_PTR(descr[0]);
@@ -290,6 +300,8 @@ static void compute_block(void *descr[], void *cl_arg)
 	int local_iy;
 	for (local_iy = 0; local_iy < block_size; local_iy++)
 	{
+		int ix, iy;
+
 		iy = iby*block_size + local_iy;
 		for (ix = 0; ix < width; ix++)
 		{
@@ -314,7 +326,7 @@ static void compute_block(void *descr[], void *cl_arg)
 				x = x2 - y2 + cx;
 				y = twoxy + cy;
 			}
-	
+
 			unsigned int v = STARPU_MIN((1024*((float)(it)/(2000))), 256);
 			data[ix + local_iy*width] = (v<<16|(255-v)<<8);
 		}
@@ -331,17 +343,21 @@ static void compute_block_spmd(void *descr[], void *cl_arg)
 
 	unsigned *data = (unsigned *)STARPU_VECTOR_GET_PTR(descr[0]);
 
-	int ix, iy; /* global coordinates */
-	int local_iy; /* current line */
-
 	while (1)
 	{
+		int ix, iy; /* global coordinates */
+		int local_iy; /* current line */
+
 		local_iy = STARPU_ATOMIC_ADD((unsigned int *)pcnt, 1) - 1;
+		ANNOTATE_HAPPENS_BEFORE(pcnt);
 		if (local_iy >= block_size)
+		{
+			ANNOTATE_HAPPENS_AFTER(pcnt);
 			break;
+		}
 
 		iy = iby*block_size + local_iy;
-	
+
 		for (ix = 0; ix < width; ix++)
 		{
 			double cx = leftX + ix * stepX;
@@ -365,7 +381,7 @@ static void compute_block_spmd(void *descr[], void *cl_arg)
 				x = x2 - y2 + cx;
 				y = twoxy + cy;
 			}
-	
+
 			unsigned int v = STARPU_MIN((1024*((float)(it)/(2000))), 256);
 			data[ix + local_iy*width] = (v<<16|(255-v)<<8);
 		}
@@ -376,23 +392,23 @@ static void compute_block_spmd(void *descr[], void *cl_arg)
 
 static struct starpu_codelet spmd_mandelbrot_cl =
 {
-	.where = STARPU_CPU|STARPU_OPENCL,
 	.type = STARPU_SPMD,
 	.max_parallelism = INT_MAX,
-	.cpu_funcs = {compute_block_spmd, NULL},
+	.cpu_funcs = {compute_block_spmd},
 #ifdef STARPU_USE_OPENCL
-	.opencl_funcs = {compute_block_opencl, NULL},
+	.opencl_funcs = {compute_block_opencl},
+	.opencl_flags = {STARPU_OPENCL_ASYNC},
 #endif
 	.nbuffers = 1
 };
 
 static struct starpu_codelet mandelbrot_cl =
 {
-	.where = STARPU_CPU|STARPU_OPENCL,
 	.type = STARPU_SEQ,
-	.cpu_funcs = {compute_block, NULL},
+	.cpu_funcs = {compute_block},
 #ifdef STARPU_USE_OPENCL
-	.opencl_funcs = {compute_block_opencl, NULL},
+	.opencl_funcs = {compute_block_opencl},
+	.opencl_flags = {STARPU_OPENCL_ASYNC},
 #endif
 	.nbuffers = 1
 };
@@ -480,7 +496,7 @@ int main(int argc, char **argv)
 	conf.ncuda = 0;
 
 	if (use_spmd)
-		conf.sched_policy_name = "pgreedy";
+		conf.sched_policy_name = "peager";
 
 	ret = starpu_init(&conf);
 	STARPU_CHECK_RETURN_VALUE(ret, "starpu_init");
@@ -501,25 +517,23 @@ int main(int argc, char **argv)
 #endif
 
 	starpu_data_handle_t block_handles[nblocks];
-	
+
 	int iby;
 	for (iby = 0; iby < nblocks; iby++)
 	{
 		unsigned *data = &buffer[iby*block_size*width];
-		starpu_vector_data_register(&block_handles[iby], 0,
+		starpu_vector_data_register(&block_handles[iby], STARPU_MAIN_RAM,
                         (uintptr_t)data, block_size*width, sizeof(unsigned));
 	}
 
 	unsigned iter = 0;
 
-	struct timeval start, end;
+	double start, end;
 
-	if (demo)
-		gettimeofday(&start, NULL);
+	start = starpu_timing_now();
 
 	while (niter-- != 0)
 	{
-		int ret;
 		double stepX = (rightX - leftX)/width;
 		double stepY = (topY - bottomY)/height;
 
@@ -529,20 +543,23 @@ int main(int argc, char **argv)
 		 * parallel task. */
 		int per_block_cnt[nblocks];
 
+		starpu_iteration_push(niter);
+
 		for (iby = 0; iby < nblocks; iby++)
 		{
 			per_block_cnt[iby] = 0;
 			int *pcnt = &per_block_cnt[iby];
 
-			ret = starpu_insert_task(use_spmd?&spmd_mandelbrot_cl:&mandelbrot_cl,
+			ret = starpu_task_insert(use_spmd?&spmd_mandelbrot_cl:&mandelbrot_cl,
 						 STARPU_VALUE, &iby, sizeof(iby),
 						 STARPU_VALUE, &block_size, sizeof(block_size),
 						 STARPU_VALUE, &stepX, sizeof(stepX),
 						 STARPU_VALUE, &stepY, sizeof(stepY),
 						 STARPU_W, block_handles[iby],
 						 STARPU_VALUE, &pcnt, sizeof(int *),
+						 STARPU_TAG_ONLY, ((starpu_tag_t)niter)*nblocks + iby,
 						 0);
-			STARPU_CHECK_RETURN_VALUE(ret, "starpu_insert_task");
+			STARPU_CHECK_RETURN_VALUE(ret, "starpu_task_insert");
 		}
 
 		for (iby = 0; iby < nblocks; iby++)
@@ -561,6 +578,7 @@ int main(int argc, char **argv)
 		}
 
 
+		starpu_iteration_pop();
 		if (demo)
 		{
 			/* Zoom in */
@@ -578,15 +596,15 @@ int main(int argc, char **argv)
 				topY = -49.35016705749115;
 				bottomY = 49.64891691946615;
 
-				gettimeofday(&end, NULL);
-				double timing = (double)((end.tv_sec - start.tv_sec)*1000000 + (end.tv_usec - start.tv_usec));
+				end = starpu_timing_now();
+				double timing = end - start;
 
 				fprintf(stderr, "Time to generate %u frames : %f s\n", iter, timing/1000000.0);
 				fprintf(stderr, "Average FPS: %f\n", ((double)iter*1e+6)/timing);
 
 				/* Reset counters */
 				iter = 0;
-				gettimeofday(&start, NULL);
+				start = starpu_timing_now();
 			}
 			else
 			{
@@ -595,7 +613,7 @@ int main(int argc, char **argv)
 				topY -= (zoom_factor/2)*heightY;
 				bottomY += (zoom_factor/2)*heightY;
 			}
-	
+
 		}
 #ifdef STARPU_HAVE_X11
 		else if (use_x11 && handle_events())
