@@ -1,8 +1,8 @@
 /* StarPU --- Runtime system for heterogeneous multicore architectures.
  *
- * Copyright (C) 2009-2012  Université de Bordeaux 1
+ * Copyright (C) 2009-2016  Université de Bordeaux
  * Copyright (C) 2010  Mehdi Juhoor <mjuhoor@gmail.com>
- * Copyright (C) 2010, 2011, 2012  Centre National de la Recherche Scientifique
+ * Copyright (C) 2010, 2011, 2012, 2013, 2016  CNRS
  *
  * StarPU is free software; you can redistribute it and/or modify
  * it under the terms of the GNU Lesser General Public License as published by
@@ -16,33 +16,52 @@
  * See the GNU Lesser General Public License in COPYING.LGPL for more details.
  */
 
+/*
+ * Simple parallel GEMM implementation: partition the output matrix in the two
+ * dimensions, and the input matrices in the corresponding dimension, and
+ * perform the output computations in parallel.
+ */
+#ifndef TYPE
+#error "Do not compile xgemm.c directly, compile sgemm.c or dgemm.c"
+#endif
+
 #include <limits.h>
 #include <string.h>
 #include <math.h>
 #include <sys/types.h>
-#include <sys/time.h>
 #include <starpu.h>
+#include <starpu_fxt.h>
 
 #include <common/blas.h>
 
 #ifdef STARPU_USE_CUDA
 #include <cuda.h>
-#include <cublas.h>
-#include <starpu_cuda.h>
+#include <starpu_cublas_v2.h>
+static const TYPE p1 = 1.0;
+static const TYPE m1 = -1.0;
+static const TYPE v0 = 0.0;
 #endif
 
-static unsigned niter = 100;
+static unsigned niter = 10;
 static unsigned nslicesx = 4;
 static unsigned nslicesy = 4;
+#if defined(STARPU_QUICK_CHECK) && !defined(STARPU_SIMGRID)
 static unsigned xdim = 256;
 static unsigned ydim = 256;
 static unsigned zdim = 64;
+#else
+static unsigned xdim = 960*4;
+static unsigned ydim = 960*4;
+static unsigned zdim = 960*4;
+#endif
 static unsigned check = 0;
+static unsigned bound = 0;
 
 static TYPE *A, *B, *C;
 static starpu_data_handle_t A_handle, B_handle, C_handle;
 
-#define FPRINTF(ofile, fmt, args ...) do { if (!getenv("STARPU_SSILENT")) {fprintf(ofile, fmt, ##args); }} while(0)
+#define FPRINTF(ofile, fmt, ...) do { if (!getenv("STARPU_SSILENT")) {fprintf(ofile, fmt, ## __VA_ARGS__); }} while(0)
+#define PRINTF(fmt, ...) do { if (!getenv("STARPU_SSILENT")) {printf(fmt, ## __VA_ARGS__); }} while(0)
 
 static void check_output(void)
 {
@@ -69,12 +88,15 @@ static void check_output(void)
 
 static void init_problem_data(void)
 {
+#ifndef STARPU_SIMGRID
 	unsigned i,j;
+#endif
 
-	starpu_malloc((void **)&A, zdim*ydim*sizeof(TYPE));
-	starpu_malloc((void **)&B, xdim*zdim*sizeof(TYPE));
-	starpu_malloc((void **)&C, xdim*ydim*sizeof(TYPE));
+	starpu_malloc_flags((void **)&A, zdim*ydim*sizeof(TYPE), STARPU_MALLOC_PINNED|STARPU_MALLOC_SIMULATION_FOLDED);
+	starpu_malloc_flags((void **)&B, xdim*zdim*sizeof(TYPE), STARPU_MALLOC_PINNED|STARPU_MALLOC_SIMULATION_FOLDED);
+	starpu_malloc_flags((void **)&C, xdim*ydim*sizeof(TYPE), STARPU_MALLOC_PINNED|STARPU_MALLOC_SIMULATION_FOLDED);
 
+#ifndef STARPU_SIMGRID
 	/* fill the A and B matrices */
 	for (j=0; j < ydim; j++)
 	{
@@ -99,25 +121,26 @@ static void init_problem_data(void)
 			C[j+i*ydim] = (TYPE)(0);
 		}
 	}
+#endif
 }
 
 static void partition_mult_data(void)
 {
-	starpu_matrix_data_register(&A_handle, 0, (uintptr_t)A,
+	starpu_matrix_data_register(&A_handle, STARPU_MAIN_RAM, (uintptr_t)A,
 		ydim, ydim, zdim, sizeof(TYPE));
-	starpu_matrix_data_register(&B_handle, 0, (uintptr_t)B,
+	starpu_matrix_data_register(&B_handle, STARPU_MAIN_RAM, (uintptr_t)B,
 		zdim, zdim, xdim, sizeof(TYPE));
-	starpu_matrix_data_register(&C_handle, 0, (uintptr_t)C,
+	starpu_matrix_data_register(&C_handle, STARPU_MAIN_RAM, (uintptr_t)C,
 		ydim, ydim, xdim, sizeof(TYPE));
 
 	struct starpu_data_filter vert;
 	memset(&vert, 0, sizeof(vert));
-	vert.filter_func = starpu_vertical_block_filter_func;
+	vert.filter_func = starpu_matrix_filter_vertical_block;
 	vert.nchildren = nslicesx;
 
 	struct starpu_data_filter horiz;
 	memset(&horiz, 0, sizeof(horiz));
-	horiz.filter_func = starpu_block_filter_func;
+	horiz.filter_func = starpu_matrix_filter_block;
 	horiz.nchildren = nslicesy;
 
 	starpu_data_partition(B_handle, &vert);
@@ -126,7 +149,8 @@ static void partition_mult_data(void)
 	starpu_data_map_filters(C_handle, 2, &vert, &horiz);
 }
 
-static void mult_kernel_common(void *descr[], int type)
+#ifdef STARPU_USE_CUDA
+static void cublas_mult(void *descr[], STARPU_ATTRIBUTE_UNUSED void *arg)
 {
 	TYPE *subA = (TYPE *)STARPU_MATRIX_GET_PTR(descr[0]);
 	TYPE *subB = (TYPE *)STARPU_MATRIX_GET_PTR(descr[1]);
@@ -140,51 +164,52 @@ static void mult_kernel_common(void *descr[], int type)
 	unsigned ldB = STARPU_MATRIX_GET_LD(descr[1]);
 	unsigned ldC = STARPU_MATRIX_GET_LD(descr[2]);
 
-	if (type == STARPU_CPU)
+	cublasStatus_t status = CUBLAS_GEMM(starpu_cublas_get_local_handle(),
+			CUBLAS_OP_N, CUBLAS_OP_N,
+			nxC, nyC, nyA,
+			&p1, subA, ldA, subB, ldB,
+			&v0, subC, ldC);
+	if (status != CUBLAS_STATUS_SUCCESS)
+		STARPU_CUBLAS_REPORT_ERROR(status);
+}
+#endif
+
+void cpu_mult(void *descr[], STARPU_ATTRIBUTE_UNUSED  void *arg)
+{
+	TYPE *subA = (TYPE *)STARPU_MATRIX_GET_PTR(descr[0]);
+	TYPE *subB = (TYPE *)STARPU_MATRIX_GET_PTR(descr[1]);
+	TYPE *subC = (TYPE *)STARPU_MATRIX_GET_PTR(descr[2]);
+
+	unsigned nxC = STARPU_MATRIX_GET_NX(descr[2]);
+	unsigned nyC = STARPU_MATRIX_GET_NY(descr[2]);
+	unsigned nyA = STARPU_MATRIX_GET_NY(descr[0]);
+
+	unsigned ldA = STARPU_MATRIX_GET_LD(descr[0]);
+	unsigned ldB = STARPU_MATRIX_GET_LD(descr[1]);
+	unsigned ldC = STARPU_MATRIX_GET_LD(descr[2]);
+
+	int worker_size = starpu_combined_worker_get_size();
+
+	if (worker_size == 1)
 	{
-		int worker_size = starpu_combined_worker_get_size();
-
-		if (worker_size == 1)
-		{
-			/* Sequential CPU task */
-			CPU_GEMM("N", "N", nxC, nyC, nyA, (TYPE)1.0, subA, ldA, subB, ldB, (TYPE)0.0, subC, ldC);
-		}
-		else
-		{
-			/* Parallel CPU task */
-			int rank = starpu_combined_worker_get_rank();
-
-			int block_size = (nyC + worker_size - 1)/worker_size;
-			int new_nyC = STARPU_MIN(nyC, block_size*(rank+1)) - block_size*rank;
-
-			STARPU_ASSERT(nyC = STARPU_MATRIX_GET_NY(descr[1]));
-
-			TYPE *new_subB = &subB[block_size*rank];
-			TYPE *new_subC = &subC[block_size*rank];
-
-			CPU_GEMM("N", "N", nxC, new_nyC, nyA, (TYPE)1.0, subA, ldA, new_subB, ldB, (TYPE)0.0, new_subC, ldC);
-		}
+		/* Sequential CPU task */
+		CPU_GEMM("N", "N", nxC, nyC, nyA, (TYPE)1.0, subA, ldA, subB, ldB, (TYPE)0.0, subC, ldC);
 	}
-#ifdef STARPU_USE_CUDA
 	else
 	{
-		CUBLAS_GEMM('n', 'n', nxC, nyC, nyA, (TYPE)1.0, subA, ldA, subB, ldB,
-					     (TYPE)0.0, subC, ldC);
-		cudaStreamSynchronize(starpu_cuda_get_local_stream());
+		/* Parallel CPU task */
+		unsigned rank = starpu_combined_worker_get_rank();
+
+		unsigned block_size = (nyC + worker_size - 1)/worker_size;
+		unsigned new_nyC = STARPU_MIN(nyC, block_size*(rank+1)) - block_size*rank;
+
+		STARPU_ASSERT(nyC == STARPU_MATRIX_GET_NY(descr[1]));
+
+		TYPE *new_subB = &subB[block_size*rank];
+		TYPE *new_subC = &subC[block_size*rank];
+
+		CPU_GEMM("N", "N", nxC, new_nyC, nyA, (TYPE)1.0, subA, ldA, new_subB, ldB, (TYPE)0.0, new_subC, ldC);
 	}
-#endif
-}
-
-#ifdef STARPU_USE_CUDA
-static void cublas_mult(void *descr[], __attribute__((unused)) void *arg)
-{
-	mult_kernel_common(descr, STARPU_CUDA);
-}
-#endif
-
-static void cpu_mult(void *descr[], __attribute__((unused))  void *arg)
-{
-	mult_kernel_common(descr, STARPU_CPU);
 }
 
 static struct starpu_perfmodel starpu_gemm_model =
@@ -195,13 +220,16 @@ static struct starpu_perfmodel starpu_gemm_model =
 
 static struct starpu_codelet cl =
 {
-	.where = STARPU_CPU|STARPU_CUDA,
 	.type = STARPU_SEQ, /* changed to STARPU_SPMD if -spmd is passed */
 	.max_parallelism = INT_MAX,
-	.cpu_funcs = {cpu_mult, NULL},
+	.cpu_funcs = {cpu_mult},
+	.cpu_funcs_name = {"cpu_mult"},
 #ifdef STARPU_USE_CUDA
-	.cuda_funcs = {cublas_mult, NULL},
+	.cuda_funcs = {cublas_mult},
+#elif defined(STARPU_SIMGRID)
+	.cuda_funcs = {(void*)1},
 #endif
+	.cuda_flags = {STARPU_CUDA_ASYNC},
 	.nbuffers = 3,
 	.modes = {STARPU_R, STARPU_R, STARPU_RW},
 	.model = &starpu_gemm_model
@@ -219,58 +247,80 @@ static void parse_args(int argc, char **argv)
 			nslicesy = nslicesx;
 		}
 
-		if (strcmp(argv[i], "-nblocksx") == 0)
+		else if (strcmp(argv[i], "-nblocksx") == 0)
 		{
 			char *argptr;
 			nslicesx = strtol(argv[++i], &argptr, 10);
 		}
 
-		if (strcmp(argv[i], "-nblocksy") == 0)
+		else if (strcmp(argv[i], "-nblocksy") == 0)
 		{
 			char *argptr;
 			nslicesy = strtol(argv[++i], &argptr, 10);
 		}
 
-		if (strcmp(argv[i], "-x") == 0)
+		else if (strcmp(argv[i], "-x") == 0)
 		{
 			char *argptr;
 			xdim = strtol(argv[++i], &argptr, 10);
 		}
 
-		if (strcmp(argv[i], "-y") == 0)
+		else if (strcmp(argv[i], "-y") == 0)
 		{
 			char *argptr;
 			ydim = strtol(argv[++i], &argptr, 10);
 		}
 
-		if (strcmp(argv[i], "-z") == 0)
+		else if (strcmp(argv[i], "-z") == 0)
 		{
 			char *argptr;
 			zdim = strtol(argv[++i], &argptr, 10);
 		}
 
-		if (strcmp(argv[i], "-iter") == 0)
+		else if (strcmp(argv[i], "-size") == 0)
+		{
+			char *argptr;
+			xdim = ydim = zdim = strtol(argv[++i], &argptr, 10);
+		}
+
+		else if (strcmp(argv[i], "-iter") == 0)
 		{
 			char *argptr;
 			niter = strtol(argv[++i], &argptr, 10);
 		}
 
-		if (strcmp(argv[i], "-check") == 0)
+		else if (strcmp(argv[i], "-bound") == 0)
+		{
+			bound = 1;
+		}
+
+		else if (strcmp(argv[i], "-check") == 0)
 		{
 			check = 1;
 		}
 
-		if (strcmp(argv[i], "-spmd") == 0)
+		else if (strcmp(argv[i], "-spmd") == 0)
 		{
 			cl.type = STARPU_SPMD;
 		}
+
+		else if (strcmp(argv[i], "-help") == 0 || strcmp(argv[i], "--help") == 0 || strcmp(argv[i], "-h") == 0)
+		{
+			fprintf(stderr,"Usage: %s [-nblocks n] [-nblocksx x] [-nblocksy y] [-x x] [-y y] [-z z] [-size size] [-iter iter] [-bound] [-check] [-spmd]\n", argv[0]);
+			fprintf(stderr,"Currently selected: %ux%u * %ux%u and %ux%u blocks, %u iterations\n", zdim, ydim, xdim, zdim, nslicesx, nslicesy, niter);
+			exit(EXIT_SUCCESS);
+		}
+		else
+		{
+			fprintf(stderr,"Unrecognized option %s", argv[i]);
+			exit(EXIT_FAILURE);
+		}
 	}
 }
 
 int main(int argc, char **argv)
 {
-	struct timeval start;
-	struct timeval end;
+	double start, end;
 	int ret;
 
 	parse_args(argc, argv);
@@ -279,17 +329,22 @@ int main(int argc, char **argv)
 	niter /= 10;
 #endif
 
+	starpu_fxt_autostart_profiling(0);
 	ret = starpu_init(NULL);
 	if (ret == -ENODEV)
 		return 77;
 	STARPU_CHECK_RETURN_VALUE(ret, "starpu_init");
 
-	starpu_helper_cublas_init();
+	starpu_cublas_init();
 
 	init_problem_data();
 	partition_mult_data();
 
-	gettimeofday(&start, NULL);
+	if (bound)
+		starpu_bound_start(0, 0);
+
+	starpu_fxt_start_profiling();
+	start = starpu_timing_now();
 
 	unsigned x, y, iter;
 	for (iter = 0; iter < niter; iter++)
@@ -305,32 +360,49 @@ int main(int argc, char **argv)
 			task->handles[1] = starpu_data_get_sub_data(B_handle, 1, x);
 			task->handles[2] = starpu_data_get_sub_data(C_handle, 2, x, y);
 
-			int ret = starpu_task_submit(task);
+			task->flops = 2ULL * (xdim/nslicesx) * (ydim/nslicesy) * zdim;
+
+			ret = starpu_task_submit(task);
 			if (ret == -ENODEV)
 			{
 			     ret = 77;
 			     goto enodev;
 			}
 			STARPU_CHECK_RETURN_VALUE(ret, "starpu_task_submit");
+			starpu_data_wont_use(starpu_data_get_sub_data(C_handle, 2, x, y));
 		}
 
 		starpu_task_wait_for_all();
 	}
 
 
-	gettimeofday(&end, NULL);
-	double timing = (double)((end.tv_sec - start.tv_sec)*1000000 + (end.tv_usec - start.tv_usec));
+	end = starpu_timing_now();
+	starpu_fxt_stop_profiling();
 
-	FPRINTF(stderr, "Time: %2.2f ms\n", timing/1000.0);
+	if (bound)
+		starpu_bound_stop();
 
-	double flops = 2.0*((unsigned long)niter)*((unsigned long)xdim)
-				*((unsigned long)ydim)*((unsigned long)zdim);
-	FPRINTF(stderr, "GFlop/s: %.2f\n", flops/timing/1000.0);
+	double timing = end - start;
+	double min, min_int;
+	double flops = 2.0*((unsigned long long)niter)*((unsigned long long)xdim)
+		           *((unsigned long long)ydim)*((unsigned long long)zdim);
+
+	if (bound)
+		starpu_bound_compute(&min, &min_int, 1);
+
+	PRINTF("# x\ty\tz\tms\tGFlops");
+	if (bound)
+		PRINTF("\tTms\tTGFlops\tTims\tTiGFlops");
+	PRINTF("\n");
+	PRINTF("%u\t%u\t%u\t%.0f\t%.1f", xdim, ydim, zdim, timing/niter/1000.0, flops/timing/1000.0);
+	if (bound)
+		PRINTF("\t%.0f\t%.1f\t%.0f\t%.1f", min, flops/min/1000000.0, min_int, flops/min_int/1000000.0);
+	PRINTF("\n");
 
 enodev:
-	starpu_data_unpartition(C_handle, 0);
-	starpu_data_unpartition(B_handle, 0);
-	starpu_data_unpartition(A_handle, 0);
+	starpu_data_unpartition(C_handle, STARPU_MAIN_RAM);
+	starpu_data_unpartition(B_handle, STARPU_MAIN_RAM);
+	starpu_data_unpartition(A_handle, STARPU_MAIN_RAM);
 
 	starpu_data_unregister(A_handle);
 	starpu_data_unregister(B_handle);
@@ -339,11 +411,11 @@ enodev:
 	if (check)
 		check_output();
 
-	starpu_free(A);
-	starpu_free(B);
-	starpu_free(C);
+	starpu_free_flags(A, zdim*ydim*sizeof(TYPE), STARPU_MALLOC_PINNED|STARPU_MALLOC_SIMULATION_FOLDED);
+	starpu_free_flags(B, xdim*zdim*sizeof(TYPE), STARPU_MALLOC_PINNED|STARPU_MALLOC_SIMULATION_FOLDED);
+	starpu_free_flags(C, xdim*ydim*sizeof(TYPE), STARPU_MALLOC_PINNED|STARPU_MALLOC_SIMULATION_FOLDED);
 
-	starpu_helper_cublas_shutdown();
+	starpu_cublas_shutdown();
 	starpu_shutdown();
 
 	return ret;
