@@ -1,7 +1,7 @@
 /* StarPU --- Runtime system for heterogeneous multicore architectures.
  *
- * Copyright (C) 2009, 2010  Université de Bordeaux 1
- * Copyright (C) 2010  Centre National de la Recherche Scientifique
+ * Copyright (C) 2009, 2010  Université de Bordeaux
+ * Copyright (C) 2010, 2012, 2013  CNRS
  *
  * StarPU is free software; you can redistribute it and/or modify
  * it under the terms of the GNU Lesser General Public License as published by
@@ -22,21 +22,11 @@
 #include <string.h>
 #include <stdint.h>
 #include <math.h>
-#include <sys/time.h>
-#include <pthread.h>
 #include <signal.h>
 
 #include <starpu.h>
 
-#ifdef STARPU_USE_CUDA
-#include <starpu_cuda.h>
-#endif
-
-#ifdef STARPU_USE_OPENCL
-#include <starpu_opencl.h>
-#endif
-
-#define FPRINTF(ofile, fmt, args ...) do { if (!getenv("STARPU_SSILENT")) {fprintf(ofile, fmt, ##args); }} while(0)
+#define FPRINTF(ofile, fmt, ...) do { if (!getenv("STARPU_SSILENT")) {fprintf(ofile, fmt, ## __VA_ARGS__); }} while(0)
 
 #ifdef STARPU_USE_CUDA
 void spmv_kernel_cuda(void *descr[], void *args);
