@@ -1,6 +1,7 @@
 /* StarPU --- Runtime system for heterogeneous multicore architectures.
  *
- * Copyright (C) 2010-2011  Université de Bordeaux 1
+ * Copyright (C) 2010-2011, 2013-2014  Université de Bordeaux
+ * Copyright (C) 2016  CNRS
  *
  * StarPU is free software; you can redistribute it and/or modify
  * it under the terms of the GNU Lesser General Public License as published by
@@ -15,9 +16,8 @@
  */
 
 #include "stencil.h"
-#include <starpu_opencl.h>
 
-/* Perform replication of data on X and Y edges, to fold the domain on 
+/* Perform replication of data on X and Y edges, to fold the domain on
    itself through mere replication of the source state. */
 
 #define str(x) #x
@@ -76,35 +76,29 @@ opencl_shadow_init(void)
 
 void opencl_shadow_free(void)
 {
-  starpu_opencl_unload_opencl(&program);
+  int ret = starpu_opencl_unload_opencl(&program);
+  STARPU_CHECK_RETURN_VALUE(ret, "starpu_opencl_unload_opencl");
 }
 
 void
 opencl_shadow_host(int bz, TYPE *ptr, int nx, int ny, int nz, int ldy, int ldz, int i)
 {
-	unsigned max_parallelism = 512;
-	unsigned threads_per_dim_x = max_parallelism;
-	while (threads_per_dim_x / 2 >= nx)
-		threads_per_dim_x /= 2;
-	unsigned threads_per_dim_y = max_parallelism / threads_per_dim_x;
-	while (threads_per_dim_y / 2 >= ny)
-		threads_per_dim_y /= 2;
 #if 0
-	unsigned threads_per_dim_z = 4;
-	size_t dimBlock[] = {threads_per_dim_x, threads_per_dim_y, threads_per_dim_z};
-	size_t dimGrid[] = {nx / threads_per_dim_x, ny / threads_per_dim_y, nz / threads_per_dim_z};
+	size_t dim[] = {nx, ny, nz};
 #else
-	size_t dimBlock[] = {threads_per_dim_x, threads_per_dim_y, 1};
-	size_t dimGrid[] = {((nx + threads_per_dim_x-1) / threads_per_dim_x)*threads_per_dim_x, ((ny + threads_per_dim_y-1) / threads_per_dim_y)*threads_per_dim_y, 1};
+	size_t dim[] = {nx, ny, 1};
 #endif
 
         int devid,id;
-        id = starpu_worker_get_id();
+        id = starpu_worker_get_id_check();
         devid = starpu_worker_get_devid(id);
 
         cl_kernel kernel;
         cl_command_queue cq;
-        starpu_opencl_load_kernel(&kernel, &cq, &program, "shadow", devid);
+	cl_int err;
+
+        err = starpu_opencl_load_kernel(&kernel, &cq, &program, "shadow", devid);
+	if (err != CL_SUCCESS) STARPU_OPENCL_REPORT_ERROR(err);
 
         clSetKernelArg(kernel, 0, sizeof(bz), &bz);
         clSetKernelArg(kernel, 1, sizeof(ptr), &ptr);
@@ -115,12 +109,6 @@ opencl_shadow_host(int bz, TYPE *ptr, int nx, int ny, int nz, int ldy, int ldz,
         clSetKernelArg(kernel, 6, sizeof(ldz), &ldz);
         clSetKernelArg(kernel, 7, sizeof(i), &i);
 
-        cl_event ev;
-        cl_int err = clEnqueueNDRangeKernel(cq, kernel, 3, NULL, dimGrid, dimBlock, 0, NULL, &ev);
-        if (err != CL_SUCCESS)
-                STARPU_OPENCL_REPORT_ERROR(err);
-        clWaitForEvents(1, &ev);
-        starpu_opencl_collect_stats(ev);
-        clReleaseEvent(ev);
+        err = clEnqueueNDRangeKernel(cq, kernel, 3, NULL, dim, NULL, 0, NULL, NULL);
+        if (err != CL_SUCCESS) STARPU_OPENCL_REPORT_ERROR(err);
 }
-
