@@ -1,6 +1,6 @@
 /* StarPU --- Runtime system for heterogeneous multicore architectures.
  *
- * Copyright (C) 2010  Université de Bordeaux 1
+ * Copyright (C) 2010, 2013-2014, 2016-2017  Université de Bordeaux
  *
  * StarPU is free software; you can redistribute it and/or modify
  * it under the terms of the GNU Lesser General Public License as published by
@@ -121,7 +121,7 @@ struct block_description *get_block_description(int z)
 	return &blocks[z];
 }
 
-unsigned get_block_mpi_node(int z)
+int get_block_mpi_node(int z)
 {
 	z = (z + nbz)%nbz;
 	return blocks[z].mpi_node;
@@ -157,6 +157,12 @@ void create_blocks_array(unsigned _sizex, unsigned _sizey, unsigned _sizez, unsi
 	}
 }
 
+void free_blocks_array()
+{
+	free(blocks);
+	free(block_sizes_z);
+}
+
 /*
  *	Initialization of the blocks
  */
@@ -242,14 +248,14 @@ void assign_blocks_to_mpi_nodes(int world_size)
 
 static size_t allocated = 0;
 
-static void allocate_block_on_node(starpu_data_handle_t *handleptr, TYPE **ptr, unsigned nx, unsigned ny, unsigned nz)
+static void allocate_block_on_node(starpu_data_handle_t *handleptr, unsigned bz, TYPE **ptr, unsigned nx, unsigned ny, unsigned nz)
 {
 	int ret;
 	size_t block_size = nx*ny*nz*sizeof(TYPE);
 
 	/* Allocate memory */
 #if 1
-	ret = starpu_malloc((void **)ptr, block_size);
+	ret = starpu_malloc_flags((void **)ptr, block_size, STARPU_MALLOC_PINNED|STARPU_MALLOC_SIMULATION_FOLDED);
 	STARPU_ASSERT(ret == 0);
 #else
 	*ptr = malloc(block_size);
@@ -258,16 +264,28 @@ static void allocate_block_on_node(starpu_data_handle_t *handleptr, TYPE **ptr,
 
 	allocated += block_size;
 
+#ifndef STARPU_SIMGRID
 	/* Fill the blocks with 0 */
 	memset(*ptr, 0, block_size);
+#endif
 
 	/* Register it to StarPU */
-	starpu_block_data_register(handleptr, 0, (uintptr_t)*ptr, nx, nx*ny, nx, ny, nz, sizeof(TYPE));
+	starpu_block_data_register(handleptr, STARPU_MAIN_RAM, (uintptr_t)*ptr, nx, nx*ny, nx, ny, nz, sizeof(TYPE));
+
+	starpu_data_set_coordinates(*handleptr, 1, bz);
+}
+
+static void free_block_on_node(starpu_data_handle_t handleptr, unsigned nx, unsigned ny, unsigned nz)
+{
+	void *ptr = (void *) starpu_block_get_local_ptr(handleptr);
+	size_t block_size = nx*ny*nz*sizeof(TYPE);
+	starpu_data_unregister(handleptr);
+	starpu_free_flags(ptr, block_size, STARPU_MALLOC_PINNED|STARPU_MALLOC_SIMULATION_FOLDED);
 }
 
 void display_memory_consumption(int rank)
 {
-	fprintf(stderr, "%lu MB of memory were allocated on node %d\n", allocated/(1024*1024), rank);
+	FPRINTF(stderr, "%lu B of memory were allocated on node %d\n", (unsigned long) allocated, rank);
 }
 
 void allocate_memory_on_node(int rank)
@@ -277,15 +295,15 @@ void allocate_memory_on_node(int rank)
 	{
 		struct block_description *block = get_block_description(bz);
 
-		unsigned node = block->mpi_node;
+		int node = block->mpi_node;
 
-		unsigned size_bz = block_sizes_z[bz];
-	
 		/* Main blocks */
 		if (node == rank)
 		{
-			allocate_block_on_node(&block->layers_handle[0], &block->layers[0],
+			unsigned size_bz = block_sizes_z[bz];
+			allocate_block_on_node(&block->layers_handle[0], bz, &block->layers[0],
 						(sizex + 2*K), (sizey + 2*K), (size_bz + 2*K));
+#ifndef STARPU_SIMGRID
 #ifdef LIFE
 			unsigned x, y, z;
 			unsigned sum = 0;
@@ -296,29 +314,64 @@ void allocate_memory_on_node(int rank)
 						sum += block->layers[0][(K+x)+(K+y)*(sizex + 2*K)+(K+z)*(sizex+2*K)*(sizey+2*K)] = (int)((x/7.+y/13.+(bz*size_bz + z)/17.) * 10.) % 2;
 /*			printf("block %d starts with %d/%d alive\n", bz, sum, sizex*sizey*size_bz);*/
 #endif
-			allocate_block_on_node(&block->layers_handle[1], &block->layers[1],
+#endif
+			allocate_block_on_node(&block->layers_handle[1], bz, &block->layers[1],
 						(sizex + 2*K), (sizey + 2*K), (size_bz + 2*K));
 		}
 
 		/* Boundary blocks : Top */
-		unsigned top_node = block->boundary_blocks[T]->mpi_node;
+		int top_node = block->boundary_blocks[T]->mpi_node;
 		if ((node == rank) || (top_node == rank))
 		{
-			allocate_block_on_node(&block->boundaries_handle[T][0], &block->boundaries[T][0],
+			allocate_block_on_node(&block->boundaries_handle[T][0], bz, &block->boundaries[T][0],
 						(sizex + 2*K), (sizey + 2*K), K);
-			allocate_block_on_node(&block->boundaries_handle[T][1], &block->boundaries[T][1],
+			allocate_block_on_node(&block->boundaries_handle[T][1], bz, &block->boundaries[T][1],
 						(sizex + 2*K), (sizey + 2*K), K);
-		} 
+		}
 
 		/* Boundary blocks : Bottom */
-		unsigned bottom_node = block->boundary_blocks[B]->mpi_node;
+		int bottom_node = block->boundary_blocks[B]->mpi_node;
 		if ((node == rank) || (bottom_node == rank))
 		{
-			allocate_block_on_node(&block->boundaries_handle[B][0], &block->boundaries[B][0],
+			allocate_block_on_node(&block->boundaries_handle[B][0], bz, &block->boundaries[B][0],
 						(sizex + 2*K), (sizey + 2*K), K);
-			allocate_block_on_node(&block->boundaries_handle[B][1], &block->boundaries[B][1],
+			allocate_block_on_node(&block->boundaries_handle[B][1], bz, &block->boundaries[B][1],
 						(sizex + 2*K), (sizey + 2*K), K);
-		} 
+		}
+	}
+}
+
+void free_memory_on_node(int rank)
+{
+	unsigned bz;
+	for (bz = 0; bz < nbz; bz++)
+	{
+		struct block_description *block = get_block_description(bz);
+
+		int node = block->mpi_node;
+
+		/* Main blocks */
+		if (node == rank)
+		{
+			free_block_on_node(block->layers_handle[0], (sizex + 2*K), (sizey + 2*K), K);
+			free_block_on_node(block->layers_handle[1], (sizex + 2*K), (sizey + 2*K), K);
+		}
+
+		/* Boundary blocks : Top */
+		int top_node = block->boundary_blocks[T]->mpi_node;
+		if ((node == rank) || (top_node == rank))
+		{
+			free_block_on_node(block->boundaries_handle[T][0], (sizex + 2*K), (sizey + 2*K), K);
+			free_block_on_node(block->boundaries_handle[T][1], (sizex + 2*K), (sizey + 2*K), K);
+		}
+
+		/* Boundary blocks : Bottom */
+		int bottom_node = block->boundary_blocks[B]->mpi_node;
+		if ((node == rank) || (bottom_node == rank))
+		{
+			free_block_on_node(block->boundaries_handle[B][0], (sizex + 2*K), (sizey + 2*K), K);
+			free_block_on_node(block->boundaries_handle[B][1], (sizex + 2*K), (sizey + 2*K), K);
+		}
 	}
 }
 
@@ -330,20 +383,20 @@ void check(int rank)
 	{
 		struct block_description *block = get_block_description(bz);
 
-		unsigned node = block->mpi_node;
+		int node = block->mpi_node;
 
 		/* Main blocks */
 		if (node == rank)
 		{
-			unsigned size_bz = block_sizes_z[bz];
 #ifdef LIFE
+			unsigned size_bz = block_sizes_z[bz];
 			unsigned x, y, z;
 			unsigned sum = 0;
 			for (x = 0; x < sizex; x++)
 				for (y = 0; y < sizey; y++)
 					for (z = 0; z < size_bz; z++)
 						sum += block->layers[0][(K+x)+(K+y)*(sizex + 2*K)+(K+z)*(sizex+2*K)*(sizey+2*K)];
-			printf("block %d got %d/%d alive\n", bz, sum, sizex*sizey*size_bz);
+			printf("block %u got %u/%u alive\n", bz, sum, sizex*sizey*size_bz);
 #endif
 		}
 	}
