@@ -1,7 +1,7 @@
 /* StarPU --- Runtime system for heterogeneous multicore architectures.
  *
- * Copyright (C) 2009, 2010, 2012  Université de Bordeaux 1
- * Copyright (C) 2010, 2011, 2012  Centre National de la Recherche Scientifique
+ * Copyright (C) 2009, 2010, 2012-2013, 2015  Université de Bordeaux
+ * Copyright (C) 2010, 2011, 2012, 2013, 2016  CNRS
  *
  * StarPU is free software; you can redistribute it and/or modify
  * it under the terms of the GNU Lesser General Public License as published by
@@ -27,19 +27,14 @@
 #include <string.h>
 #include <math.h>
 #include <sys/types.h>
-#include <pthread.h>
 #include <signal.h>
 
 #include <starpu.h>
 
-#ifdef STARPU_USE_GORDON
-#include <gordon/null.h>
-#endif
-
-#define FPRINTF(ofile, fmt, args ...) do { if (!getenv("STARPU_SSILENT")) {fprintf(ofile, fmt, ##args); }} while(0)
+#define FPRINTF(ofile, fmt, ...) do { if (!getenv("STARPU_SSILENT")) {fprintf(ofile, fmt, ## __VA_ARGS__); }} while(0)
 #define TAG(i, iter)	((starpu_tag_t)  (((uint64_t)iter)<<32 | (i)) )
 
-struct starpu_codelet cl = {};
+struct starpu_codelet cl;
 
 #define Ni	64
 #define Nk	256
@@ -72,7 +67,7 @@ static void parse_args(int argc, char **argv)
 
 void callback_cpu(void *argcb);
 
-static void tag_cleanup_grid(unsigned ni, unsigned iter)
+static void tag_cleanup_grid(unsigned iter)
 {
 	unsigned i;
 
@@ -83,12 +78,13 @@ static void tag_cleanup_grid(unsigned ni, unsigned iter)
 static int create_task_grid(unsigned iter)
 {
 	int i;
-	int ret;
 
 /*	FPRINTF(stderr, "start iter %d ni %d...\n", iter, ni); */
 
 	for (i = ni - 1; i > 0; i--)
 	{
+		int ret;
+
 		/* create a new task */
 		struct starpu_task *task = starpu_task_create();
 
@@ -108,11 +104,11 @@ static int create_task_grid(unsigned iter)
 	return 0;
 }
 
-void cpu_codelet(void *descr[] __attribute__ ((unused)), void *_args __attribute__ ((unused)))
+void cpu_codelet(void *descr[] STARPU_ATTRIBUTE_UNUSED, void *_args STARPU_ATTRIBUTE_UNUSED)
 {
 }
 
-int main(int argc __attribute__((unused)) , char **argv __attribute__((unused)))
+int main(int argc STARPU_ATTRIBUTE_UNUSED , char **argv STARPU_ATTRIBUTE_UNUSED)
 {
 	unsigned i;
 	int ret;
@@ -127,20 +123,15 @@ int main(int argc __attribute__((unused)) , char **argv __attribute__((unused)))
 	nk /= 16;
 #endif
 
-#ifdef STARPU_USE_GORDON
-	/* load an empty kernel and get its identifier */
-	unsigned gordon_null_kernel = load_gordon_null_kernel();
-#endif
-
 	parse_args(argc, argv);
 
+	starpu_codelet_init(&cl);
 	cl.cpu_funcs[0] = cpu_codelet;
+	cl.cpu_funcs_name[0] = "cpu_codelet";
 	cl.cuda_funcs[0] = cpu_codelet;
 	cl.opencl_funcs[0] = cpu_codelet;
-#ifdef STARPU_USE_GORDON
-	cl.gordon_func = gordon_null_kernel;
-#endif
 	cl.nbuffers = 0;
+	cl.name = "dummy";
 
 	FPRINTF(stderr, "ITER : %u\n", nk);
 
@@ -153,7 +144,7 @@ int main(int argc __attribute__((unused)) , char **argv __attribute__((unused)))
 
 		/* cleanup old grids ... */
 		if (i > 1)
-			tag_cleanup_grid(ni, i-1);
+			tag_cleanup_grid(i-1);
 	}
 
 enodev:
