@@ -1,7 +1,7 @@
 /* StarPU --- Runtime system for heterogeneous multicore architectures.
  *
- * Copyright (C) 2009, 2010  Université de Bordeaux 1
- * Copyright (C) 2010, 2011, 2012  Centre National de la Recherche Scientifique
+ * Copyright (C) 2009, 2010, 2013, 2015  Université de Bordeaux
+ * Copyright (C) 2010, 2011, 2012, 2013, 2015, 2016  CNRS
  *
  * StarPU is free software; you can redistribute it and/or modify
  * it under the terms of the GNU Lesser General Public License as published by
@@ -15,27 +15,38 @@
  * See the GNU Lesser General Public License in COPYING.LGPL for more details.
  */
 
+/*
+ * This example shows how to reuse a tag.
+ * We define a series of dependency chains, shaped this way:
+ *
+ *           ...               ...
+ *            v                 v
+ * ...    task (i,  j)     task (i,  j+1)     ...
+ *            v                 v
+ * ...    task (i+1,j)     task (i+1,j+1)     ...
+ *            v                 v
+ *           ...               ...
+ *
+ * And this grid is used several times, by waiting for the completion of a
+ * chain before starting it over.
+ */
+
 #include <sys/types.h>
 #include <semaphore.h>
 #include <string.h>
 #include <math.h>
-#include <pthread.h>
 #include <unistd.h>
 #include <signal.h>
 
 #include <starpu.h>
 
-#ifdef STARPU_USE_GORDON
-#include <gordon/null.h>
-#endif
-
 #define Nrolls	4
 #define SLEEP 1
 
-#define FPRINTF(ofile, fmt, args ...) do { if (!getenv("STARPU_SSILENT")) {fprintf(ofile, fmt, ##args); }} while(0)
+#define FPRINTF(ofile, fmt, ...) do { if (!getenv("STARPU_SSILENT")) {fprintf(ofile, fmt, ## __VA_ARGS__); }} while(0)
 #define TAG(i, iter)	((starpu_tag_t)  (((uint64_t)((iter)%Nrolls))<<32 | (i)) )
 
-struct starpu_codelet cl = {};
+struct starpu_codelet cl;
 
 #define Ni	64
 #define Nk	256
@@ -81,7 +92,7 @@ static void create_task_grid(unsigned iter)
 		struct starpu_task *task = tasks[iter][i] = starpu_task_create();
 
 		task->cl = &cl;
-		task->cl_arg = (void*)(uintptr_t) (i | (iter << 16));
+		/* task->cl_arg = (void*)(uintptr_t) (i | (iter << 16)); */
 
 		task->use_tag = 1;
 		task->tag_id = TAG(i, iter);
@@ -98,11 +109,12 @@ static void create_task_grid(unsigned iter)
 static int start_task_grid(unsigned iter)
 {
 	unsigned i;
-	int ret;
 
 	/* FPRINTF(stderr, "start grid %d ni %d...\n", iter, ni); */
 
-	for (i = 0; i < ni; i++) {
+	for (i = 0; i < ni; i++)
+	{
+		int ret;
 		ret = starpu_task_submit(tasks[iter][i]);
 		if (ret == -ENODEV) return 77;
 		STARPU_CHECK_RETURN_VALUE(ret, "starpu_task_submit");
@@ -110,7 +122,7 @@ static int start_task_grid(unsigned iter)
 	return 0;
 }
 
-void cpu_codelet(void *descr[], void *_args __attribute__((unused)))
+void cpu_codelet(void *descr[] STARPU_ATTRIBUTE_UNUSED, void *_args STARPU_ATTRIBUTE_UNUSED)
 {
 /*	int i = (uintptr_t) _args;
 	printf("doing %x\n", i);
@@ -119,9 +131,9 @@ void cpu_codelet(void *descr[], void *_args __attribute__((unused)))
 */
 }
 
-int main(int argc __attribute__((unused)) , char **argv __attribute__((unused)))
+int main(int argc STARPU_ATTRIBUTE_UNUSED , char **argv STARPU_ATTRIBUTE_UNUSED)
 {
-	unsigned i;
+	unsigned i, j;
 	int ret;
 
 	ret = starpu_init(NULL);
@@ -134,20 +146,15 @@ int main(int argc __attribute__((unused)) , char **argv __attribute__((unused)))
 	nk /= 16;
 #endif
 
-#ifdef STARPU_USE_GORDON
-	/* load an empty kernel and get its identifier */
-	unsigned gordon_null_kernel = load_gordon_null_kernel();
-#endif
-
 	parse_args(argc, argv);
 
+	starpu_codelet_init(&cl);
 	cl.cpu_funcs[0] = cpu_codelet;
+	cl.cpu_funcs_name[0] = "cpu_codelet";
 	cl.cuda_funcs[0] = cpu_codelet;
 	cl.opencl_funcs[0] = cpu_codelet;
-#ifdef STARPU_USE_GORDON
-	cl.gordon_func = gordon_null_kernel;
-#endif
 	cl.nbuffers = 0;
+	cl.name = "dummy";
 
 	FPRINTF(stderr, "ITER : %u\n", nk);
 
@@ -170,9 +177,14 @@ int main(int argc __attribute__((unused)) , char **argv __attribute__((unused)))
 
 	starpu_shutdown();
 	FPRINTF(stderr, "TEST DONE ...\n");
-	return EXIT_SUCCESS;
 
 enodev:
-	starpu_shutdown();
-	return 77;
+	for (i = 0; i < Nrolls; i++)
+	{
+		for (j = 0; j < ni; j++)
+			starpu_task_destroy(tasks[i][j]);
+		free(tasks[i]);
+	}
+
+	return ret;
 }
