@@ -1,7 +1,8 @@
 /* StarPU --- Runtime system for heterogeneous multicore architectures.
  *
- * Copyright (C) 2009-2013  Université de Bordeaux 1
- * Copyright (C) 2010-2013  Centre National de la Recherche Scientifique
+ * Copyright (C) 2009-2014, 2016  Université de Bordeaux
+ * Copyright (C) 2010-2014  CNRS
+ * Copyright (C) 2014  INRIA
  *
  * StarPU is free software; you can redistribute it and/or modify
  * it under the terms of the GNU Lesser General Public License as published by
@@ -45,6 +46,7 @@ typedef UINT_PTR uintptr_t;
 #include <starpu_thread_util.h>
 #include <starpu_util.h>
 #include <starpu_data.h>
+#include <starpu_disk.h>
 #include <starpu_data_interfaces.h>
 #include <starpu_data_filters.h>
 #include <starpu_stdlib.h>
@@ -58,22 +60,23 @@ typedef UINT_PTR uintptr_t;
 #include <starpu_rand.h>
 #include <starpu_cuda.h>
 #include <starpu_cublas.h>
+#include <starpu_cusparse.h>
 #include <starpu_bound.h>
 #include <starpu_hash.h>
 #include <starpu_profiling.h>
 #include <starpu_top.h>
 #include <starpu_fxt.h>
 #include <starpu_driver.h>
+#include <starpu_tree.h>
+#include <starpu_openmp.h>
+#include <starpu_simgrid_wrap.h>
+#include <starpu_bitmap.h>
 
 #ifdef __cplusplus
 extern "C"
 {
 #endif
 
-#ifdef STARPU_SIMGRID
-#define main starpu_main
-#endif
-
 struct starpu_conf
 {
 	int magic;
@@ -84,6 +87,8 @@ struct starpu_conf
 	int ncpus;
 	int ncuda;
 	int nopencl;
+	int nmic;
+	int nscc;
 
 	unsigned use_explicit_workers_bindid;
 	unsigned workers_bindid[STARPU_NMAXWORKERS];
@@ -94,14 +99,24 @@ struct starpu_conf
 	unsigned use_explicit_workers_opencl_gpuid;
 	unsigned workers_opencl_gpuid[STARPU_NMAXWORKERS];
 
+	unsigned use_explicit_workers_mic_deviceid;
+	unsigned workers_mic_deviceid[STARPU_NMAXWORKERS];
+
+	unsigned use_explicit_workers_scc_deviceid;
+	unsigned workers_scc_deviceid[STARPU_NMAXWORKERS];
+
 	int bus_calibrate;
 	int calibrate;
 
 	int single_combined_worker;
 
+	char *mic_sink_program_path;
+
 	int disable_asynchronous_copy;
 	int disable_asynchronous_cuda_copy;
 	int disable_asynchronous_opencl_copy;
+	int disable_asynchronous_mic_copy;
+
 	unsigned *cuda_opengl_interoperability;
 	unsigned n_cuda_opengl_interoperability;
 
@@ -109,14 +124,19 @@ struct starpu_conf
 	unsigned n_not_launched_drivers;
 
 	unsigned trace_buffer_size;
+	int global_sched_ctx_min_priority;
+	int global_sched_ctx_max_priority;
+
 };
 
 int starpu_conf_init(struct starpu_conf *conf);
 
 int starpu_init(struct starpu_conf *conf) STARPU_WARN_UNUSED_RESULT;
 
-void starpu_pause();
-void starpu_resume();
+int starpu_initialize(struct starpu_conf *user_conf, int *argc, char ***argv);
+
+void starpu_pause(void);
+void starpu_resume(void);
 
 void starpu_shutdown(void);
 
@@ -125,6 +145,7 @@ void starpu_topology_print(FILE *f);
 int starpu_asynchronous_copy_disabled(void);
 int starpu_asynchronous_cuda_copy_disabled(void);
 int starpu_asynchronous_opencl_copy_disabled(void);
+int starpu_asynchronous_mic_copy_disabled(void);
 
 void starpu_display_stats();
 
