@@ -1,7 +1,7 @@
 /* StarPU --- Runtime system for heterogeneous multicore architectures.
  *
- * Copyright (C) 2010-2011  Université de Bordeaux 1
- * Copyright (C) 2010, 2011  Centre National de la Recherche Scientifique
+ * Copyright (C) 2010-2011, 2013, 2015-2017  Université de Bordeaux
+ * Copyright (C) 2010, 2011, 2013, 2014  CNRS
  *
  * StarPU is free software; you can redistribute it and/or modify
  * it under the terms of the GNU Lesser General Public License as published by
@@ -20,13 +20,18 @@
 
 #include <starpu_perfmodel.h>
 
+#ifdef __cplusplus
+extern "C"
+{
+#endif
+
 #define STARPU_FXT_MAX_FILES	64
 
 struct starpu_fxt_codelet_event
 {
-	char symbol[256]; /* name of the codelet */
+	char symbol[256];
 	int workerid;
-	enum starpu_perf_archtype archtype;
+	char perfmodel_archname[256];
 	uint32_t hash;
 	size_t size;
 	float time;
@@ -35,35 +40,43 @@ struct starpu_fxt_codelet_event
 struct starpu_fxt_options
 {
 	unsigned per_task_colour;
+	unsigned no_events;
 	unsigned no_counter;
 	unsigned no_bus;
+	unsigned no_flops;
 	unsigned ninputfiles;
 	char *filenames[STARPU_FXT_MAX_FILES];
 	char *out_paje_path;
 	char *distrib_time_path;
 	char *activity_path;
 	char *dag_path;
+	char *tasks_path;
+	char *data_path;
+	char *anim_path;
 
-	/* In case we are going to gather multiple traces (eg in the case of
-	 * MPI processes), we may need to prefix the name of the containers. */
 	char *file_prefix;
 	uint64_t file_offset;
 	int file_rank;
 
-	/*
-	 *	Output parameters
-	 */
-
-	char worker_names[STARPU_NMAXWORKERS][256]; 
-	enum starpu_perf_archtype worker_archtypes[STARPU_NMAXWORKERS];
+	char worker_names[STARPU_NMAXWORKERS][256];
+	struct starpu_perfmodel_arch worker_archtypes[STARPU_NMAXWORKERS];
 	int nworkers;
 
-	/* In case we want to dump the list of codelets to an external tool */
 	struct starpu_fxt_codelet_event **dumped_codelets;
 	long dumped_codelets_count;
 };
 
 void starpu_fxt_options_init(struct starpu_fxt_options *options);
 void starpu_fxt_generate_trace(struct starpu_fxt_options *options);
+void starpu_fxt_autostart_profiling(int autostart);
+void starpu_fxt_start_profiling(void);
+void starpu_fxt_stop_profiling(void);
+void starpu_fxt_write_data_trace(char *filename_in);
+void starpu_fxt_trace_user_event(unsigned long code);
+void starpu_fxt_trace_user_event_string(const char *s);
+
+#ifdef __cplusplus
+}
+#endif
 
 #endif /* __STARPU_FXT_H__ */
