@@ -1,7 +1,7 @@
 /* StarPU --- Runtime system for heterogeneous multicore architectures.
  *
- * Copyright (C) 2010, 2011  Université de Bordeaux 1
- * Copyright (C) 2010, 2011  Centre National de la Recherche Scientifique
+ * Copyright (C) 2010-2014, 2016  Université de Bordeaux
+ * Copyright (C) 2010, 2011, 2013  CNRS
  *
  * StarPU is free software; you can redistribute it and/or modify
  * it under the terms of the GNU Lesser General Public License as published by
@@ -20,7 +20,8 @@
 
 #include <starpu.h>
 #include <errno.h>
-#include <sys/time.h>
+#include <time.h>
+#include <starpu_util.h>
 
 #ifdef __cplusplus
 extern "C"
@@ -30,30 +31,24 @@ extern "C"
 #define STARPU_PROFILING_DISABLE	0
 #define STARPU_PROFILING_ENABLE		1
 
-struct starpu_task_profiling_info
+struct starpu_profiling_task_info
 {
-	/* Task submission */
 	struct timespec submit_time;
 
-	/* Scheduling overhead */
 	struct timespec push_start_time;
 	struct timespec push_end_time;
 	struct timespec pop_start_time;
 	struct timespec pop_end_time;
 
-	/* Take input data */
 	struct timespec acquire_data_start_time;
 	struct timespec acquire_data_end_time;
 
-	/* Execution */
 	struct timespec start_time;
 	struct timespec end_time;
 
-	/* Release data */
 	struct timespec release_data_start_time;
 	struct timespec release_data_end_time;
 
-	/* Callback */
 	struct timespec callback_start_time;
 	struct timespec callback_end_time;
 
@@ -62,11 +57,10 @@ struct starpu_task_profiling_info
 
 	uint64_t used_cycles;
 	uint64_t stall_cycles;
-	double power_consumed;
+	double energy_consumed;
 };
 
-/* The timing is provided since the previous call to starpu_worker_get_profiling_info */
-struct starpu_worker_profiling_info
+struct starpu_profiling_worker_info
 {
 	struct timespec start_time;
 	struct timespec total_time;
@@ -76,10 +70,10 @@ struct starpu_worker_profiling_info
 
 	uint64_t used_cycles;
 	uint64_t stall_cycles;
-	double power_consumed;
+	double energy_consumed;
 };
 
-struct starpu_bus_profiling_info
+struct starpu_profiling_bus_info
 {
 	struct timespec start_time;
 	struct timespec total_time;
@@ -87,43 +81,46 @@ struct starpu_bus_profiling_info
 	int transfer_count;
 };
 
-/* This function sets the ID used for profiling trace filename */
-void starpu_set_profiling_id(int new_id);
-
-/* This function sets the profiling status:
- * - enable with STARPU_PROFILING_ENABLE
- * - disable with STARPU_PROFILING_DISABLE
- * Negative return values indicate an error, otherwise the previous status is
- * returned. Calling this function resets the profiling measurements. */
+void starpu_profiling_init(void);
+void starpu_profiling_set_id(int new_id);
 int starpu_profiling_status_set(int status);
-
-/* Return the current profiling status or a negative value in case there was an
- * error. */
 int starpu_profiling_status_get(void);
 
-/* Get the profiling info associated to a worker, and reset the profiling
- * measurements. If worker_info is NULL, we only reset the counters. */
-int starpu_worker_get_profiling_info(int workerid, struct starpu_worker_profiling_info *worker_info);
+#ifdef BUILDING_STARPU
+#include <common/utils.h>
+#ifdef __GNUC__
+extern int _starpu_profiling;
+#define starpu_profiling_status_get() ({ \
+	int __ret; \
+	ANNOTATE_HAPPENS_AFTER(&_starpu_profiling); \
+	__ret = _starpu_profiling; \
+	ANNOTATE_HAPPENS_BEFORE(&_starpu_profiling); \
+	__ret; \
+})
+#endif
+#endif
+
+int starpu_profiling_worker_get_info(int workerid, struct starpu_profiling_worker_info *worker_info);
 
 int starpu_bus_get_count(void);
 int starpu_bus_get_id(int src, int dst);
 int starpu_bus_get_src(int busid);
 int starpu_bus_get_dst(int busid);
 
-int starpu_bus_get_profiling_info(int busid, struct starpu_bus_profiling_info *bus_info);
+int starpu_bus_get_profiling_info(int busid, struct starpu_profiling_bus_info *bus_info);
 
 /* Some helper functions to manipulate profiling API output */
 /* Reset timespec */
-static inline void starpu_timespec_clear(struct timespec *tsp)
+static __starpu_inline void starpu_timespec_clear(struct timespec *tsp)
 {
 	tsp->tv_sec = 0;
 	tsp->tv_nsec = 0;
 }
 
 /* Computes result = a + b */
-static inline void starpu_timespec_add(struct timespec *a,
-					struct timespec *b,
-					struct timespec *result)
+static __starpu_inline void starpu_timespec_add(struct timespec *a,
+						struct timespec *b,
+						struct timespec *result)
 {
 	result->tv_sec = a->tv_sec + b->tv_sec;
 	result->tv_nsec = a->tv_nsec + b->tv_nsec;
@@ -136,8 +133,8 @@ static inline void starpu_timespec_add(struct timespec *a,
 }
 
 /* Computes res += b */
-static inline void starpu_timespec_accumulate(struct timespec *result,
-						struct timespec *a)
+static __starpu_inline void starpu_timespec_accumulate(struct timespec *result,
+						       struct timespec *a)
 {
 	result->tv_sec += a->tv_sec;
 	result->tv_nsec += a->tv_nsec;
@@ -150,9 +147,9 @@ static inline void starpu_timespec_accumulate(struct timespec *result,
 }
 
 /* Computes result = a - b */
-static inline void starpu_timespec_sub(struct timespec *a,
-					struct timespec *b,
-					struct timespec *result)
+static __starpu_inline void starpu_timespec_sub(const struct timespec *a,
+						const struct timespec *b,
+						struct timespec *result)
 {
 	result->tv_sec = a->tv_sec - b->tv_sec;
 	result->tv_nsec = a->tv_nsec - b->tv_nsec;
@@ -167,12 +164,11 @@ static inline void starpu_timespec_sub(struct timespec *a,
 #define starpu_timespec_cmp(a, b, CMP)                          \
 	(((a)->tv_sec == (b)->tv_sec) ? ((a)->tv_nsec CMP (b)->tv_nsec) : ((a)->tv_sec CMP (b)->tv_sec))
 
-/* Returns the time elapsed between start and end in microseconds */
 double starpu_timing_timespec_delay_us(struct timespec *start, struct timespec *end);
 double starpu_timing_timespec_to_us(struct timespec *ts);
 
-void starpu_bus_profiling_helper_display_summary(void);
-void starpu_worker_profiling_helper_display_summary(void);
+void starpu_profiling_bus_helper_display_summary(void);
+void starpu_profiling_worker_helper_display_summary(void);
 
 #ifdef __cplusplus
 }
