@@ -1,7 +1,7 @@
 /* StarPU --- Runtime system for heterogeneous multicore architectures.
  *
- * Copyright (C) 2009, 2010  Université de Bordeaux 1
- * Copyright (C) 2010, 2011, 2012  Centre National de la Recherche Scientifique
+ * Copyright (C) 2009, 2010, 2014-2016  Université de Bordeaux
+ * Copyright (C) 2010, 2011, 2012, 2013, 2016, 2017  CNRS
  *
  * StarPU is free software; you can redistribute it and/or modify
  * it under the terms of the GNU Lesser General Public License as published by
@@ -20,32 +20,45 @@
 
 #ifdef STARPU_QUICK_CHECK
 #  define NITER	32
+#elif !defined(STARPU_LONG_CHECK)
+#  define NITER	256
 #else
 #  define NITER	2048
 #endif
 
-unsigned token = 42;
+int token = 42;
 starpu_data_handle_t token_handle;
 
 #ifdef STARPU_USE_CUDA
-extern void increment_cuda(void *descr[], __attribute__ ((unused)) void *_args);
+extern void increment_cuda(void *descr[], STARPU_ATTRIBUTE_UNUSED void *_args);
 #endif
 
-void increment_cpu(void *descr[], __attribute__ ((unused)) void *_args)
+void increment_cpu(void *descr[], STARPU_ATTRIBUTE_UNUSED void *_args)
 {
-	unsigned *tokenptr = (unsigned *)STARPU_VECTOR_GET_PTR(descr[0]);
+	int *tokenptr = (int *)STARPU_VECTOR_GET_PTR(descr[0]);
 	(*tokenptr)++;
 }
 
+/* Dummy cost function for simgrid */
+static double cost_function(struct starpu_task *task STARPU_ATTRIBUTE_UNUSED, unsigned nimpl STARPU_ATTRIBUTE_UNUSED)
+{
+	return 0.000001;
+}
+static struct starpu_perfmodel dumb_model =
+{
+	.type		= STARPU_COMMON,
+	.cost_function	= cost_function
+};
+
 static struct starpu_codelet increment_cl =
 {
-	.where = STARPU_CPU|STARPU_CUDA,
 #ifdef STARPU_USE_CUDA
-	.cuda_funcs = {increment_cuda, NULL},
+	.cuda_funcs = {increment_cuda},
 #endif
-	.cpu_funcs = {increment_cpu, NULL},
+	.cpu_funcs = {increment_cpu},
 	.nbuffers = 1,
-	.modes = {STARPU_RW}
+	.modes = {STARPU_RW},
+	.model = &dumb_model
 };
 
 void increment_token(void)
@@ -64,31 +77,33 @@ int main(int argc, char **argv)
 {
 	int ret, rank, size;
 
-	MPI_Init(NULL, NULL);
-	MPI_Comm_rank(MPI_COMM_WORLD, &rank);
-	MPI_Comm_size(MPI_COMM_WORLD, &size);
+	MPI_INIT_THREAD(&argc, &argv, MPI_THREAD_SERIALIZED);
+	starpu_mpi_comm_rank(MPI_COMM_WORLD, &rank);
+	starpu_mpi_comm_size(MPI_COMM_WORLD, &size);
+
+	ret = starpu_init(NULL);
+	STARPU_CHECK_RETURN_VALUE(ret, "starpu_init");
+	ret = starpu_mpi_init(NULL, NULL, 0);
+	STARPU_CHECK_RETURN_VALUE(ret, "starpu_mpi_init");
 
 	if (size < 2)
 	{
 		if (rank == 0)
 			FPRINTF(stderr, "We need at least 2 processes.\n");
 
+		starpu_mpi_shutdown();
+		starpu_shutdown();
 		MPI_Finalize();
 		return STARPU_TEST_SKIPPED;
 	}
 
-	ret = starpu_init(NULL);
-	STARPU_CHECK_RETURN_VALUE(ret, "starpu_init");
-	ret = starpu_mpi_initialize();
-	STARPU_CHECK_RETURN_VALUE(ret, "starpu_mpi_initialize");
+	starpu_vector_data_register(&token_handle, STARPU_MAIN_RAM, (uintptr_t)&token, 1, sizeof(token));
 
-	starpu_vector_data_register(&token_handle, 0, (uintptr_t)&token, 1, sizeof(unsigned));
+	int nloops = NITER;
+	int loop;
 
-	unsigned nloops = NITER;
-	unsigned loop;
-
-	unsigned last_loop = nloops - 1;
-	unsigned last_rank = size - 1;
+	int last_loop = nloops - 1;
+	int last_rank = size - 1;
 
 	for (loop = 0; loop < nloops; loop++)
 	{
@@ -119,15 +134,18 @@ int main(int argc, char **argv)
 		}
 	}
 
+	starpu_data_unregister(token_handle);
 	starpu_mpi_shutdown();
 	starpu_shutdown();
 
 	MPI_Finalize();
 
+#ifndef STARPU_SIMGRID
 	if (rank == last_rank)
 	{
 		STARPU_ASSERT(token == nloops*size);
 	}
+#endif
 
 	return 0;
 }
