@@ -1,6 +1,7 @@
 /* StarPU --- Runtime system for heterogeneous multicore architectures.
  *
  * Copyright (C) 2010,2011, 2014 University of Bordeaux
+ * Copyright (C) 2016, 2017  CNRS
  *
  * StarPU is free software; you can redistribute it and/or modify
  * it under the terms of the GNU Lesser General Public License as published by
@@ -24,7 +25,7 @@ static void soclEnqueueReadBuffer_cpu_task(void *descr[], void *args) {
   gc_entity_release(ev);
 
    char * ptr = (void*)STARPU_VARIABLE_GET_PTR(descr[0]);
-   DEBUG_MSG("[Buffer %d] Reading %ld bytes from %p to %p\n", cmd->buffer->id, cmd->cb, ptr+cmd->offset, cmd->ptr);
+   DEBUG_MSG("[Buffer %d] Reading %ld bytes from %p to %p\n", cmd->buffer->id, (long)cmd->cb, ptr+cmd->offset, cmd->ptr);
 
    //This fix is for people who use USE_HOST_PTR and still use ReadBuffer to sync the buffer in host mem at host_ptr.
    //They should use buffer mapping facilities instead.
@@ -43,9 +44,9 @@ static void soclEnqueueReadBuffer_opencl_task(void *descr[], void *args) {
 
    cl_mem mem = (cl_mem)STARPU_VARIABLE_GET_PTR(descr[0]);
 
-   DEBUG_MSG("[Buffer %d] Reading %ld bytes from offset %ld into %p\n", cmd->buffer->id, cmd->cb, cmd->offset, cmd->ptr);
+   DEBUG_MSG("[Buffer %d] Reading %ld bytes from offset %ld into %p\n", cmd->buffer->id, (long)cmd->cb, (long)cmd->offset, cmd->ptr);
 
-   int wid = starpu_worker_get_id();
+   int wid = starpu_worker_get_id_check();
    cl_command_queue cq;
    starpu_opencl_get_queue(wid, &cq);
 
@@ -68,8 +69,8 @@ static struct starpu_perfmodel read_buffer_perfmodel = {
 static struct starpu_codelet codelet_readbuffer = {
    .where = STARPU_OPENCL,
    .model = &read_buffer_perfmodel,
-   .cpu_funcs = { &soclEnqueueReadBuffer_cpu_task, NULL },
-   .opencl_funcs = { &soclEnqueueReadBuffer_opencl_task, NULL },
+   .cpu_funcs = { &soclEnqueueReadBuffer_cpu_task },
+   .opencl_funcs = { &soclEnqueueReadBuffer_opencl_task },
    .modes = {STARPU_R},
    .nbuffers = 1
 };
