@@ -20,10 +20,10 @@ cl_int command_unmap_mem_object_submit(command_unmap_mem_object cmd) {
 	/* Aliases */
 	cl_mem buffer = cmd->buffer;
 
-	//FIXME: use a callback
-	starpu_task task = task_create_cpu((void(*)(void*))starpu_data_release, buffer->handle, 0);
-
-	task_submit(task, cmd);
+	static struct starpu_codelet codelet = {
+		.name = "SOCL_UNMAP_MEM_OBJECT"
+	};
+	cpu_task_submit(cmd, (void(*)(void*))starpu_data_release, buffer->handle, 0, 1, &codelet, 0, NULL);
 
 	return CL_SUCCESS;
 }
@@ -38,9 +38,11 @@ soclEnqueueUnmapMemObject(cl_command_queue cq,
 {
 	command_unmap_mem_object cmd = command_unmap_mem_object_create(buffer, ptr);
 
+   cl_event ev = command_event_get(cmd);
+
 	command_queue_enqueue(cq, cmd, num_events, events);
 
-	RETURN_EVENT(cmd, event);
+	RETURN_EVENT(ev, event);
 
 	return CL_SUCCESS;
 }
