@@ -19,11 +19,13 @@
 CL_API_ENTRY cl_int CL_API_CALL
 soclFinish(cl_command_queue cq) CL_API_SUFFIX__VERSION_1_0 {
 
-	command_marker cmd = command_barrier_create();
+	command_barrier cmd = command_barrier_create();
+
+   cl_event ev = command_event_get(cmd);
 
 	command_queue_enqueue(cq, cmd, 0, NULL);
 
-	MAY_BLOCK(CL_TRUE)
+	MAY_BLOCK_THEN_RETURN_EVENT(ev, CL_TRUE, (cl_event*)NULL);
 
 	return CL_SUCCESS;
 }
