@@ -1,6 +1,8 @@
 /* StarPU --- Runtime system for heterogeneous multicore architectures.
  *
- * Copyright (C) 2010,2011 University of Bordeaux
+ * Copyright (C) 2010-2012 University of Bordeaux
+ * Copyright (C) 2012, 2014, 2017 CNRS
+ * Copyright (C) 2012 Vincent Danjean <Vincent.Danjean@ens-lyon.org>
  *
  * StarPU is free software; you can redistribute it and/or modify
  * it under the terms of the GNU Lesser General Public License as published by
@@ -29,16 +31,16 @@ static volatile entity gc_list = NULL;
 static volatile entity entities = NULL;
 
 /* Mutex and cond for release */
-static pthread_mutex_t gc_mutex = PTHREAD_MUTEX_INITIALIZER;
-static pthread_cond_t  gc_cond = PTHREAD_COND_INITIALIZER;
+static starpu_pthread_mutex_t gc_mutex = STARPU_PTHREAD_MUTEX_INITIALIZER;
+static starpu_pthread_cond_t  gc_cond = STARPU_PTHREAD_COND_INITIALIZER;
 
 /* Set to 1 to stop release thread execution */
 static volatile int gc_stop_required = 0;
 
-#define GC_LOCK pthread_mutex_lock(&gc_mutex)
-#define GC_UNLOCK { pthread_cond_signal(&gc_cond); \
-                    pthread_mutex_unlock(&gc_mutex);}
-#define GC_UNLOCK_NO_SIGNAL pthread_mutex_unlock(&gc_mutex)
+#define GC_LOCK starpu_pthread_mutex_lock(&gc_mutex)
+#define GC_UNLOCK { starpu_pthread_cond_signal(&gc_cond); \
+                    starpu_pthread_mutex_unlock(&gc_mutex);}
+#define GC_UNLOCK_NO_SIGNAL starpu_pthread_mutex_unlock(&gc_mutex)
 
 /* Thread routine */
 static void * gc_thread_routine(void *UNUSED(arg)) {
@@ -61,8 +63,7 @@ static void * gc_thread_routine(void *UNUSED(arg)) {
 
       /* Release entity */
       entity next = r->next;
-#warning FIXME: free memory
-//      free(r);
+      free(r);
 
       r = next;
     }
@@ -80,18 +81,18 @@ static void * gc_thread_routine(void *UNUSED(arg)) {
     }
 
     /* Otherwise we sleep */
-    pthread_cond_wait(&gc_cond, &gc_mutex);
+    starpu_pthread_cond_wait(&gc_cond, &gc_mutex);
 
   } while (1);
 
-  pthread_exit(NULL);
+  starpu_pthread_exit(NULL);
 }
 
-static pthread_t gc_thread;
+static starpu_pthread_t gc_thread;
 
 /* Start garbage collection */
 void gc_start(void) {
-  pthread_create(&gc_thread, NULL, gc_thread_routine, NULL);
+  starpu_pthread_create(&gc_thread, NULL, gc_thread_routine, NULL);
 }
 
 /* Stop garbage collection */
@@ -102,18 +103,24 @@ void gc_stop(void) {
 
   GC_UNLOCK;
 
-  pthread_join(gc_thread, NULL);
+  starpu_pthread_join(gc_thread, NULL);
 }
 
-int gc_entity_release_ex(entity e) {
+int gc_entity_release_ex(entity e, const char * DEBUG_PARAM(caller)) {
+
+  DEBUG_MSG("[%s] Decrementing refcount of %s %p to ", caller, e->name, (void *)e);
 
   /* Decrement reference count */
   int refs = __sync_sub_and_fetch(&e->refs, 1);
 
+  DEBUG_MSG_NOHEAD("%d\n", refs);
+
+  assert(refs >= 0);
+
   if (refs != 0)
     return 0;
 
-  DEBUG_MSG("Releasing entity %lx\n", e);
+  DEBUG_MSG("[%s] Releasing %s %p\n", caller, e->name, (void *)e);
 
   GC_LOCK;
 
@@ -139,12 +146,17 @@ int gc_entity_release_ex(entity e) {
 /**
  * Initialize entity
  */
-void gc_entity_init(void *arg, void (*release_callback)(void*)) {
+void gc_entity_init(void *arg, void (*release_callback)(void*), char * name) {
+
+  DEBUG_MSG("Initializing entity %p (%s)\n", arg, name);
+
   struct entity * e = (entity)arg;
 
+  e->dispatch = &socl_master_dispatch;
   e->refs = 1;
   e->release_callback = release_callback;
   e->prev = NULL;
+  e->name = name;
 
   GC_LOCK;
 
@@ -159,17 +171,23 @@ void gc_entity_init(void *arg, void (*release_callback)(void*)) {
 /**
  * Allocate and initialize entity
  */
-void * gc_entity_alloc(unsigned int size, void (*release_callback)(void*)) {
+void * gc_entity_alloc(unsigned int size, void (*release_callback)(void*), char * name) {
   void * e = malloc(size);
-  gc_entity_init(e, release_callback);
+  gc_entity_init(e, release_callback, name);
   return e;
 }
 
 /** Retain entity */
-void gc_entity_retain(void *arg) {
+void gc_entity_retain_ex(void *arg, const char * DEBUG_PARAM(caller)) {
 	struct entity * e = (entity)arg;
 
-	__sync_fetch_and_add(&e->refs, 1);
+   #ifdef DEBUG
+	int refs =
+   #endif
+      __sync_add_and_fetch(&e->refs, 1);
+
+
+   DEBUG_MSG("[%s] Incrementing refcount of %s %p to %d\n", caller, e->name, e, refs);
 }
 
 int gc_active_entity_count(void) {
@@ -184,6 +202,20 @@ int gc_active_entity_count(void) {
 	return i;
 }
 
+void gc_print_remaining_entities(void) {
+   DEBUG_MSG("Remaining entities:\n");
+
+   GC_LOCK;
+
+   entity e = entities;
+   while (e != NULL) {
+      DEBUG_MSG("  - %s %p\n", e->name, (void *)e);
+      e = e->next;
+   }
+
+   GC_UNLOCK;
+}
+
 #undef GC_LOCK
 #undef GC_UNLOCK
 #undef GC_UNLOCK_NO_SIGNAL
