@@ -1,6 +1,7 @@
 /* StarPU --- Runtime system for heterogeneous multicore architectures.
  *
- * Copyright (C) 2010,2011 University of Bordeaux
+ * Copyright (C) 2010-2012 University of Bordeaux
+ * Copyright (C) 2012,2014,2015 CNRS
  *
  * StarPU is free software; you can redistribute it and/or modify
  * it under the terms of the GNU Lesser General Public License as published by
@@ -17,26 +18,17 @@
 #ifndef SOCL_H
 #define SOCL_H
 
-#ifndef CL_HEADERS
-#include "CL/cl.h"
-#else
-#include CL_HEADERS "CL/cl.h"
-#endif
-
-/* Additional command type */
-#define CL_COMMAND_BARRIER 0x99987
+#define CL_CONTEXT_SCHEDULER_SOCL   0xFF01
+#define CL_CONTEXT_NAME_SOCL        0xFF02
 
 #include <string.h>
 #include <stdlib.h>
 #include <stdint.h>
 #include <unistd.h>
-#include <pthread.h>
+#include "CL/cl.h"
+#include "ocl_icd.h"
 
 #include <starpu.h>
-#include <starpu_opencl.h>
-#include <starpu_data_interfaces.h>
-#include <starpu_profiling.h>
-#include <starpu_task.h>
 
 typedef struct starpu_task * starpu_task;
 
@@ -53,25 +45,17 @@ typedef struct starpu_task * starpu_task;
  */
 typedef struct entity * entity;
 
-#include "command.h"
-#include "command_list.h"
-#include "command_queue.h"
-#include "debug.h"
-#include "devices.h"
-#include "event.h"
-#include "gc.h"
-#include "mem_objects.h"
-#include "task.h"
-#include "util.h"
-
-
 struct entity {
+  struct _cl_icd_dispatch * dispatch;
   /* Reference count */
   size_t refs;
 
   /* Callback called on release */
   void (*release_callback)(void*entity);
 
+  /* Entity identifier (used for debugging purpose) */
+  char * name;
+
   /* Next entity in garbage collector queue */
   entity prev;
   entity next;
@@ -81,42 +65,50 @@ struct entity {
  * this macro as their first field */
 #define CL_ENTITY struct entity _entity;
 
-struct _cl_platform_id {};
 
-#define RETURN_EVENT(cmd, event) \
-	if (event != NULL) { \
-		cl_event ev = command_event_get(cmd);\
-		gc_entity_retain(ev);\
+#include "command.h"
+#include "command_list.h"
+#include "command_queue.h"
+#include "debug.h"
+#include "event.h"
+#include "gc.h"
+#include "mem_objects.h"
+#include "task.h"
+#include "util.h"
+
+
+
+struct _cl_platform_id {
+   struct _cl_icd_dispatch *dispatch;
+};
+
+struct _cl_device_id {
+   struct _cl_icd_dispatch *dispatch; 
+   int device_id;
+   int worker_id;
+};
+
+#define RETURN_EVENT(ev, event) \
+	if ((event) != NULL) { \
 		*event = ev; \
-	}
+	} \
+   else {\
+      gc_entity_release(ev);\
+   }
 
-#define RETURN_CUSTOM_EVENT(src, tgt) \
-	if (tgt != NULL) { \
-		gc_entity_retain(src); \
-		*tgt = src; \
-	}
-
-#define MAY_BLOCK(blocking) \
+#define MAY_BLOCK_THEN_RETURN_EVENT(ev,blocking,event) \
 	if ((blocking) == CL_TRUE) {\
-		cl_event ev = command_event_get(cmd);\
 		soclWaitForEvents(1, &ev);\
-		gc_entity_release(ev);\
-	}
-
-#define MAY_BLOCK_CUSTOM(blocking,event) \
-	if ((blocking) == CL_TRUE) {\
-		cl_event ev = (event);\
-		soclWaitForEvents(1, &ev);\
-		gc_entity_release(ev);\
-	}
+	}\
+   RETURN_EVENT(ev,event);\
 
 /* Constants */
-struct _cl_platform_id socl_platform;
-const char * SOCL_PROFILE;
-const char * SOCL_VERSION;
-const char * SOCL_PLATFORM_NAME;
-const char * SOCL_VENDOR;
-const char * SOCL_PLATFORM_EXTENSIONS;
+extern const char * SOCL_PROFILE;
+extern const char * SOCL_VERSION;
+extern const char * SOCL_PLATFORM_NAME;
+extern const char * SOCL_VENDOR;
+extern const char * SOCL_PLATFORM_EXTENSIONS;
+extern const char * SOCL_PLATFORM_ICD_SUFFIX_KHR;
 
 struct _cl_context {
   CL_ENTITY;
@@ -128,6 +120,9 @@ struct _cl_context {
   cl_device_id * devices;
   cl_uint num_devices;
 
+  /* Scheduling context */
+  unsigned sched_ctx;
+
   /* Properties */
   cl_context_properties * properties;
   cl_uint num_properties;
@@ -153,7 +148,7 @@ struct _cl_command_queue {
   cl_command barrier;
 
   /* Mutex */
-  pthread_mutex_t mutex;
+  starpu_pthread_mutex_t mutex;
 
   /* ID  */
 #ifdef DEBUG
@@ -178,8 +173,8 @@ struct _cl_event {
    */
   int id;
 
-  /* Profiling info are copied here */
-  struct starpu_task_profiling_info *profiling_info;
+  /* Profiling info */
+  cl_ulong prof_queued, prof_submit, prof_start, prof_end;
 };
 
 struct _cl_mem {
@@ -249,12 +244,17 @@ struct _cl_program {
 
 enum kernel_arg_type { Null, Buffer, Immediate };
 
+typedef cl_int (*split_func_t)(cl_command_queue, cl_uint, void *, const cl_event, cl_event *);
+
 struct _cl_kernel {
   CL_ENTITY;
 
   /* Associated program */
   cl_program program;
 
+  /* StarPU codelet */
+  struct starpu_perfmodel * perfmodel;
+
   /* Kernel name */
   char * kernel_name;
 
@@ -270,6 +270,13 @@ struct _cl_kernel {
   enum kernel_arg_type  *arg_type;
   void  **arg_value;
 
+  /* Partition function */
+  cl_uint split_space;
+  split_func_t split_func;
+  cl_ulong * split_perfs;
+  void * split_data;
+  starpu_pthread_mutex_t split_lock;
+
   /* ID  */
 #ifdef DEBUG
   int id;
@@ -282,7 +289,7 @@ struct _cl_kernel {
  * This allows us to disable StarPU profiling it
  * is equal to 0
  */
-int profiling_queue_count;
+extern int profiling_queue_count;
 
 /***************************************************************************/
 
@@ -738,6 +745,20 @@ soclEnqueueWaitForEvents(cl_command_queue /* command_queue */,
 extern CL_API_ENTRY cl_int CL_API_CALL
 soclEnqueueBarrier(cl_command_queue /* command_queue */) CL_API_SUFFIX__VERSION_1_0;
 
+extern CL_API_ENTRY cl_int soclEnqueueMarkerWithWaitList(
+    cl_command_queue /* command_queue */,
+    cl_uint           /* num_events_in_wait_list */,
+    const cl_event *  /* event_wait_list */,
+    cl_event *        /* event */
+  ) CL_API_SUFFIX__VERSION_1_2;
+
+extern CL_API_ENTRY cl_int soclEnqueueBarrierWithWaitList(
+    cl_command_queue /* command_queue */,
+    cl_uint           /* num_events_in_wait_list */,
+    const cl_event *  /* event_wait_list */,
+    cl_event *        /* event */
+  ) CL_API_SUFFIX__VERSION_1_2;
+
 /* Extension function access
  *
  * Returns the extension function address for the given function name,
@@ -748,4 +769,18 @@ soclEnqueueBarrier(cl_command_queue /* command_queue */) CL_API_SUFFIX__VERSION_
 extern CL_API_ENTRY void * CL_API_CALL
 soclGetExtensionFunctionAddress(const char * /* func_name */) CL_API_SUFFIX__VERSION_1_0;
 
+extern void * CL_API_CALL
+soclGetExtensionFunctionAddressForPlatform(cl_platform_id p, const char * func_name) CL_API_SUFFIX__VERSION_1_2;
+
+extern CL_API_ENTRY cl_int CL_API_CALL
+soclIcdGetPlatformIDsKHR(cl_uint          /* num_entries */,
+                 cl_platform_id * /* platforms */,
+                 cl_uint *        /* num_platforms */) CL_EXT_SUFFIX__VERSION_1_0;
+
+
+extern struct _cl_icd_dispatch socl_master_dispatch;
+extern struct _cl_platform_id socl_platform;
+extern struct _cl_device_id * socl_devices;
+extern unsigned int socl_device_count;
+
 #endif /* SOCL_H */
