@@ -1,6 +1,7 @@
 /* StarPU --- Runtime system for heterogeneous multicore architectures.
  *
  * Copyright (C) 2010,2011 University of Bordeaux
+ * Copyright (C) 2016  CNRS
  *
  * StarPU is free software; you can redistribute it and/or modify
  * it under the terms of the GNU Lesser General Public License as published by
@@ -15,9 +16,10 @@
  */
 
 #include "socl.h"
+#include "common/timing.h"
 
-int starpu_worker_get_range() {
-   int id = starpu_worker_get_id();
+
+int starpu_worker_get_range_by_id(int id) {
    int i, oid = 0;
    for (i=0; i<id; i++)
       if (starpu_worker_get_type(i) == STARPU_OPENCL_WORKER) oid++;
@@ -25,6 +27,11 @@ int starpu_worker_get_range() {
    return oid;
 }
 
+int starpu_worker_get_range() {
+   int id = starpu_worker_get_id_check();
+   return starpu_worker_get_range_by_id(id);
+}
+
 void * memdupa(const void *p, size_t size) {
 	void * s = malloc(size);
 	memcpy(s,p,size);
@@ -48,3 +55,9 @@ void ** memdup_deep_varsize_safea(const void **p, unsigned n, size_t * size) {
 	}
 	return s;
 }
+
+cl_ulong _socl_nanotime() {
+   struct timespec ts;
+   _starpu_clock_gettime(&ts);
+   return (ts.tv_sec * 1e9 + ts.tv_nsec);
+}
