@@ -1,7 +1,8 @@
 /* StarPU --- Runtime system for heterogeneous multicore architectures.
  *
- * Copyright (C) 2010-2012,2014  Université de Bordeaux 1
- * Copyright (C) 2010, 2011, 2012, 2013  Centre National de la Recherche Scientifique
+ * Copyright (C) 2010-2015  Université de Bordeaux
+ * Copyright (C) 2010, 2011, 2012, 2013, 2017  CNRS
+ * Copyright (C) 2015  Inria
  *
  * StarPU is free software; you can redistribute it and/or modify
  * it under the terms of the GNU Lesser General Public License as published by
@@ -20,31 +21,60 @@
 #include <core/sched_policy.h>
 #include <common/starpu_spinlock.h>
 #include <datawizard/sort_data_handles.h>
+#include <datawizard/memory_nodes.h>
 
 /*
- * The different types of data accesses are STARPU_R, STARPU_RW, STARPU_W,
+ * We have a kind of dining philosophers problem: various tasks are accessing
+ * various data concurrently in different modes: STARPU_R, STARPU_RW, STARPU_W,
  * STARPU_SCRATCH and STARPU_REDUX. STARPU_RW is managed as a STARPU_W access.
+ * We have the following constraints:
+ *
  * - A single STARPU_W access is allowed at a time.
  * - Concurrent STARPU_R accesses are allowed.
  * - Concurrent STARPU_SCRATCH accesses are allowed.
  * - Concurrent STARPU_REDUX accesses are allowed.
+ *
+ * What we do here is implementing the Dijkstra solutions: handles are sorted
+ * by pointer value order, and tasks call
+ * _starpu_attempt_to_submit_data_request for each requested data in that order
+ * (see _starpu_sort_task_handles call in _starpu_submit_job_enforce_data_deps).
+ *
+ * _starpu_attempt_to_submit_data_request will either:
+ * - obtain access to the data, and thus the task can proceed with acquiring
+ *   other data (see _submit_job_enforce_data_deps)
+ * - queue a request on the data handle
+ *
+ * When a task finishes, it calls _starpu_notify_data_dependencies for each
+ * data, to free its acquisitions. This will look whether the first queued
+ * request can be fulfilled, and in such case make the task try to acquire its
+ * next data.
+ *
+ * The same mechanism is used for application data aquisition
+ * (starpu_data_acquire).
+ *
+ * For data with an arbiter, we have a second step, performed after this first
+ * step, implemented in data_arbiter_concurrency.c
  */
 
-/* the header lock must be taken by the caller */
+/*
+ * Check to see whether the first queued request can proceed, and return it in
+ * such case.
+ */
+/* the handle header lock must be taken by the caller */
 static struct _starpu_data_requester *may_unlock_data_req_list_head(starpu_data_handle_t handle)
 {
 	struct _starpu_data_requester_list *req_list;
 
 	if (handle->reduction_refcnt > 0)
 	{
-		req_list = handle->reduction_req_list;
+		req_list = &handle->reduction_req_list;
 	}
 	else
 	{
-		if (_starpu_data_requester_list_empty(handle->reduction_req_list))
-			req_list = handle->req_list;
+		if (_starpu_data_requester_list_empty(&handle->reduction_req_list))
+			req_list = &handle->req_list;
 		else
-			req_list = handle->reduction_req_list;
+			req_list = &handle->reduction_req_list;
 	}
 
 	/* if there is no one to unlock ... */
@@ -55,6 +85,7 @@ static struct _starpu_data_requester *may_unlock_data_req_list_head(starpu_data_
 	if (handle->refcnt == 0)
 		return _starpu_data_requester_list_pop_front(req_list);
 
+	/* Already writing to it, do not let another write access through */
 	if (handle->current_mode == STARPU_W)
 		return NULL;
 
@@ -66,7 +97,7 @@ static struct _starpu_data_requester *may_unlock_data_req_list_head(starpu_data_
 		r_mode = STARPU_W;
 
 	/* If this is a STARPU_R, STARPU_SCRATCH or STARPU_REDUX type of
-	 * access, we only proceed if the cuurrent mode is the same as the
+	 * access, we only proceed if the current mode is the same as the
 	 * requested mode. */
 	if (r_mode == handle->current_mode)
 		return _starpu_data_requester_list_pop_front(req_list);
@@ -78,11 +109,15 @@ static struct _starpu_data_requester *may_unlock_data_req_list_head(starpu_data_
  * immediatly, return 0, if there is still a dependency that is not compatible
  * with the current mode, the request is put in the per-handle list of
  * "requesters", and this function returns 1. */
+/* No lock is held, this acquires and releases the handle header lock */
 static unsigned _starpu_attempt_to_submit_data_request(unsigned request_from_codelet,
 						       starpu_data_handle_t handle, enum starpu_data_access_mode mode,
 						       void (*callback)(void *), void *argcb,
 						       struct _starpu_job *j, unsigned buffer_index)
 {
+	if (handle->arbiter)
+		return _starpu_attempt_to_submit_arbitered_data_request(request_from_codelet, handle, mode, callback, argcb, j, buffer_index);
+
 	if (mode == STARPU_RW)
 		mode = STARPU_W;
 
@@ -162,7 +197,7 @@ static unsigned _starpu_attempt_to_submit_data_request(unsigned request_from_cod
 
 		/* We put the requester in a specific list if this is a reduction task */
 		struct _starpu_data_requester_list *req_list =
-			is_a_reduction_task?handle->reduction_req_list:handle->req_list;
+			is_a_reduction_task?&handle->reduction_req_list:&handle->req_list;
 
 		_starpu_data_requester_list_push_back(req_list, r);
 
@@ -174,7 +209,20 @@ static unsigned _starpu_attempt_to_submit_data_request(unsigned request_from_cod
 		handle->refcnt++;
 		handle->busy_count++;
 
-		handle->current_mode = mode;
+		/* Do not write to handle->current_mode if it is already
+		 * R. This avoids a spurious warning from helgrind when
+		 * the following happens:
+		 * acquire(R) in thread A
+		 * acquire(R) in thread B
+		 * release_data_on_node() in thread A
+		 * helgrind would shout that the latter reads current_mode
+		 * unsafely.
+		 *
+		 * This actually basically explains helgrind that it is a
+		 * shared R acquisition.
+		 */
+		if (mode != STARPU_R || handle->current_mode != mode)
+			handle->current_mode = mode;
 
 		if ((mode == STARPU_REDUX) && (previous_mode != STARPU_REDUX))
 			_starpu_data_start_reduction_mode(handle);
@@ -188,33 +236,38 @@ static unsigned _starpu_attempt_to_submit_data_request(unsigned request_from_cod
 
 }
 
+/* No lock is held */
 unsigned _starpu_attempt_to_submit_data_request_from_apps(starpu_data_handle_t handle, enum starpu_data_access_mode mode,
 							  void (*callback)(void *), void *argcb)
 {
 	return _starpu_attempt_to_submit_data_request(0, handle, mode, callback, argcb, NULL, 0);
 }
 
+/* No lock is held */
 static unsigned attempt_to_submit_data_request_from_job(struct _starpu_job *j, unsigned buffer_index)
 {
 	/* Note that we do not access j->task->handles, but j->ordered_buffers
 	 * which is a sorted copy of it. */
 	starpu_data_handle_t handle = _STARPU_JOB_GET_ORDERED_BUFFER_HANDLE(j, buffer_index);
-	enum starpu_data_access_mode mode = _STARPU_JOB_GET_ORDERED_BUFFER_MODE(j, buffer_index);
+	enum starpu_data_access_mode mode = _STARPU_JOB_GET_ORDERED_BUFFER_MODE(j, buffer_index) & ~STARPU_COMMUTE;
 
 	return _starpu_attempt_to_submit_data_request(1, handle, mode, NULL, NULL, j, buffer_index);
 }
 
+/* Acquire all data of the given job, one by one in handle pointer value order
+ */
+/* No lock is held */
 static unsigned _submit_job_enforce_data_deps(struct _starpu_job *j, unsigned start_buffer_index)
 {
 	unsigned buf;
 
-	unsigned nbuffers = j->task->cl->nbuffers;
+	unsigned nbuffers = STARPU_TASK_GET_NBUFFERS(j->task);
 	for (buf = start_buffer_index; buf < nbuffers; buf++)
 	{
+		starpu_data_handle_t handle = _STARPU_JOB_GET_ORDERED_BUFFER_HANDLE(j, buf);
 		if (buf)
 		{
 			starpu_data_handle_t handle_m1 = _STARPU_JOB_GET_ORDERED_BUFFER_HANDLE(j, buf-1);
-			starpu_data_handle_t handle = _STARPU_JOB_GET_ORDERED_BUFFER_HANDLE(j, buf);
 			if (handle_m1 == handle)
 				/* We have already requested this data, skip it. This
 				 * depends on ordering putting writes before reads, see
@@ -223,6 +276,15 @@ static unsigned _submit_job_enforce_data_deps(struct _starpu_job *j, unsigned st
 		}
 
                 j->task->status = STARPU_TASK_BLOCKED_ON_DATA;
+
+		if(handle->arbiter)
+		{
+			/* We arrived on an arbitered data, we stop and proceed
+			 * with the arbiter second step.  */
+			_starpu_submit_job_enforce_arbitered_deps(j, buf, nbuffers);
+			return 1;
+		}
+
                 if (attempt_to_submit_data_request_from_job(j, buf))
 		{
 			return 1;
@@ -232,42 +294,51 @@ static unsigned _submit_job_enforce_data_deps(struct _starpu_job *j, unsigned st
 	return 0;
 }
 
-/* When a new task is submitted, we make sure that there cannot be codelets
-   with concurrent data-access at the same time in the scheduling engine (eg.
-   there can be 2 tasks reading a piece of data, but there cannot be one
-   reading and another writing) */
-unsigned _starpu_submit_job_enforce_data_deps(struct _starpu_job *j)
+void _starpu_job_set_ordered_buffers(struct _starpu_job *j)
 {
-	struct starpu_codelet *cl = j->task->cl;
-
-	if ((cl == NULL) || (cl->nbuffers == 0))
-		return 0;
-
 	/* Compute an ordered list of the different pieces of data so that we
 	 * grab then according to a total order, thus avoiding a deadlock
 	 * condition */
 	unsigned i;
-	for (i=0 ; i<cl->nbuffers ; i++)
+	unsigned nbuffers = STARPU_TASK_GET_NBUFFERS(j->task);
+	struct starpu_task *task = j->task;
+
+	for (i=0 ; i<nbuffers; i++)
 	{
-		starpu_data_handle_t handle = STARPU_TASK_GET_HANDLE(j->task, i);
+		starpu_data_handle_t handle = STARPU_TASK_GET_HANDLE(task, i);
 		_STARPU_JOB_SET_ORDERED_BUFFER_HANDLE(j, handle, i);
-		enum starpu_data_access_mode mode = STARPU_CODELET_GET_MODE(j->task->cl, i);
+		enum starpu_data_access_mode mode = STARPU_TASK_GET_MODE(task, i);
 		_STARPU_JOB_SET_ORDERED_BUFFER_MODE(j, mode, i);
 		int node = -1;
-		if (j->task->cl->specific_nodes)
-			node = STARPU_CODELET_GET_NODE(j->task->cl, i);
+		if (task->cl->specific_nodes)
+			node = STARPU_CODELET_GET_NODE(task->cl, i);
 		_STARPU_JOB_SET_ORDERED_BUFFER_NODE(j, node, i);
 	}
+	_starpu_sort_task_handles(_STARPU_JOB_GET_ORDERED_BUFFERS(j), nbuffers);
+}
 
-	_starpu_sort_task_handles(_STARPU_JOB_GET_ORDERED_BUFFERS(j), cl->nbuffers);
+/* Sort the data used by the given job by handle pointer value order, and
+ * acquire them in that order */
+/* No  lock is held */
+unsigned _starpu_submit_job_enforce_data_deps(struct _starpu_job *j)
+{
+	struct starpu_codelet *cl = j->task->cl;
+
+	if ((cl == NULL) || (STARPU_TASK_GET_NBUFFERS(j->task) == 0))
+		return 0;
+
+	_starpu_job_set_ordered_buffers(j);
 
 	return _submit_job_enforce_data_deps(j, 0);
 }
 
+/* This request got fulfilled, continue with the other requests of the
+ * corresponding job */
+/* No lock is held */
 static unsigned unlock_one_requester(struct _starpu_data_requester *r)
 {
 	struct _starpu_job *j = r->j;
-	unsigned nbuffers = j->task->cl->nbuffers;
+	unsigned nbuffers = STARPU_TASK_GET_NBUFFERS(j->task);
 	unsigned buffer_index = r->buffer_index;
 
 	if (buffer_index + 1 < nbuffers)
@@ -277,7 +348,10 @@ static unsigned unlock_one_requester(struct _starpu_data_requester *r)
 		return 0;
 }
 
-/* The header lock must already be taken by the caller.
+/* This is called when a task is finished with a piece of data
+ * (or on starpu_data_release)
+ *
+ * The header lock must already be taken by the caller.
  * This may free the handle if it was lazily unregistered (1 is returned in
  * that case). The handle pointer thus becomes invalid for the caller.
  */
@@ -293,6 +367,20 @@ int _starpu_notify_data_dependencies(starpu_data_handle_t handle)
 		/* Handle was destroyed, nothing left to do.  */
 		return 1;
 
+	if (handle->arbiter)
+	{
+		unsigned refcnt = handle->refcnt;
+		STARPU_ASSERT(_starpu_data_requester_list_empty(&handle->req_list));
+		STARPU_ASSERT(_starpu_data_requester_list_empty(&handle->reduction_req_list));
+		_starpu_spin_unlock(&handle->header_lock);
+		/* _starpu_notify_arbitered_dependencies will handle its own locking */
+		if (!refcnt)
+			_starpu_notify_arbitered_dependencies(handle);
+		/* We have already unlocked */
+		return 1;
+	}
+	STARPU_ASSERT(_starpu_data_requester_list_empty(&handle->arbitered_req_list));
+
 	/* In case there is a pending reduction, and that this is the last
 	 * requester, we may go back to a "normal" coherency model. */
 	if (handle->reduction_refcnt > 0)
@@ -303,6 +391,17 @@ int _starpu_notify_data_dependencies(starpu_data_handle_t handle)
 			_starpu_data_end_reduction_mode_terminate(handle);
 	}
 
+	if (handle->unlocking_reqs)
+		/*
+		 * Our caller is already running the unlock loop below (we were
+		 * most probably called from the ready_data_callback call
+		 * below). Avoid looping again (which would potentially mean
+		 * unbounded recursion), our caller will continue doing the
+		 * unlock work for us.
+		 */
+		return 0;
+
+	handle->unlocking_reqs = 1;
 	struct _starpu_data_requester *r;
 	while ((r = may_unlock_data_req_list_head(handle)))
 	{
@@ -329,7 +428,7 @@ int _starpu_notify_data_dependencies(starpu_data_handle_t handle)
 		{
 			/* We need to put the request back because we must
 			 * perform a reduction before. */
-			_starpu_data_requester_list_push_front(handle->req_list, r);
+			_starpu_data_requester_list_push_front(&handle->req_list, r);
 		}
 		else
 		{
@@ -372,6 +471,7 @@ int _starpu_notify_data_dependencies(starpu_data_handle_t handle)
 				return 1;
 		}
 	}
+	handle->unlocking_reqs = 0;
 
 	return 0;
 }
