@@ -812,7 +812,7 @@ void _starpu_release_data_on_node(starpu_data_handle_t handle, uint32_t default_
 	unsigned memory_node = replicate->memory_node;
 
 	if (replicate->state != STARPU_INVALID && handle->current_mode & STARPU_W)
-	if ((wt_mask & ~(1<<memory_node)))
+	if (wt_mask & ~(1<<memory_node))
 		_starpu_write_through_data(handle, memory_node, wt_mask);
 
 	unsigned local_node = _starpu_memory_node_get_local_key();
@@ -871,6 +871,10 @@ int starpu_prefetch_task_input_on_node(struct starpu_task *task, unsigned node)
 		if (mode & (STARPU_SCRATCH|STARPU_REDUX))
 			continue;
 
+		if (!(mode & STARPU_R))
+			/* Don't bother prefetching some data which will be overwritten */
+			continue;
+
 		struct _starpu_data_replicate *replicate = &handle->per_node[node];
 		prefetch_data_on_node(handle, node, replicate, mode);
 
