@@ -2,7 +2,7 @@
  *
  * Copyright (C) 2009-2016  Université de Bordeaux
  * Copyright (C) 2010  Mehdi Juhoor <mjuhoor@gmail.com>
- * Copyright (C) 2010, 2011, 2012, 2013, 2014  CNRS
+ * Copyright (C) 2010, 2011, 2012, 2013, 2014, 2017  CNRS
  *
  * StarPU is free software; you can redistribute it and/or modify
  * it under the terms of the GNU Lesser General Public License as published by
@@ -45,6 +45,8 @@ static size_t csr_interface_get_size(starpu_data_handle_t handle);
 static int csr_compare(void *data_interface_a, void *data_interface_b);
 static uint32_t footprint_csr_interface_crc32(starpu_data_handle_t handle);
 static starpu_ssize_t describe(void *data_interface, char *buf, size_t size);
+static int pack_data(starpu_data_handle_t handle, unsigned node, void **ptr, starpu_ssize_t *count);
+static int unpack_data(starpu_data_handle_t handle, unsigned node, void *ptr, size_t count);
 
 struct starpu_data_interface_ops starpu_interface_csr_ops =
 {
@@ -57,7 +59,10 @@ struct starpu_data_interface_ops starpu_interface_csr_ops =
 	.interface_size = sizeof(struct starpu_csr_interface),
 	.footprint = footprint_csr_interface_crc32,
 	.compare = csr_compare,
-	.describe = describe
+	.describe = describe,
+	.name = "STARPU_CSR_INTERFACE",
+	.pack_data = pack_data,
+	.unpack_data = unpack_data
 };
 
 static void register_csr_handle(starpu_data_handle_t handle, unsigned home_node, void *data_interface)
@@ -132,9 +137,9 @@ static int csr_compare(void *data_interface_a, void *data_interface_b)
 	struct starpu_csr_interface *csr_b = (struct starpu_csr_interface *) data_interface_b;
 
 	/* Two matricess are considered compatible if they have the same size */
-	return ((csr_a->nnz == csr_b->nnz)
-			&& (csr_a->nrow == csr_b->nrow)
-			&& (csr_a->elemsize == csr_b->elemsize));
+	return (csr_a->nnz == csr_b->nnz)
+		&& (csr_a->nrow == csr_b->nrow)
+		&& (csr_a->elemsize == csr_b->elemsize);
 }
 
 /* offer an access to the data parameters */
@@ -143,6 +148,10 @@ uint32_t starpu_csr_get_nnz(starpu_data_handle_t handle)
 	struct starpu_csr_interface *csr_interface = (struct starpu_csr_interface *)
 		starpu_data_get_interface_on_node(handle, STARPU_MAIN_RAM);
 
+#ifdef STARPU_DEBUG
+	STARPU_ASSERT_MSG(csr_interface->id == STARPU_CSR_INTERFACE_ID, "Error. The given data is not a csr.");
+#endif
+
 	return csr_interface->nnz;
 }
 
@@ -151,6 +160,10 @@ uint32_t starpu_csr_get_nrow(starpu_data_handle_t handle)
 	struct starpu_csr_interface *csr_interface = (struct starpu_csr_interface *)
 		starpu_data_get_interface_on_node(handle, STARPU_MAIN_RAM);
 
+#ifdef STARPU_DEBUG
+	STARPU_ASSERT_MSG(csr_interface->id == STARPU_CSR_INTERFACE_ID, "Error. The given data is not a csr.");
+#endif
+
 	return csr_interface->nrow;
 }
 
@@ -159,6 +172,10 @@ uint32_t starpu_csr_get_firstentry(starpu_data_handle_t handle)
 	struct starpu_csr_interface *csr_interface = (struct starpu_csr_interface *)
 		starpu_data_get_interface_on_node(handle, STARPU_MAIN_RAM);
 
+#ifdef STARPU_DEBUG
+	STARPU_ASSERT_MSG(csr_interface->id == STARPU_CSR_INTERFACE_ID, "Error. The given data is not a csr.");
+#endif
+
 	return csr_interface->firstentry;
 }
 
@@ -167,6 +184,10 @@ size_t starpu_csr_get_elemsize(starpu_data_handle_t handle)
 	struct starpu_csr_interface *csr_interface = (struct starpu_csr_interface *)
 		starpu_data_get_interface_on_node(handle, STARPU_MAIN_RAM);
 
+#ifdef STARPU_DEBUG
+	STARPU_ASSERT_MSG(csr_interface->id == STARPU_CSR_INTERFACE_ID, "Error. The given data is not a csr.");
+#endif
+
 	return csr_interface->elemsize;
 }
 
@@ -180,6 +201,10 @@ uintptr_t starpu_csr_get_local_nzval(starpu_data_handle_t handle)
 	struct starpu_csr_interface *csr_interface = (struct starpu_csr_interface *)
 		starpu_data_get_interface_on_node(handle, node);
 
+#ifdef STARPU_DEBUG
+	STARPU_ASSERT_MSG(csr_interface->id == STARPU_CSR_INTERFACE_ID, "Error. The given data is not a csr.");
+#endif
+
 	return csr_interface->nzval;
 }
 
@@ -193,6 +218,10 @@ uint32_t *starpu_csr_get_local_colind(starpu_data_handle_t handle)
 	struct starpu_csr_interface *csr_interface = (struct starpu_csr_interface *)
 		starpu_data_get_interface_on_node(handle, node);
 
+#ifdef STARPU_DEBUG
+	STARPU_ASSERT_MSG(csr_interface->id == STARPU_CSR_INTERFACE_ID, "Error. The given data is not a csr.");
+#endif
+
 	return csr_interface->colind;
 }
 
@@ -206,6 +235,10 @@ uint32_t *starpu_csr_get_local_rowptr(starpu_data_handle_t handle)
 	struct starpu_csr_interface *csr_interface = (struct starpu_csr_interface *)
 		starpu_data_get_interface_on_node(handle, node);
 
+#ifdef STARPU_DEBUG
+	STARPU_ASSERT_MSG(csr_interface->id == STARPU_CSR_INTERFACE_ID, "Error. The given data is not a csr.");
+#endif
+
 	return csr_interface->rowptr;
 }
 
@@ -313,3 +346,47 @@ static starpu_ssize_t describe(void *data_interface, char *buf, size_t size)
 			(unsigned) csr->nrow,
 			(unsigned) csr->elemsize);
 }
+
+static int pack_data(starpu_data_handle_t handle, unsigned node, void **ptr, starpu_ssize_t *count)
+{
+	STARPU_ASSERT(starpu_data_test_if_allocated_on_node(handle, node));
+
+	struct starpu_csr_interface *csr = (struct starpu_csr_interface *) starpu_data_get_interface_on_node(handle, node);
+
+	// We first pack colind
+	*count = csr->nnz * sizeof(csr->colind[0]);
+	// Then rowptr
+	*count += (csr->nrow + 1) * sizeof(csr->rowptr[0]);
+	// Then nnzval
+	*count += csr->nnz * csr->elemsize;
+
+	if (ptr != NULL)
+	{
+		starpu_malloc_flags(ptr, *count, 0);
+		char *tmp = *ptr;
+		memcpy(tmp, (void*)csr->colind, csr->nnz * sizeof(csr->colind[0]));
+		tmp += csr->nnz * sizeof(csr->colind[0]);
+		memcpy(tmp, (void*)csr->rowptr, (csr->nrow + 1) * sizeof(csr->rowptr[0]));
+		tmp += (csr->nrow + 1) * sizeof(csr->rowptr[0]);
+		memcpy(tmp, (void*)csr->nzval, csr->nnz * csr->elemsize);
+	}
+
+	return 0;
+}
+
+static int unpack_data(starpu_data_handle_t handle, unsigned node, void *ptr, size_t count)
+{
+	STARPU_ASSERT(starpu_data_test_if_allocated_on_node(handle, node));
+
+	struct starpu_csr_interface *csr = (struct starpu_csr_interface *) starpu_data_get_interface_on_node(handle, node);
+
+	STARPU_ASSERT(count == (csr->nnz * sizeof(csr->colind[0]))+((csr->nrow + 1) * sizeof(csr->rowptr[0]))+(csr->nnz * csr->elemsize));
+
+	char *tmp = ptr;
+	memcpy((void*)csr->colind, tmp, csr->nnz * sizeof(csr->colind[0]));
+	tmp += csr->nnz * sizeof(csr->colind[0]);
+	memcpy((void*)csr->rowptr, tmp, (csr->nrow + 1) * sizeof(csr->rowptr[0]));
+	tmp += (csr->nrow + 1) * sizeof(csr->rowptr[0]);
+	memcpy((void*)csr->nzval, tmp, csr->nnz * csr->elemsize);
+	return 0;
+}
