@@ -37,9 +37,7 @@ static unsigned mp_node_memory_node(struct _starpu_mp_node *node)
 static int _starpu_src_common_finalize_job (struct _starpu_job *j, struct _starpu_worker *worker)
 {
 	int profiling = starpu_profiling_status_get();
-	struct timespec codelet_end;
-	_starpu_driver_end_job(worker, j, &worker->perf_arch, &codelet_end, 0,
-			profiling);
+	_starpu_driver_end_job(worker, j, &worker->perf_arch, 0, profiling);
 
 	int count = worker->current_rank;
 
@@ -47,6 +45,7 @@ static int _starpu_src_common_finalize_job (struct _starpu_job *j, struct _starp
 	if(j->task_size > 1)
 	{
 		struct _starpu_combined_worker * cb_worker = _starpu_get_combined_worker_struct(worker->combined_workerid);
+		(void) STARPU_ATOMIC_ADD(&j->after_work_busy_barrier, -1);
 
 		STARPU_PTHREAD_MUTEX_LOCK(&cb_worker->count_mutex);
 		count = cb_worker->count--;
@@ -60,7 +59,6 @@ static int _starpu_src_common_finalize_job (struct _starpu_job *j, struct _starp
 	{
 
 		_starpu_driver_update_job_feedback(j, worker, &worker->perf_arch,
-				&j->cl_start, &codelet_end,
 				profiling);
 
 		_starpu_push_task_output (j);
@@ -159,14 +157,13 @@ static void _starpu_src_common_handle_stored_async(struct _starpu_mp_node *node)
  * return 1 if the message has been stored
  * return 0 if the message is unknown or synchrone */
 int _starpu_src_common_store_message(struct _starpu_mp_node *node,
-		void * arg, int arg_size, enum _starpu_mp_command answer)
+				     void * arg, int arg_size, enum _starpu_mp_command answer)
 {
-	struct mp_message * message = NULL;
 	switch(answer)
 	{
 		case STARPU_EXECUTION_COMPLETED:
 		case STARPU_PRE_EXECUTION:
-			message = mp_message_new();
+			struct mp_message *message = mp_message_new();
 			message->type = answer;
 			_STARPU_MALLOC(message->buffer, arg_size);
 			memcpy(message->buffer, arg, arg_size);
@@ -433,7 +430,7 @@ static int _starpu_src_common_execute(struct _starpu_job *j,
 
 	void (*kernel)(void)  = node->get_kernel_from_job(node,j);
 
-	_starpu_driver_start_job(worker, j, &worker->perf_arch, &j->cl_start, 0, profiling);
+	_starpu_driver_start_job(worker, j, &worker->perf_arch, 0, profiling);
 
 	//_STARPU_DEBUG("\nworkerid:%d, rank:%d, type:%d,	cb_workerid:%d, task_size:%d\n\n",worker->devid,worker->current_rank,task->cl->type,j->combined_workerid,j->task_size);
 
@@ -545,37 +542,35 @@ int _starpu_src_common_copy_sink_to_sink(const struct _starpu_mp_node *src_node,
 /* 5 functions to determine the executable to run on the device (MIC, SCC,
  * MPI).
  */
-static void _starpu_src_common_cat_3(char *final, const char *first,
-		const char *second, const char *third)
+static void _starpu_src_common_cat_3(char *final, const size_t len, const char *first,
+				     const char *second, const char *third)
 {
-	strcpy(final, first);
-	strcat(final, second);
-	strcat(final, third);
+	snprintf(final, len, "%s%s%s", first, second, third);
 }
 
-static void _starpu_src_common_cat_2(char *final, const char *first, const char *second)
+static void _starpu_src_common_cat_2(char *final, const size_t len, const char *first, const char *second)
 {
-	_starpu_src_common_cat_3(final, first, second, "");
+	_starpu_src_common_cat_3(final, len, first, second, "");
 }
 
-static void _starpu_src_common_dir_cat(char *final, const char *dir, const char *file)
+static void _starpu_src_common_dir_cat(char *final, const size_t len, const char *dir, const char *file)
 {
 	if (file[0] == '/')
 		++file;
 
 	size_t size = strlen(dir);
 	if (dir[size - 1] == '/')
-		_starpu_src_common_cat_2(final, dir, file);
+		_starpu_src_common_cat_2(final, len, dir, file);
 	else
-		_starpu_src_common_cat_3(final, dir, "/", file);
+		_starpu_src_common_cat_3(final, len, dir, "/", file);
 }
 
-static int _starpu_src_common_test_suffixes(char *located_file_name, const char *base, const char **suffixes)
+static int _starpu_src_common_test_suffixes(char *located_file_name, const size_t len, const char *base, const char **suffixes)
 {
 	unsigned int i;
 	for (i = 0; suffixes[i] != NULL; ++i)
 	{
-		_starpu_src_common_cat_2(located_file_name, base, suffixes[i]);
+		_starpu_src_common_cat_2(located_file_name, len, base, suffixes[i]);
 		if (access(located_file_name, R_OK) == 0)
 			return 0;
 	}
@@ -583,21 +578,21 @@ static int _starpu_src_common_test_suffixes(char *located_file_name, const char
 	return 1;
 }
 
-int _starpu_src_common_locate_file(char *located_file_name,
-		const char *env_file_name, const char *env_mic_path,
-		const char *config_file_name, const char *actual_file_name,
-		const char **suffixes)
+int _starpu_src_common_locate_file(char *located_file_name, size_t len,
+				   const char *env_file_name, const char *env_mic_path,
+				   const char *config_file_name, const char *actual_file_name,
+				   const char **suffixes)
 {
 	if (env_file_name != NULL)
 	{
 		if (access(env_file_name, R_OK) == 0)
 		{
-			strcpy(located_file_name, env_file_name);
+			strncpy(located_file_name, env_file_name, len);
 			return 0;
 		}
 		else if(env_mic_path != NULL)
 		{
-			_starpu_src_common_dir_cat(located_file_name, env_mic_path, env_file_name);
+			_starpu_src_common_dir_cat(located_file_name, len, env_mic_path, env_file_name);
 
 			return access(located_file_name, R_OK);
 		}
@@ -606,40 +601,40 @@ int _starpu_src_common_locate_file(char *located_file_name,
 	{
 		if (access(config_file_name, R_OK) == 0)
 		{
-			strcpy(located_file_name, config_file_name);
+			strncpy(located_file_name, config_file_name, len);
 			return 0;
 		}
 		else if (env_mic_path != NULL)
 		{
-			_starpu_src_common_dir_cat(located_file_name, env_mic_path, config_file_name);
+			_starpu_src_common_dir_cat(located_file_name, len, env_mic_path, config_file_name);
 
 			return access(located_file_name, R_OK);
 		}
 	}
 	else if (actual_file_name != NULL)
 	{
-		if (_starpu_src_common_test_suffixes(located_file_name, actual_file_name, suffixes) == 0)
+		if (_starpu_src_common_test_suffixes(located_file_name, len, actual_file_name, suffixes) == 0)
 			return 0;
 
 		if (env_mic_path != NULL)
 		{
 			char actual_cpy[1024];
-			strcpy(actual_cpy, actual_file_name);
+			strncpy(actual_cpy, actual_file_name, sizeof(actual_cpy));
 
 			char *last =  strrchr(actual_cpy, '/');
 			while (last != NULL)
 			{
 				char tmp[1024];
 
-				_starpu_src_common_dir_cat(tmp, env_mic_path, last);
+				_starpu_src_common_dir_cat(tmp, sizeof(tmp), env_mic_path, last);
 
 				if (access(tmp, R_OK) == 0)
 				{
-					strcpy(located_file_name, tmp);
+					strncpy(located_file_name, tmp, len);
 					return 0;
 				}
 
-				if (_starpu_src_common_test_suffixes(located_file_name, tmp, suffixes) == 0)
+				if (_starpu_src_common_test_suffixes(located_file_name, len, tmp, suffixes) == 0)
 					return 0;
 
 				*last = '\0';
