@@ -167,9 +167,7 @@ char *_starpu_opencl_load_program_source(const char *filename)
 	}
 	source[x] = '\0';
 
-#ifdef STARPU_EXTRA_VERBOSE
-	_STARPU_DEBUG("OpenCL kernel <%s>\n", source);
-#endif
+	_STARPU_EXTRA_DEBUG("OpenCL kernel <%s>\n", source);
 
 	fclose(fh);
 
