@@ -1,7 +1,7 @@
 /* StarPU --- Runtime system for heterogeneous multicore architectures.
  *
- * Copyright (C) 2010-2013  Université de Bordeaux 1
- * Copyright (C) 2010, 2011, 2013  Centre National de la Recherche Scientifique
+ * Copyright (C) 2010-2016  Université de Bordeaux
+ * Copyright (C) 2010, 2011, 2013, 2017  CNRS
  * Copyright (C) 2011  Télécom-SudParis
  *
  * StarPU is free software; you can redistribute it and/or modify
@@ -22,11 +22,29 @@
 
 #include <sched_policies/fifo_queues.h>
 #include <common/fxt.h>
+/*
+static int is_sorted_task_list(struct starpu_task * task)
+{
+	if(!task)
+		return 1;
+	struct starpu_task * next = task->next;
+	if(!next)
+		return 1;
+	while(next)
+	{
+		if(task->priority < next->priority)
+			return 0;
+		task = next;
+		next = next->next;
+	}
+	return 1;
+}
+*/
 
 struct _starpu_fifo_taskq *_starpu_create_fifo(void)
 {
 	struct _starpu_fifo_taskq *fifo;
-	fifo = (struct _starpu_fifo_taskq *) malloc(sizeof(struct _starpu_fifo_taskq));
+	_STARPU_MALLOC(fifo, sizeof(struct _starpu_fifo_taskq));
 
 	/* note that not all mechanisms (eg. the semaphore) have to be used */
 	starpu_task_list_init(&fifo->taskq);
@@ -37,6 +55,7 @@ struct _starpu_fifo_taskq *_starpu_create_fifo(void)
 	fifo->exp_start = starpu_timing_now();
 	fifo->exp_len = 0.0;
 	fifo->exp_end = fifo->exp_start;
+	fifo->exp_len_per_priority = NULL;
 
 	return fifo;
 }
@@ -51,6 +70,60 @@ int _starpu_fifo_empty(struct _starpu_fifo_taskq *fifo)
 	return fifo->ntasks == 0;
 }
 
+double 
+_starpu_fifo_get_exp_len_prev_task_list(struct _starpu_fifo_taskq *fifo_queue, struct starpu_task *task, int workerid, int nimpl, int *fifo_ntasks)
+{
+	struct starpu_task_list *list = &fifo_queue->taskq;
+	struct starpu_perfmodel_arch* perf_arch = starpu_worker_get_perf_archtype(workerid, task->sched_ctx);
+	double exp_len = 0.0;
+	
+	if (list->head != NULL)
+	{
+		struct starpu_task *current = list->head;
+		struct starpu_task *prev = NULL;
+
+		if (list->head->priority == task->priority &&
+		    list->head->priority == list->tail->priority)
+		{
+			/* They all have the same priority, the task's place is at the end */
+			prev = list->tail;
+			current = NULL;
+		}
+		else
+		while (current)
+		{
+			if (current->priority < task->priority)
+				break;
+
+			prev = current;
+			current = current->next;
+		}
+
+		if (prev != NULL)
+		{
+			if (current)
+			{
+				/* the task's place is between prev and current */
+				struct starpu_task *it;
+				for(it = list->head; it != current; it = it->next)
+				{
+					exp_len += starpu_task_expected_length(it, perf_arch, nimpl);
+					(*fifo_ntasks) ++;
+				}
+			}
+			else
+			{
+				/* the task's place is at the tail of the list */
+				exp_len = fifo_queue->exp_len;
+				*fifo_ntasks = fifo_queue->ntasks;
+			}
+		}
+	}
+
+
+	return exp_len;
+}
+
 int
 _starpu_fifo_push_sorted_task(struct _starpu_fifo_taskq *fifo_queue, struct starpu_task *task)
 {
@@ -63,6 +136,15 @@ _starpu_fifo_push_sorted_task(struct _starpu_fifo_taskq *fifo_queue, struct star
 		task->prev = NULL;
 		task->next = NULL;
 	}
+	else if (list->head->priority == task->priority &&
+		 list->head->priority == list->tail->priority)
+	{
+		/* They all have the same priority, just put at the end */
+		list->tail->next = task;
+		task->next = NULL;
+		task->prev = list->tail;
+		list->tail = task;
+	}
 	else
 	{
 		struct starpu_task *current = list->head;
@@ -130,6 +212,42 @@ int _starpu_fifo_push_task(struct _starpu_fifo_taskq *fifo_queue, struct starpu_
 	return 0;
 }
 
+int _starpu_fifo_push_back_task(struct _starpu_fifo_taskq *fifo_queue, struct starpu_task *task)
+{
+
+	if (task->priority > 0)
+	{
+		_starpu_fifo_push_sorted_task(fifo_queue, task);
+	}
+	else
+	{
+		starpu_task_list_push_front(&fifo_queue->taskq, task);
+
+		fifo_queue->ntasks++;
+	}
+
+	return 0;
+}
+
+int _starpu_fifo_pop_this_task(struct _starpu_fifo_taskq *fifo_queue, int workerid, struct starpu_task *task)
+{
+	unsigned nimpl = 0;
+	STARPU_ASSERT(task);
+#ifdef STARPU_DEBUG
+	STARPU_ASSERT(starpu_task_list_ismember(&fifo_queue->taskq, task));
+#endif
+
+	if (workerid < 0 || starpu_worker_can_execute_task_first_impl(workerid, task, &nimpl))
+	{
+		starpu_task_set_implementation(task, nimpl);
+		starpu_task_list_erase(&fifo_queue->taskq, task);
+		fifo_queue->ntasks--;
+		return 1;
+	}
+
+	return 0;
+}
+
 struct starpu_task *_starpu_fifo_pop_task(struct _starpu_fifo_taskq *fifo_queue, int workerid)
 {
 	struct starpu_task *task;
@@ -138,18 +256,8 @@ struct starpu_task *_starpu_fifo_pop_task(struct _starpu_fifo_taskq *fifo_queue,
 	     task != starpu_task_list_end(&fifo_queue->taskq);
 	     task  = starpu_task_list_next(task))
 	{
-		unsigned nimpl;
-		STARPU_ASSERT(task);
-
-		for (nimpl = 0; nimpl < STARPU_MAXIMPLEMENTATIONS; nimpl++)
-			if (starpu_worker_can_execute_task(workerid, task, nimpl))
-			{
-				starpu_task_set_implementation(task, nimpl);
-				starpu_task_list_erase(&fifo_queue->taskq, task);
-				fifo_queue->ntasks--;
-				_STARPU_TRACE_JOB_POP(task, 0);
-				return task;
-			}
+		if (_starpu_fifo_pop_this_task(fifo_queue, workerid, task))
+			return task;
 	}
 
 	return NULL;
@@ -166,7 +274,6 @@ struct starpu_task *_starpu_fifo_pop_local_task(struct _starpu_fifo_taskq *fifo_
 	{
 		task = starpu_task_list_pop_front(&fifo_queue->taskq);
 		fifo_queue->ntasks--;
-		_STARPU_TRACE_JOB_POP(task, 0);
 	}
 
 	return task;
@@ -175,17 +282,13 @@ struct starpu_task *_starpu_fifo_pop_local_task(struct _starpu_fifo_taskq *fifo_
 /* pop every task that can be executed on the calling driver */
 struct starpu_task *_starpu_fifo_pop_every_task(struct _starpu_fifo_taskq *fifo_queue, int workerid)
 {
-	struct starpu_task_list *old_list;
-	unsigned size;
-
+	unsigned size = fifo_queue->ntasks;
 	struct starpu_task *new_list = NULL;
-	struct starpu_task *new_list_tail = NULL;
-
-	size = fifo_queue->ntasks;
 
 	if (size > 0)
 	{
-		old_list = &fifo_queue->taskq;
+		struct starpu_task_list *old_list = &fifo_queue->taskq;
+		struct starpu_task *new_list_tail = NULL;
 		unsigned new_list_size = 0;
 
 		struct starpu_task *task, *next_task;
@@ -197,8 +300,7 @@ struct starpu_task *_starpu_fifo_pop_every_task(struct _starpu_fifo_taskq *fifo_
 			unsigned nimpl;
 			next_task = task->next;
 
-			for (nimpl = 0; nimpl < STARPU_MAXIMPLEMENTATIONS; nimpl++)
-			if (starpu_worker_can_execute_task(workerid, task, nimpl))
+			if (starpu_worker_can_execute_task_first_impl(workerid, task, &nimpl))
 			{
 				/* this elements can be moved into the new list */
 				new_list_size++;
@@ -220,7 +322,6 @@ struct starpu_task *_starpu_fifo_pop_every_task(struct _starpu_fifo_taskq *fifo_
 					task->next = NULL;
 				}
 				starpu_task_set_implementation(task, nimpl);
-				break;
 			}
 
 			task = next_task;
