@@ -1,7 +1,8 @@
 /* StarPU --- Runtime system for heterogeneous multicore architectures.
  *
  * Copyright (C) 2011 William Braik, Yann Courtois, Jean-Marie Couteyen, Anthony Roy
- * Copyright (C) 2011, 2012 Centre National de la Recherche Scientifique
+ * Copyright (C) 2011, 2012, 2013, 2016, 2017 CNRS
+ * Copyright (C) 2016, 2017  Inria
  *
  * StarPU is free software; you can redistribute it and/or modify
  * it under the terms of the GNU Lesser General Public License as published by
@@ -24,12 +25,12 @@
 #include <stdio.h>
 #include <string.h>
 #include <math.h>
-#include <pthread.h>
 #include <common/timing.h>
-#include <common/utils.h>
+#include <common/thread.h>
+#include <common/config.h>
 
 extern struct _starpu_top_message_queue*  _starpu_top_mt;
-int starpu_top = 0;
+int _starpu_top = 0;
 int starpu_top_debug_on = 0;
 unsigned int starpu_top_data_cpt = 0;
 unsigned int starpu_top_param_cpt = 0;
@@ -39,16 +40,17 @@ struct starpu_top_data** starpu_top_datas;
 struct starpu_top_param** starpu_top_params;
 
 sem_t starpu_top_wait_for_go;
-pthread_mutex_t starpu_top_wait_for_continue_mutex;
-pthread_cond_t starpu_top_wait_for_continue_cond = PTHREAD_COND_INITIALIZER;
+starpu_pthread_mutex_t starpu_top_wait_for_continue_mutex;
+starpu_pthread_cond_t starpu_top_wait_for_continue_cond = STARPU_PTHREAD_COND_INITIALIZER;
 
-int _starpu_top_status_get()
+#undef _starpu_top_status_get
+int _starpu_top_status_get(void)
 {
-	return starpu_top;
+	return _starpu_top;
 }
 
 static
-unsigned long long current_timestamp();
+unsigned long long current_timestamp(void);
 
 /*********************************************
 *****************INIT FUNC********************
@@ -65,11 +67,11 @@ char *message_for_topparam_init(struct starpu_top_param* param);
  * running
  */
 static
-void copy_data_and_param()
+void copy_data_and_param(void)
 {
 	printf("%s:%d trace\n", __FILE__, __LINE__);
 	//copying datas
-	starpu_top_datas = (struct starpu_top_data **) malloc(starpu_top_data_cpt*sizeof(struct starpu_top_data*));
+	_STARPU_MALLOC(starpu_top_datas, starpu_top_data_cpt*sizeof(struct starpu_top_data*));
 	struct starpu_top_data* cur = starpu_top_first_data;
 	unsigned int i = 0;
 	for(i = 0; i < starpu_top_data_cpt; i++)
@@ -78,7 +80,7 @@ void copy_data_and_param()
 		cur = cur->next;
 	}
 	//copying params
-	starpu_top_params = (struct starpu_top_param **) malloc(starpu_top_param_cpt*sizeof(struct starpu_top_param*));
+	_STARPU_MALLOC(starpu_top_params, starpu_top_param_cpt*sizeof(struct starpu_top_param*));
 	struct starpu_top_param* cur2 = starpu_top_first_param;
 	for(i = 0; i < starpu_top_param_cpt; i++)
 	{
@@ -89,7 +91,7 @@ void copy_data_and_param()
 
 static void starpu_top_get_device_type(int id, char* type)
 {
-	enum starpu_archtype device_type=starpu_worker_get_type(id);
+	enum starpu_worker_archtype device_type=starpu_worker_get_type(id);
 	switch (device_type)
 	{
 	case STARPU_CPU_WORKER:
@@ -101,22 +103,30 @@ static void starpu_top_get_device_type(int id, char* type)
 	case STARPU_OPENCL_WORKER:
 		strncpy(type, "OPENCL",9);
 		break;
-	case STARPU_GORDON_WORKER:
-		strncpy(type, "GORDON",9);
+	case STARPU_ANY_WORKER:
+		strncpy(type, "ANY",9);
+		break;
+	case STARPU_MIC_WORKER:
+		strncpy(type, "MIC", 9);
+		break;
+	case STARPU_SCC_WORKER:
+		strncpy(type, "SCC", 9);
 		break;
 	}
+	type[9] = 0;
 }
 
-static void starpu_top_send_devices_info()
+static void starpu_top_send_devices_info(void)
 {
-	char* message=(char *)malloc(5*sizeof(char));
+	char *message;
+	_STARPU_MALLOC(message, 5*sizeof(char));
 	snprintf(message,5,"DEV\n");
 	_starpu_top_message_add(_starpu_top_mt,message);
 
 	unsigned int i;
 	for(i=0;i<starpu_worker_get_count();i++)
 	{
-		message=(char *)malloc(sizeof(char)*128);
+		_STARPU_MALLOC(message, sizeof(char)*128);
 		char dev_type[10];
 		char dev_name[64];
 		starpu_top_get_device_type(i,dev_type);
@@ -125,7 +135,7 @@ static void starpu_top_send_devices_info()
 		_starpu_top_message_add(_starpu_top_mt,message);
 	}
 
-	message=(char*)malloc(6*sizeof(char));
+	_STARPU_MALLOC(message, 6*sizeof(char));
 	snprintf(message,6,"/DEV\n");
 	_starpu_top_message_add(_starpu_top_mt,message);
 }
@@ -133,10 +143,10 @@ static void starpu_top_send_devices_info()
 
 void starpu_top_init_and_wait(const char* server_name)
 {
-	starpu_top=1;
+	_starpu_top=1;
 	sem_init(&starpu_top_wait_for_go,0,0);
 
-	_STARPU_PTHREAD_MUTEX_INIT(&starpu_top_wait_for_continue_mutex, NULL);
+	STARPU_PTHREAD_MUTEX_INIT(&starpu_top_wait_for_continue_mutex, NULL);
 
 	//profiling activation
 	starpu_profiling_status_set(STARPU_PROFILING_ENABLE);
@@ -150,22 +160,23 @@ void starpu_top_init_and_wait(const char* server_name)
 	_starpu_top_communications_threads_launcher();
 
 	//sending server information (report to protocol)
-	char* message = (char *) malloc(strlen("SERVERINFO\n")+1);
+	char *message;
+	_STARPU_MALLOC(message, strlen("SERVERINFO\n")+1);
 	sprintf(message, "%s", "SERVERINFO\n");
 	_starpu_top_message_add(_starpu_top_mt,message);
-	message = (char *) malloc(strlen(server_name)+2);
+	_STARPU_MALLOC(message, strlen(server_name)+2);
 	sprintf(message, "%s\n", server_name);
 	_starpu_top_message_add(_starpu_top_mt,message);
-	message = (char *) malloc(25);
-	sprintf(message, "%lld\n", current_timestamp());
+	_STARPU_MALLOC(message, 25);
+	sprintf(message, "%llu\n", current_timestamp());
 	_starpu_top_message_add(_starpu_top_mt,message);
-	message = (char *) malloc(strlen("/SERVERINFO\n")+1);
+	_STARPU_MALLOC(message, strlen("/SERVERINFO\n")+1);
 	sprintf(message,"%s", "/SERVERINFO\n");
 	_starpu_top_message_add(_starpu_top_mt,message);
 
 
 	//sending data list
-	message = (char *) malloc(strlen("DATA\n")+1);
+	_STARPU_MALLOC(message, strlen("DATA\n")+1);
 	sprintf(message, "%s", "DATA\n");
 	_starpu_top_message_add(_starpu_top_mt,message);
 	struct starpu_top_data * cur_data = starpu_top_first_data;
@@ -174,12 +185,12 @@ void starpu_top_init_and_wait(const char* server_name)
 		_starpu_top_message_add(_starpu_top_mt,message_for_topdata_init(cur_data));
 		cur_data = cur_data->next;
 	}
-	message = (char *) malloc(strlen("/DATA\n")+1);
+	_STARPU_MALLOC(message, strlen("/DATA\n")+1);
 	sprintf(message, "%s", "/DATA\n");
 	_starpu_top_message_add(_starpu_top_mt,message);
 
 	//sending parameter list
-	message = (char *) malloc(strlen("PARAMS\n")+1);
+	_STARPU_MALLOC(message, strlen("PARAMS\n")+1);
 	sprintf(message, "%s", "PARAMS\n");
 	_starpu_top_message_add(_starpu_top_mt,message);
 	struct starpu_top_param * cur_param = starpu_top_first_param;
@@ -190,7 +201,7 @@ void starpu_top_init_and_wait(const char* server_name)
 	  cur_param = cur_param->next;
 	}
 	printf("%s:%d parameters sended\n", __FILE__, __LINE__);
-	message = (char *) malloc(strlen("/PARAMS\n")+1);
+	_STARPU_MALLOC(message, strlen("/PARAMS\n")+1);
 	sprintf(message, "%s", "/PARAMS\n");
 	_starpu_top_message_add(_starpu_top_mt,message);
 
@@ -203,7 +214,7 @@ void starpu_top_init_and_wait(const char* server_name)
 	copy_data_and_param();
 
 	//sending READY message
-	message = (char *) malloc(strlen("READY\n")+1);
+	_STARPU_MALLOC(message, strlen("READY\n")+1);
 	sprintf(message, "%s", "READY\n");
 	_starpu_top_message_add(_starpu_top_mt,message);
 
@@ -212,6 +223,31 @@ void starpu_top_init_and_wait(const char* server_name)
 	sem_wait(&starpu_top_wait_for_go);
 }
 
+void _starpu_top_shutdown(void)
+{
+	{
+		struct starpu_top_data * cur = starpu_top_first_data, * next;
+		while (cur)
+		{
+			next = cur->next;
+			free(cur);
+			cur = next;
+		}
+		starpu_top_first_data = NULL;
+	}
+
+	{
+		struct starpu_top_param * cur = starpu_top_first_param, *next;
+		while (cur)
+		{
+			next = cur->next;
+			free(cur);
+			cur = next;
+		}
+		starpu_top_first_param = NULL;
+	}
+}
+
 static
 void starpu_top_enqueue_data(struct starpu_top_data * data)
 {
@@ -231,7 +267,8 @@ void starpu_top_enqueue_data(struct starpu_top_data * data)
 struct starpu_top_data * starpu_top_add_data_boolean(const char* data_name,
 						     int active)
 {
-	struct starpu_top_data * data = (struct starpu_top_data *) malloc(sizeof(struct starpu_top_data));
+	struct starpu_top_data *data;
+	_STARPU_MALLOC(data, sizeof(struct starpu_top_data));
 	data->id = starpu_top_data_cpt++;
 	data->name = data_name;
 	data->type = STARPU_TOP_DATA_BOOLEAN;
@@ -248,7 +285,8 @@ struct starpu_top_data * starpu_top_add_data_integer(const char* data_name,
 						     int maximum_value,
 						     int active)
 {
-	struct starpu_top_data * data = (struct starpu_top_data *) malloc(sizeof(struct starpu_top_data));
+	struct starpu_top_data *data;
+	_STARPU_MALLOC(data, sizeof(struct starpu_top_data));
 	data->id = starpu_top_data_cpt++;
 	data->name = data_name;
 	data->type = STARPU_TOP_DATA_INTEGER;
@@ -267,7 +305,8 @@ struct starpu_top_data* starpu_top_add_data_float(const char* data_name,
 						  double maximum_value,
 						  int active)
 {
-	struct starpu_top_data * data = (struct starpu_top_data *) malloc(sizeof(struct starpu_top_data));
+	struct starpu_top_data *data;
+	_STARPU_MALLOC(data, sizeof(struct starpu_top_data));
 	data->id = starpu_top_data_cpt++;
 	data->name = data_name;
 	data->type = STARPU_TOP_DATA_FLOAT;
@@ -283,7 +322,8 @@ struct starpu_top_data* starpu_top_add_data_float(const char* data_name,
 
 char *message_for_topdata_init(struct starpu_top_data* data)
 {
-	char*message = (char *) malloc(256+strlen(data->name));
+	char *message;
+	_STARPU_MALLOC(message, 256+strlen(data->name));
 	switch(data->type)
 	{
 		case STARPU_TOP_DATA_BOOLEAN:
@@ -323,7 +363,7 @@ char *message_for_topparam_init(struct starpu_top_param* param)
 	switch(param->type)
 	{
 	case STARPU_TOP_PARAM_BOOLEAN:
-		message = (char *) malloc(256);
+		_STARPU_MALLOC(message, 256);
 		sprintf(message,
 				"BOOL;%u;%s;%d\n",
 				param->id,
@@ -331,7 +371,7 @@ char *message_for_topparam_init(struct starpu_top_param* param)
 				(*(int*)(param->value)) ? 1 : 0);
 		break;
 	case STARPU_TOP_PARAM_INTEGER:
-		message = (char *) malloc(256);
+		_STARPU_MALLOC(message, 256);
 		sprintf(message,
 				"INT;%u;%s;%d;%d;%d\n",param->id,
 				param->name,
@@ -340,7 +380,7 @@ char *message_for_topparam_init(struct starpu_top_param* param)
 				*(int*)(param->value));
 		break;
 	case STARPU_TOP_PARAM_FLOAT:
-		message = (char *) malloc(256);
+		_STARPU_MALLOC(message, 256);
 		sprintf(message,
 				"FLOAT;%u;%s;%f;%f;%f\n",
 				param->id,
@@ -355,7 +395,7 @@ char *message_for_topparam_init(struct starpu_top_param* param)
 		{
 			length += strlen(param->enum_values[i])+1;
 		}
-		message = (char *) malloc(256+length);
+		_STARPU_MALLOC(message, 256+length);
 		sprintf(message,
 				"ENUM;%u;%s;",
 				param->id,
@@ -400,7 +440,8 @@ struct starpu_top_param* starpu_top_register_parameter_boolean(const char* param
 							       void (*callback)(struct starpu_top_param*))
 {
 	STARPU_ASSERT(!_starpu_top_status_get());
-	struct starpu_top_param * param = (struct starpu_top_param *) malloc(sizeof(struct starpu_top_param));
+	struct starpu_top_param *param;
+	_STARPU_MALLOC(param, sizeof(struct starpu_top_param));
 	param->callback = callback;
 	param->name = param_name;
 	param->id = starpu_top_param_cpt++;
@@ -420,7 +461,8 @@ struct starpu_top_param* starpu_top_register_parameter_integer(const char* param
 							       void (*callback)(struct starpu_top_param*))
 {
 	STARPU_ASSERT(!_starpu_top_status_get());
-	struct starpu_top_param * param = (struct starpu_top_param *) malloc(sizeof(struct starpu_top_param));
+	struct starpu_top_param *param;
+	_STARPU_MALLOC(param, sizeof(struct starpu_top_param));
 	param->callback = callback;
 	param->name = param_name;
 	param->id = starpu_top_param_cpt++;
@@ -442,7 +484,8 @@ struct starpu_top_param* starpu_top_register_parameter_float(const char* param_n
 							     void (*callback)(struct starpu_top_param*))
 {
 	STARPU_ASSERT(!_starpu_top_status_get());
-	struct starpu_top_param * param = (struct starpu_top_param *) malloc(sizeof(struct starpu_top_param));
+	struct starpu_top_param *param;
+	_STARPU_MALLOC(param, sizeof(struct starpu_top_param));
 	param->callback = callback;
 	param->name = param_name;
 	param->id = starpu_top_param_cpt++;
@@ -464,7 +507,8 @@ struct starpu_top_param* starpu_top_register_parameter_enum(const char* param_na
 							    void (*callback)(struct starpu_top_param*))
 {
 	STARPU_ASSERT(!_starpu_top_status_get());
-	struct starpu_top_param * param = (struct starpu_top_param *) malloc(sizeof(struct starpu_top_param));
+	struct starpu_top_param *param;
+	_STARPU_MALLOC(param, sizeof(struct starpu_top_param));
 	param->callback = callback;
 	param->name = param_name;
 	param->id = starpu_top_param_cpt++;
@@ -488,9 +532,10 @@ void starpu_top_update_data_boolean(const struct starpu_top_data* data, int valu
 		return;
 	if(data->active)
 	{
-		char*message = (char *) malloc(256+strlen(data->name));
+		char *message;
+		_STARPU_MALLOC(message, 256+strlen(data->name));
 		sprintf(message,
-				"U;%u;%d;%lld\n",
+				"U;%u;%d;%llu\n",
 				data->id,
 				(value?1:0),
 				current_timestamp());
@@ -504,9 +549,10 @@ void starpu_top_update_data_integer(const struct starpu_top_data* data, int valu
 		return;
 	if(data->active)
 	{
-		char*message = (char *) malloc(256+strlen(data->name));
+		char *message;
+		_STARPU_MALLOC(message, 256+strlen(data->name));
 		sprintf(message,
-				"U;%u;%d;%lld\n",
+				"U;%u;%d;%llu\n",
 				data->id,
 				value,
 				current_timestamp());
@@ -520,9 +566,10 @@ void starpu_top_update_data_float(const struct starpu_top_data* data, double val
 		return;
 	if(data->active)
 	{
-		char*message = (char *) malloc(256+strlen(data->name));
+		char *message;
+		_STARPU_MALLOC(message, 256+strlen(data->name));
 		sprintf(message,
-				"U;%u;%f;%lld\n",
+				"U;%u;%f;%llu\n",
 				data->id, value,
 				current_timestamp());
 		_starpu_top_message_add(_starpu_top_mt,message);
@@ -533,7 +580,8 @@ void starpu_top_update_parameter(const struct starpu_top_param* param)
 {
 	if (!_starpu_top_status_get())
 		return;
-	char*message = (char *) malloc(50);
+	char *message;
+	_STARPU_MALLOC(message, 50);
 
 	switch(param->type)
 	{
@@ -541,7 +589,7 @@ void starpu_top_update_parameter(const struct starpu_top_param* param)
 		case STARPU_TOP_PARAM_INTEGER:
 		case STARPU_TOP_PARAM_ENUM:
 			sprintf(message,
-					"SET;%u;%d;%lld\n",
+					"SET;%u;%d;%llu\n",
 					param->id,
 					*((int*)param->value),
 					current_timestamp());
@@ -549,7 +597,7 @@ void starpu_top_update_parameter(const struct starpu_top_param* param)
 
 		case STARPU_TOP_PARAM_FLOAT:
 			sprintf(message,
-					"SET;%u;%f;%lld\n",
+					"SET;%u;%f;%llu\n",
 					param->id,
 					*((double*)param->value),
 					current_timestamp());
@@ -568,7 +616,8 @@ void starpu_top_debug_log(const char* debug_message)
 	if(starpu_top_debug_on)
 	{
 		//length can be up to strlen*2, if message contains only unwanted chars
-		char * message = (char *) malloc(strlen(debug_message)*2+16);
+		char *message;
+		_STARPU_MALLOC(message, strlen(debug_message)*2+16);
 		sprintf(message,"MESSAGE;");
 
 		//escape unwanted char : ; and \n
@@ -596,7 +645,8 @@ void starpu_top_debug_lock(const char* debug_message)
 {
 	if(starpu_top_debug_on)
 	{
-		char * message = (char *) malloc(strlen(debug_message)*2+16);
+		char *message;
+		_STARPU_MALLOC(message, strlen(debug_message)*2+16);
 		sprintf(message,"LOCK;");
 		char* cur = message+5;
 		while(*debug_message!='\0')
@@ -616,10 +666,10 @@ void starpu_top_debug_lock(const char* debug_message)
 		_starpu_top_message_add(_starpu_top_mt,message);
 
 		//This threads keeps locked while we don't receive an STEP message
-		_STARPU_PTHREAD_MUTEX_LOCK(&starpu_top_wait_for_continue_mutex);
-		_STARPU_PTHREAD_COND_WAIT(&starpu_top_wait_for_continue_cond,
+		STARPU_PTHREAD_MUTEX_LOCK(&starpu_top_wait_for_continue_mutex);
+		STARPU_PTHREAD_COND_WAIT(&starpu_top_wait_for_continue_cond,
 					  &starpu_top_wait_for_continue_mutex);
-		_STARPU_PTHREAD_MUTEX_UNLOCK(&starpu_top_wait_for_continue_mutex);
+		STARPU_PTHREAD_MUTEX_UNLOCK(&starpu_top_wait_for_continue_mutex);
 	}
 }
 
@@ -629,7 +679,7 @@ void starpu_top_debug_lock(const char* debug_message)
  **************TIME FUNCTION****************
  *******************************************/
 
-unsigned long long current_timestamp()
+unsigned long long current_timestamp(void)
 {
 	struct timespec now;
 	_starpu_clock_gettime(&now);
@@ -665,7 +715,7 @@ enum starpu_top_message_type starpu_top_get_message_type(const char* message)
 }
 
 static
-void starpu_top_unlock_starpu()
+void starpu_top_unlock_starpu(void)
 {
 	sem_post(&starpu_top_wait_for_go);
 	printf("%s:%d starpu started\n", __FILE__, __LINE__);
@@ -730,7 +780,8 @@ void starpu_top_change_debug_mode(const char*message)
 		printf("%s:%d debug is now OFF\n", __FILE__, __LINE__);
 	}
 
-	char * m = (char *) malloc(strlen(message)+1);
+	char *m;
+	_STARPU_MALLOC(m, strlen(message)+1);
 	sprintf(m,"%s",message);
 	_starpu_top_message_add(_starpu_top_mt,m);
 }
@@ -739,9 +790,9 @@ void starpu_top_change_debug_mode(const char*message)
  * Unlock starpu if it was locked in debug state
 */
 static
-void starpu_top_debug_next_step()
+void starpu_top_debug_next_step(void)
 {
-	_STARPU_PTHREAD_COND_SIGNAL(&starpu_top_wait_for_continue_cond);
+	STARPU_PTHREAD_COND_SIGNAL(&starpu_top_wait_for_continue_cond);
 }
 
 
