@@ -1453,7 +1453,6 @@ void starpu_omp_critical_inline_end(const char *name)
 	{
 		_starpu_spin_lock(&_global_state.named_criticals_lock);
 		HASH_FIND_STR(_global_state.named_criticals, name, critical);
-		STARPU_ASSERT(critical != NULL);
 		_starpu_spin_unlock(&_global_state.named_criticals_lock);
 	}
 	else
@@ -1461,6 +1460,7 @@ void starpu_omp_critical_inline_end(const char *name)
 		critical = _global_state.default_critical;
 	}
 
+	STARPU_ASSERT(critical != NULL);
 	_starpu_spin_lock(&critical->lock);
 	STARPU_ASSERT(critical->state == 1);
 	critical->state = 0;
@@ -1963,7 +1963,7 @@ int starpu_omp_for_inline_first(unsigned long long nb_iterations, unsigned long
 	{
 		_starpu_omp_for_loop_end(parallel_region, task, loop, ordered);
 	}
-	return (*_nb_i != 0);
+	return *_nb_i != 0;
 }
 
 int starpu_omp_for_inline_next(unsigned long long nb_iterations, unsigned long long chunk, int schedule, int ordered, unsigned long long *_first_i, unsigned long long *_nb_i)
@@ -1977,7 +1977,7 @@ int starpu_omp_for_inline_next(unsigned long long nb_iterations, unsigned long l
 	{
 		_starpu_omp_for_loop_end(parallel_region, task, loop, ordered);
 	}
-	return (*_nb_i != 0);
+	return *_nb_i != 0;
 }
 
 int starpu_omp_for_inline_first_alt(unsigned long long nb_iterations, unsigned long long chunk, int schedule, int ordered, unsigned long long *_begin_i, unsigned long long *_end_i)
