@@ -674,9 +674,7 @@ void CommunicationManager::parseInitDevMessage(QString messageString)
                         _inMessageStrings.key(COM_MSG_IN_DEV_CUDA))
                 == 0 || deviceTypeString.compare(
                         _inMessageStrings.key(COM_MSG_IN_DEV_OPENCL)) == 0
-                || deviceTypeString.compare(
-                        _inMessageStrings.key(COM_MSG_IN_DEV_GORDON))
-                == 0,
+		,
                 "CommunicationManager::parseInitDevMessage()",
                 "Bogus message received in INIT DEV");
 
@@ -695,11 +693,6 @@ void CommunicationManager::parseInitDevMessage(QString messageString)
         {
             deviceType = SERVERDEVICE_OPENCL;
         }
-        else if (deviceTypeString.compare(
-                _inMessageStrings.key(COM_MSG_IN_DEV_GORDON)) == 0)
-        {
-            deviceType = SERVERDEVICE_GORDON;
-        }
 
         starpu_top_device device;
         device.id = deviceId;
@@ -1303,7 +1296,11 @@ void CommunicationManager::sendMessage(QString messageString)
 {
     messageString.append(COM_MSG_ENDL);
 
+#if QT_VERSION >= QT_VERSION_CHECK(5, 0, 0)
+    write(messageString.toLatin1());
+#else
     write(messageString.toAscii());
+#endif
 }
 
 void CommunicationManager::sendGoMessage()
@@ -1517,7 +1514,6 @@ void CommunicationManager::initInMessageStrings()
     _inMessageStrings.insert("CPU", COM_MSG_IN_DEV_CPU);
     _inMessageStrings.insert("GPU", COM_MSG_IN_DEV_CUDA);
     _inMessageStrings.insert("OPENCL", COM_MSG_IN_DEV_OPENCL);
-    _inMessageStrings.insert("GORDON", COM_MSG_IN_DEV_GORDON);
     _beginEndMessageStrings.insert("DEV");
     _beginEndMessageStrings.insert("/DEV");
     // Server ready
