@@ -1,6 +1,6 @@
 /* StarPU --- Runtime system for heterogeneous multicore architectures.
  *
- * Copyright (C) 2011  Université de Bordeaux 1
+ * Copyright (C) 2011, 2016  Université de Bordeaux
  *
  * StarPU is free software; you can redistribute it and/or modify
  * it under the terms of the GNU Lesser General Public License as published by
@@ -15,13 +15,17 @@
  */
 
 #include <starpu.h>
+#include "../helper.h"
 
-#define FPRINTF(ofile, fmt, args ...) do { if (!getenv("STARPU_SSILENT")) {fprintf(ofile, fmt, ##args); }} while(0)
+/*
+ * Test that when using starpu_data_acquire_cb, the callback is properly called
+ */
 
 unsigned token = 0;
-starpu_data_handle token_handle;
+starpu_data_handle_t token_handle;
 
-void callback(void *arg __attribute__ ((unused)))
+static
+void callback(void *arg STARPU_ATTRIBUTE_UNUSED)
 {
 	token = 42;
         starpu_data_release(token_handle);
@@ -29,17 +33,20 @@ void callback(void *arg __attribute__ ((unused)))
 
 int main(int argc, char **argv)
 {
-        starpu_init(NULL);
+	int ret;
 
-	starpu_variable_data_register(&token_handle, 0, (uintptr_t)&token, sizeof(unsigned));
+        ret = starpu_initialize(NULL, &argc, &argv);
+	if (ret == -ENODEV) return STARPU_TEST_SKIPPED;
+	STARPU_CHECK_RETURN_VALUE(ret, "starpu_init");
+
+	starpu_variable_data_register(&token_handle, STARPU_MAIN_RAM, (uintptr_t)&token, sizeof(unsigned));
         starpu_data_acquire_cb(token_handle, STARPU_RW, callback, NULL);
 
 	starpu_data_unregister(token_handle);
 
         FPRINTF(stderr, "Token: %u\n", token);
-        STARPU_ASSERT(token == 42);
 
 	starpu_shutdown();
 
-	return 0;
+	return (token == 42) ? EXIT_SUCCESS : EXIT_FAILURE;
 }
