@@ -1,7 +1,7 @@
 /* StarPU --- Runtime system for heterogeneous multicore architectures.
  *
- * Copyright (C) 2010  Université de Bordeaux 1
- * Copyright (C) 2010, 2011, 2012  Centre National de la Recherche Scientifique
+ * Copyright (C) 2010, 2012, 2015-2016  Université de Bordeaux
+ * Copyright (C) 2010, 2011, 2012, 2013  CNRS
  *
  * StarPU is free software; you can redistribute it and/or modify
  * it under the terms of the GNU Lesser General Public License as published by
@@ -20,8 +20,12 @@
 #include <errno.h>
 #include <starpu.h>
 #include <stdlib.h>
-#include <pthread.h>
 #include "../helper.h"
+#include <common/thread.h>
+
+/*
+ * Trigger various combinations of access modes
+ */
 
 #ifdef STARPU_QUICK_CHECK
 #  define N	100
@@ -31,8 +35,8 @@
 
 #define VECTORSIZE	1024
 
-static pthread_mutex_t mutex = PTHREAD_MUTEX_INITIALIZER;
-static pthread_cond_t cond = PTHREAD_COND_INITIALIZER;
+static starpu_pthread_mutex_t mutex = STARPU_PTHREAD_MUTEX_INITIALIZER;
+static starpu_pthread_cond_t cond = STARPU_PTHREAD_COND_INITIALIZER;
 
 static unsigned finished = 0;
 
@@ -45,31 +49,33 @@ static unsigned *v2;
 static void callback(void *arg)
 {
 	unsigned res = STARPU_ATOMIC_ADD(&cnt, -1);
+	ANNOTATE_HAPPENS_BEFORE(&cnt);
 
 	if (res == 0)
 	{
-		_STARPU_PTHREAD_MUTEX_LOCK(&mutex);
+		ANNOTATE_HAPPENS_AFTER(&cnt);
+		STARPU_PTHREAD_MUTEX_LOCK(&mutex);
 		finished = 1;
-		_STARPU_PTHREAD_COND_SIGNAL(&cond);
-		_STARPU_PTHREAD_MUTEX_UNLOCK(&mutex);
+		STARPU_PTHREAD_COND_SIGNAL(&cond);
+		STARPU_PTHREAD_MUTEX_UNLOCK(&mutex);
 	}
 }
 
 
 
-static void cuda_codelet_null(void *descr[], __attribute__ ((unused)) void *_args)
+static void cuda_codelet_null(void *descr[], STARPU_ATTRIBUTE_UNUSED void *_args)
 {
 }
 
-static void opencl_codelet_null(void *descr[], __attribute__ ((unused)) void *_args)
+static void opencl_codelet_null(void *descr[], STARPU_ATTRIBUTE_UNUSED void *_args)
 {
 }
 
-static void cpu_codelet_null(void *descr[], __attribute__ ((unused)) void *_args)
+void cpu_codelet_null(void *descr[], STARPU_ATTRIBUTE_UNUSED void *_args)
 {
 }
 
-static enum starpu_access_mode select_random_mode(void)
+static enum starpu_data_access_mode select_random_mode(void)
 {
 	int r = rand();
 
@@ -87,90 +93,90 @@ static enum starpu_access_mode select_random_mode(void)
 
 static struct starpu_codelet cl_r_r =
 {
-	.where = STARPU_CPU|STARPU_CUDA|STARPU_OPENCL,
-	.cpu_funcs = {cpu_codelet_null, NULL},
-	.cuda_funcs = {cuda_codelet_null, NULL},
-        .opencl_funcs = {opencl_codelet_null, NULL},
+	.cpu_funcs = {cpu_codelet_null},
+	.cuda_funcs = {cuda_codelet_null},
+        .opencl_funcs = {opencl_codelet_null},
+	.cpu_funcs_name = {"cpu_codelet_null"},
 	.nbuffers = 2,
 	.modes = {STARPU_R, STARPU_R}
 };
 
 static struct starpu_codelet cl_r_w =
 {
-	.where = STARPU_CPU|STARPU_CUDA|STARPU_OPENCL,
-	.cpu_funcs = {cpu_codelet_null, NULL},
-	.cuda_funcs = {cuda_codelet_null, NULL},
-        .opencl_funcs = {opencl_codelet_null, NULL},
+	.cpu_funcs = {cpu_codelet_null},
+	.cuda_funcs = {cuda_codelet_null},
+        .opencl_funcs = {opencl_codelet_null},
+	.cpu_funcs_name = {"cpu_codelet_null"},
 	.nbuffers = 2,
 	.modes = {STARPU_R, STARPU_W}
 };
 
 static struct starpu_codelet cl_r_rw =
 {
-	.where = STARPU_CPU|STARPU_CUDA|STARPU_OPENCL,
-	.cpu_funcs = {cpu_codelet_null, NULL},
-	.cuda_funcs = {cuda_codelet_null, NULL},
-        .opencl_funcs = {opencl_codelet_null, NULL},
+	.cpu_funcs = {cpu_codelet_null},
+	.cuda_funcs = {cuda_codelet_null},
+        .opencl_funcs = {opencl_codelet_null},
+	.cpu_funcs_name = {"cpu_codelet_null"},
 	.nbuffers = 2,
 	.modes = {STARPU_R, STARPU_RW}
 };
 
 static struct starpu_codelet cl_w_r =
 {
-	.where = STARPU_CPU|STARPU_CUDA|STARPU_OPENCL,
-	.cpu_funcs = {cpu_codelet_null, NULL},
-	.cuda_funcs = {cuda_codelet_null, NULL},
-        .opencl_funcs = {opencl_codelet_null, NULL},
+	.cpu_funcs = {cpu_codelet_null},
+	.cuda_funcs = {cuda_codelet_null},
+        .opencl_funcs = {opencl_codelet_null},
+	.cpu_funcs_name = {"cpu_codelet_null"},
 	.nbuffers = 2,
 	.modes = {STARPU_W, STARPU_R}
 };
 
 static struct starpu_codelet cl_w_w =
 {
-	.where = STARPU_CPU|STARPU_CUDA|STARPU_OPENCL,
-	.cpu_funcs = {cpu_codelet_null, NULL},
-	.cuda_funcs = {cuda_codelet_null, NULL},
-        .opencl_funcs = {opencl_codelet_null, NULL},
+	.cpu_funcs = {cpu_codelet_null},
+	.cuda_funcs = {cuda_codelet_null},
+        .opencl_funcs = {opencl_codelet_null},
+	.cpu_funcs_name = {"cpu_codelet_null"},
 	.nbuffers = 2,
 	.modes = {STARPU_W, STARPU_W}
 };
 
 static struct starpu_codelet cl_w_rw =
 {
-	.where = STARPU_CPU|STARPU_CUDA|STARPU_OPENCL,
-	.cpu_funcs = {cpu_codelet_null, NULL},
-	.cuda_funcs = {cuda_codelet_null, NULL},
-        .opencl_funcs = {opencl_codelet_null, NULL},
+	.cpu_funcs = {cpu_codelet_null},
+	.cuda_funcs = {cuda_codelet_null},
+        .opencl_funcs = {opencl_codelet_null},
+	.cpu_funcs_name = {"cpu_codelet_null"},
 	.nbuffers = 2,
 	.modes = {STARPU_W, STARPU_RW}
 };
 
 static struct starpu_codelet cl_rw_r =
 {
-	.where = STARPU_CPU|STARPU_CUDA|STARPU_OPENCL,
-	.cpu_funcs = {cpu_codelet_null, NULL},
-	.cuda_funcs = {cuda_codelet_null, NULL},
-        .opencl_funcs = {opencl_codelet_null, NULL},
+	.cpu_funcs = {cpu_codelet_null},
+	.cuda_funcs = {cuda_codelet_null},
+        .opencl_funcs = {opencl_codelet_null},
+	.cpu_funcs_name = {"cpu_codelet_null"},
 	.nbuffers = 2,
 	.modes = {STARPU_RW, STARPU_R}
 };
 
 static struct starpu_codelet cl_rw_w =
 {
-	.where = STARPU_CPU|STARPU_CUDA|STARPU_OPENCL,
-	.cpu_funcs = {cpu_codelet_null, NULL},
-	.cuda_funcs = {cuda_codelet_null, NULL},
-        .opencl_funcs = {opencl_codelet_null, NULL},
+	.cpu_funcs = {cpu_codelet_null},
+	.cuda_funcs = {cuda_codelet_null},
+        .opencl_funcs = {opencl_codelet_null},
+	.cpu_funcs_name = {"cpu_codelet_null"},
 	.nbuffers = 2,
 	.modes = {STARPU_RW, STARPU_W}
 };
 
 static struct starpu_codelet cl_rw_rw =
 {
-	.where = STARPU_CPU|STARPU_CUDA|STARPU_OPENCL,
-	.cpu_funcs = {cpu_codelet_null, NULL},
-	.cuda_funcs = {cuda_codelet_null, NULL},
-        .opencl_funcs = {opencl_codelet_null, NULL},
+	.cpu_funcs = {cpu_codelet_null},
+	.cuda_funcs = {cuda_codelet_null},
+        .opencl_funcs = {opencl_codelet_null},
+	.cpu_funcs_name = {"cpu_codelet_null"},
 	.nbuffers = 2,
 	.modes = {STARPU_RW, STARPU_RW}
 };
@@ -180,7 +186,7 @@ int main(int argc, char **argv)
 {
 	int ret;
 
-	ret = starpu_init(NULL);
+	ret = starpu_initialize(NULL, &argc, &argv);
 	if (ret == -ENODEV) return STARPU_TEST_SKIPPED;
 	STARPU_CHECK_RETURN_VALUE(ret, "starpu_init");
 
@@ -189,8 +195,8 @@ int main(int argc, char **argv)
 	ret = starpu_malloc((void **)&v2, VECTORSIZE*sizeof(unsigned));
 	STARPU_CHECK_RETURN_VALUE(ret, "starpu_malloc");
 
-	starpu_vector_data_register(&v_handle, 0, (uintptr_t)v, VECTORSIZE, sizeof(unsigned));
-	starpu_vector_data_register(&v_handle2, 0, (uintptr_t)v2, VECTORSIZE, sizeof(unsigned));
+	starpu_vector_data_register(&v_handle, STARPU_MAIN_RAM, (uintptr_t)v, VECTORSIZE, sizeof(unsigned));
+	starpu_vector_data_register(&v_handle2, STARPU_MAIN_RAM, (uintptr_t)v2, VECTORSIZE, sizeof(unsigned));
 
 	unsigned iter;
 	for (iter = 0; iter < N; iter++)
@@ -200,8 +206,8 @@ int main(int argc, char **argv)
 		task->handles[0] = v_handle;
 		task->handles[1] = v_handle2;
 
-		enum starpu_access_mode mode0 = select_random_mode();
-		enum starpu_access_mode mode1 = select_random_mode();
+		enum starpu_data_access_mode mode0 = select_random_mode();
+		enum starpu_data_access_mode mode1 = select_random_mode();
 
 		if (mode0 == STARPU_R && mode1 == STARPU_R)
 			task->cl = &cl_r_r;
@@ -225,15 +231,16 @@ int main(int argc, char **argv)
 		task->callback_func = callback;
 		task->callback_arg = NULL;
 
-		int ret = starpu_task_submit(task);
+		ret = starpu_task_submit(task);
 		if (ret == -ENODEV) goto enodev;
 		STARPU_CHECK_RETURN_VALUE(ret, "starpu_task_submit");
 	}
 
-	_STARPU_PTHREAD_MUTEX_LOCK(&mutex);
+	starpu_do_schedule();
+	STARPU_PTHREAD_MUTEX_LOCK(&mutex);
 	if (!finished)
-		_STARPU_PTHREAD_COND_WAIT(&cond, &mutex);
-	_STARPU_PTHREAD_MUTEX_UNLOCK(&mutex);
+		STARPU_PTHREAD_COND_WAIT(&cond, &mutex);
+	STARPU_PTHREAD_MUTEX_UNLOCK(&mutex);
 
 	starpu_data_unregister(v_handle);
 	starpu_data_unregister(v_handle2);
