@@ -1,6 +1,6 @@
 /* StarPU --- Runtime system for heterogeneous multicore architectures.
  *
- * Copyright (C) 2011  Université de Bordeaux 1
+ * Copyright (C) 2011, 2015-2016  Université de Bordeaux
  *
  * StarPU is free software; you can redistribute it and/or modify
  * it under the terms of the GNU Lesser General Public License as published by
@@ -15,18 +15,22 @@
  */
 
 #include <starpu.h>
-#include <starpu_opencl.h>
 #include "../helper.h"
 #include "scal.h"
 
+/*
+ * Trigger in-place partitioning by prefetching the whole data before
+ * partitioning
+ */
+
 int main(int argc, char **argv)
 {
 	unsigned *foo;
 	starpu_data_handle_t handle;
 	int ret;
-	int n, i, size;
+	unsigned n, i, size;
 
-	ret = starpu_init(NULL);
+	ret = starpu_initialize(NULL, &argc, &argv);
 	if (ret == -ENODEV) return STARPU_TEST_SKIPPED;
 	STARPU_CHECK_RETURN_VALUE(ret, "starpu_init");
 
@@ -48,7 +52,7 @@ int main(int argc, char **argv)
 	for (i = 0; i < size; i++)
 		foo[i] = i;
 
-	starpu_vector_data_register(&handle, 0, (uintptr_t)foo, size, sizeof(*foo));
+	starpu_vector_data_register(&handle, STARPU_MAIN_RAM, (uintptr_t)foo, size, sizeof(*foo));
 
 	/* Broadcast the data to force in-place partitioning */
 	for (i = 0; i < n; i++)
@@ -56,13 +60,14 @@ int main(int argc, char **argv)
 
 	struct starpu_data_filter f =
 	{
-		.filter_func = starpu_block_filter_func_vector,
+		.filter_func = starpu_vector_filter_block,
 		.nchildren = n,
 	};
 
 	starpu_data_partition(handle, &f);
 
-	for (i = 0; i < f.nchildren; i++) {
+	for (i = 0; i < f.nchildren; i++)
+	{
 		struct starpu_task *task = starpu_task_create();
 
 		task->handles[0] = starpu_data_get_sub_data(handle, 1, i);
@@ -78,17 +83,20 @@ int main(int argc, char **argv)
 	ret = starpu_task_wait_for_all();
 	STARPU_CHECK_RETURN_VALUE(ret, "starpu_task_wait_for_all");
 
-	starpu_data_unpartition(handle, 0);
+	starpu_data_unpartition(handle, STARPU_MAIN_RAM);
 	starpu_data_unregister(handle);
 	starpu_shutdown();
 
 	ret = EXIT_SUCCESS;
-	for (i = 0; i < size; i++) {
-		if (foo[i] != i*2) {
-			FPRINTF(stderr,"value %d is %d instead of %d\n", i, foo[i], 2*i);
+	for (i = 0; i < size; i++)
+	{
+		if (foo[i] != i*2)
+		{
+			FPRINTF(stderr,"value %u is %u instead of %u\n", i, foo[i], 2*i);
 			ret = EXIT_FAILURE;
 		}
 	}
+	free(foo);
 
         return ret;
 
