@@ -1,6 +1,6 @@
 /* StarPU --- Runtime system for heterogeneous multicore architectures.
  *
- * Copyright (C) 2011  Institut National de Recherche en Informatique et Automatique
+ * Copyright (C) 2011  INRIA
  *
  * StarPU is free software; you can redistribute it and/or modify
  * it under the terms of the GNU Lesser General Public License as published by
@@ -25,7 +25,7 @@
 /* Prototypes */
 static void register_data(void);
 static void unregister_data(void);
-static void test_block_cpu_func(void *buffers[], void *args);
+void test_block_cpu_func(void *buffers[], void *args);
 #ifdef STARPU_USE_CUDA
 extern void test_block_cuda_func(void *buffers[], void *_args);
 #endif
@@ -33,9 +33,8 @@ extern void test_block_cuda_func(void *buffers[], void *_args);
 extern void test_block_opencl_func(void *buffers[], void *args);
 #endif
 
-
-static starpu_data_handle_t block_handle;
-static starpu_data_handle_t block2_handle;
+static starpu_data_handle_t _block_handle;
+static starpu_data_handle_t _block2_handle;
 
 struct test_config block_config =
 {
@@ -46,14 +45,17 @@ struct test_config block_config =
 #ifdef STARPU_USE_OPENCL
 	.opencl_func   = test_block_opencl_func,
 #endif
-	.handle        = &block_handle,
-	.dummy_handle  = &block2_handle,
-	.copy_failed   = 0,
+#ifdef STARPU_USE_MIC
+	.cpu_func_name = "test_block_cpu_func",
+#endif
+	.handle        = &_block_handle,
+	.dummy_handle  = &_block2_handle,
+	.copy_failed   = SUCCESS,
 	.name          = "block_interface"
 };
 
-static int block[NX*NY*NZ];
-static int block2[NX*NY*NZ];
+static int _block[NX*NY*NZ];
+static int _block2[NX*NY*NZ];
 
 static void
 register_data(void)
@@ -64,37 +66,37 @@ register_data(void)
 	for (k = 0; k < NZ; k++)
 		for (j = 0; j < NY; j++)
 			for (i = 0; i < NX; i++)
-                                block[(k*NX*NY)+(j*NX)+i] = val++;
+                                _block[(k*NX*NY)+(j*NX)+i] = val++;
 
 	/* Registering data */
-	starpu_block_data_register(&block_handle,
-                                    0,
-                                    (uintptr_t)block,
+	starpu_block_data_register(&_block_handle,
+                                    STARPU_MAIN_RAM,
+                                    (uintptr_t)_block,
 				    NX,
 				    NX * NY,
 				    NX,
 				    NY,
 				    NZ,
-				    sizeof(block[0]));
-	starpu_block_data_register(&block2_handle,
-                                    0,
-                                    (uintptr_t)block2,
+				    sizeof(_block[0]));
+	starpu_block_data_register(&_block2_handle,
+                                    STARPU_MAIN_RAM,
+                                    (uintptr_t)_block2,
 				    NX,
 				    NX * NY,
 				    NX,
 				    NY,
 				    NZ,
-				    sizeof(block2[0]));
+				    sizeof(_block2[0]));
 }
 
 static void
 unregister_data(void)
 {
-	starpu_data_unregister(block_handle);
-	starpu_data_unregister(block2_handle);
+	starpu_data_unregister(_block_handle);
+	starpu_data_unregister(_block2_handle);
 }
 
-static void test_block_cpu_func(void *buffers[], void *args)
+void test_block_cpu_func(void *buffers[], void *args)
 {
 	STARPU_SKIP_IF_VALGRIND;
 
@@ -105,9 +107,9 @@ static void test_block_cpu_func(void *buffers[], void *args)
         unsigned ldy = STARPU_BLOCK_GET_LDY(buffers[0]);
         unsigned ldz = STARPU_BLOCK_GET_LDZ(buffers[0]);
 	int *block = (int *) STARPU_BLOCK_GET_PTR(buffers[0]);
-	unsigned int i, j, k;
+	int i, j, k;
 	int val = 0;
-	block_config.copy_failed = 0;
+	block_config.copy_failed = SUCCESS;
 	for (k = 0; k < nz; k++)
 	{
 		for (j = 0; j < ny; j++)
@@ -116,7 +118,7 @@ static void test_block_cpu_func(void *buffers[], void *args)
 			{
                                 if (block[(k*ldz)+(j*ldy)+i] != factor * val)
 				{
-					block_config.copy_failed = 1;
+					block_config.copy_failed = FAILURE;
 					return;
 				}
 				else
@@ -130,17 +132,16 @@ static void test_block_cpu_func(void *buffers[], void *args)
 }
 
 int
-main(void)
+main(int argc, char **argv)
 {
 	data_interface_test_summary *summary;
-	struct starpu_conf conf =
-	{
-		.ncpus   = -1,
-		.ncuda   = 2,
-		.nopencl = 1
-	};
+	struct starpu_conf conf;
+	starpu_conf_init(&conf);
+	conf.ncuda = 2;
+	conf.nopencl = 1;
+	conf.nmic = -1;
 
-	if (starpu_init(&conf) == -ENODEV || starpu_cpu_worker_get_count() == 0)
+	if (starpu_initialize(&conf, &argc, &argv) == -ENODEV || starpu_cpu_worker_get_count() == 0)
 		goto enodev;
 
 	register_data();
