@@ -1,7 +1,7 @@
 /* StarPU --- Runtime system for heterogeneous multicore architectures.
  *
- * Copyright (C) 2010-2011  Université de Bordeaux 1
- * Copyright (C) 2010, 2011, 2012  Centre National de la Recherche Scientifique
+ * Copyright (C) 2010-2014, 2016  Université de Bordeaux
+ * Copyright (C) 2010, 2011, 2012, 2013, 2016  CNRS
  *
  * StarPU is free software; you can redistribute it and/or modify
  * it under the terms of the GNU Lesser General Public License as published by
@@ -17,13 +17,17 @@
 
 #include <config.h>
 #include <starpu.h>
-#ifdef STARPU_USE_OPENCL
-#include <starpu_opencl.h>
-#endif
-#include <pthread.h>
 #include "../helper.h"
+#include <common/thread.h>
 
-#ifdef STARPU_SLOW_MACHINE
+/*
+ * Mimic the behavior of libstarpumpi, tested by a ring of threads which
+ * increment the same variable one after the other.
+ * This is the asynchronous version: the threads submit the series of
+ * synchronizations and tasks.
+ */
+
+#ifdef STARPU_QUICK_CHECK
 #  define NTHREADS_DEFAULT	4
 #  define NITER_DEFAULT		8
 #else
@@ -31,22 +35,22 @@
 #  define NITER_DEFAULT		128
 #endif
 
-static int nthreads = NTHREADS_DEFAULT;
-static int niter = NITER_DEFAULT;
+static unsigned nthreads = NTHREADS_DEFAULT;
+static unsigned niter = NITER_DEFAULT;
 
 //#define DEBUG_MESSAGES	1
 
-//static pthread_cond_t cond;
-//static pthread_mutex_t mutex;
+//static starpu_pthread_cond_t cond;
+//static starpu_pthread_mutex_t mutex;
 
 struct thread_data
 {
 	unsigned index;
 	unsigned val;
 	starpu_data_handle_t handle;
-	pthread_t thread;
+	starpu_pthread_t thread;
 
-	pthread_mutex_t recv_mutex;
+	starpu_pthread_mutex_t recv_mutex;
 	unsigned recv_flag; // set when a message is received
 	unsigned recv_buf;
 	struct thread_data *neighbour;
@@ -59,8 +63,8 @@ struct data_req
 	struct data_req *next;
 };
 
-static pthread_mutex_t data_req_mutex;
-static pthread_cond_t data_req_cond;
+static starpu_pthread_mutex_t data_req_mutex;
+static starpu_pthread_cond_t data_req_cond;
 struct data_req *data_req_list;
 unsigned progress_thread_running;
 
@@ -71,16 +75,14 @@ static struct thread_data problem_data[NTHREADS_DEFAULT];
  * successor. */
 
 #ifdef STARPU_USE_CUDA
-void cuda_codelet_unsigned_inc(void *descr[], __attribute__ ((unused)) void *cl_arg);
+void cuda_codelet_unsigned_inc(void *descr[], STARPU_ATTRIBUTE_UNUSED void *cl_arg);
 #endif
 #ifdef STARPU_USE_OPENCL
 void opencl_codelet_unsigned_inc(void *buffers[], void *args);
 #endif
 
-static void increment_handle_cpu_kernel(void *descr[], void *cl_arg __attribute__((unused)))
+void increment_handle_cpu_kernel(void *descr[], void *cl_arg STARPU_ATTRIBUTE_UNUSED)
 {
-	STARPU_SKIP_IF_VALGRIND;
-
 	unsigned *val = (unsigned *)STARPU_VARIABLE_GET_PTR(descr[0]);
 	*val += 1;
 
@@ -90,13 +92,16 @@ static void increment_handle_cpu_kernel(void *descr[], void *cl_arg __attribute_
 static struct starpu_codelet increment_handle_cl =
 {
 	.modes = { STARPU_RW },
-	.cpu_funcs = {increment_handle_cpu_kernel, NULL},
+	.cpu_funcs = {increment_handle_cpu_kernel},
 #ifdef STARPU_USE_CUDA
-	.cuda_funcs = {cuda_codelet_unsigned_inc, NULL},
+	.cuda_funcs = {cuda_codelet_unsigned_inc},
+	.cuda_flags = {STARPU_CUDA_ASYNC},
 #endif
 #ifdef STARPU_USE_OPENCL
-	.opencl_funcs = { opencl_codelet_unsigned_inc, NULL},
+	.opencl_funcs = { opencl_codelet_unsigned_inc},
+	.opencl_flags = {STARPU_OPENCL_ASYNC},
 #endif
+	.cpu_funcs_name = {"increment_handle_cpu_kernel"},
 	.nbuffers = 1
 };
 
@@ -123,7 +128,7 @@ static int test_recv_handle_async(void *arg)
 	int ret;
 	struct thread_data *thread_data = (struct thread_data *) arg;
 
-	_STARPU_PTHREAD_MUTEX_LOCK(&thread_data->recv_mutex);
+	STARPU_PTHREAD_MUTEX_LOCK(&thread_data->recv_mutex);
 
 	ret = (thread_data->recv_flag == 1);
 
@@ -133,12 +138,12 @@ static int test_recv_handle_async(void *arg)
 		thread_data->val = thread_data->recv_buf;
 	}
 
-	_STARPU_PTHREAD_MUTEX_UNLOCK(&thread_data->recv_mutex);
+	STARPU_PTHREAD_MUTEX_UNLOCK(&thread_data->recv_mutex);
 
 	if (ret)
 	{
 #ifdef DEBUG_MESSAGES
-		FPRINTF(stderr, "Thread %d received value %d from thread %d\n",
+		FPRINTF(stderr, "Thread %u received value %u from thread %d\n",
 			thread_data->index, thread_data->val, (thread_data->index - 1)%nthreads);
 #endif
 		starpu_data_release(thread_data->handle);
@@ -156,11 +161,11 @@ static void recv_handle_async(void *_thread_data)
 	req->test_arg = thread_data;
 	req->next = NULL;
 
-	_STARPU_PTHREAD_MUTEX_LOCK(&data_req_mutex);
+	STARPU_PTHREAD_MUTEX_LOCK(&data_req_mutex);
 	req->next = data_req_list;
 	data_req_list = req;
-	_STARPU_PTHREAD_COND_SIGNAL(&data_req_cond);
-	_STARPU_PTHREAD_MUTEX_UNLOCK(&data_req_mutex);
+	STARPU_PTHREAD_COND_SIGNAL(&data_req_cond);
+	STARPU_PTHREAD_MUTEX_UNLOCK(&data_req_mutex);
 }
 
 static int test_send_handle_async(void *arg)
@@ -169,14 +174,14 @@ static int test_send_handle_async(void *arg)
 	struct thread_data *thread_data = (struct thread_data *) arg;
 	struct thread_data *neighbour_data = thread_data->neighbour;
 
-	_STARPU_PTHREAD_MUTEX_LOCK(&neighbour_data->recv_mutex);
+	STARPU_PTHREAD_MUTEX_LOCK(&neighbour_data->recv_mutex);
 	ret = (neighbour_data->recv_flag == 0);
-	_STARPU_PTHREAD_MUTEX_UNLOCK(&neighbour_data->recv_mutex);
+	STARPU_PTHREAD_MUTEX_UNLOCK(&neighbour_data->recv_mutex);
 
 	if (ret)
 	{
 #ifdef DEBUG_MESSAGES
-		FPRINTF(stderr, "Thread %d sends value %d to thread %d\n", thread_data->index, thread_data->val, neighbour_data->index);
+		FPRINTF(stderr, "Thread %u sends value %u to thread %u\n", thread_data->index, thread_data->val, neighbour_data->index);
 #endif
 		starpu_data_release(thread_data->handle);
 	}
@@ -192,36 +197,36 @@ static void send_handle_async(void *_thread_data)
 //	FPRINTF(stderr, "send_handle_async\n");
 
 	/* send the message */
-	_STARPU_PTHREAD_MUTEX_LOCK(&neighbour_data->recv_mutex);
+	STARPU_PTHREAD_MUTEX_LOCK(&neighbour_data->recv_mutex);
 	neighbour_data->recv_buf = thread_data->val;
 	neighbour_data->recv_flag = 1;
-	_STARPU_PTHREAD_MUTEX_UNLOCK(&neighbour_data->recv_mutex);
+	STARPU_PTHREAD_MUTEX_UNLOCK(&neighbour_data->recv_mutex);
 
 	struct data_req *req = (struct data_req *) malloc(sizeof(struct data_req));
 	req->test_func = test_send_handle_async;
 	req->test_arg = thread_data;
 	req->next = NULL;
 
-	_STARPU_PTHREAD_MUTEX_LOCK(&data_req_mutex);
+	STARPU_PTHREAD_MUTEX_LOCK(&data_req_mutex);
 	req->next = data_req_list;
 	data_req_list = req;
-	_STARPU_PTHREAD_COND_SIGNAL(&data_req_cond);
-	_STARPU_PTHREAD_MUTEX_UNLOCK(&data_req_mutex);
+	STARPU_PTHREAD_COND_SIGNAL(&data_req_cond);
+	STARPU_PTHREAD_MUTEX_UNLOCK(&data_req_mutex);
 }
 
 static void *progress_func(void *arg)
 {
-	_STARPU_PTHREAD_MUTEX_LOCK(&data_req_mutex);
+	STARPU_PTHREAD_MUTEX_LOCK(&data_req_mutex);
 
 	progress_thread_running = 1;
-	_STARPU_PTHREAD_COND_SIGNAL(&data_req_cond);
+	STARPU_PTHREAD_COND_SIGNAL(&data_req_cond);
 
 	while (progress_thread_running)
 	{
 		struct data_req *req;
 
 		if (data_req_list == NULL)
-			_STARPU_PTHREAD_COND_WAIT(&data_req_cond, &data_req_mutex);
+			STARPU_PTHREAD_COND_WAIT(&data_req_cond, &data_req_mutex);
 
 		req = data_req_list;
 
@@ -230,19 +235,19 @@ static void *progress_func(void *arg)
 			data_req_list = req->next;
 			req->next = NULL;
 
-			_STARPU_PTHREAD_MUTEX_UNLOCK(&data_req_mutex);
+			STARPU_PTHREAD_MUTEX_UNLOCK(&data_req_mutex);
 
 			int ret = req->test_func(req->test_arg);
 
 			if (ret)
 			{
 				free(req);
-				_STARPU_PTHREAD_MUTEX_LOCK(&data_req_mutex);
+				STARPU_PTHREAD_MUTEX_LOCK(&data_req_mutex);
 			}
 			else
 			{
 				/* ret = 0 : the request is not finished, we put it back at the end of the list */
-				_STARPU_PTHREAD_MUTEX_LOCK(&data_req_mutex);
+				STARPU_PTHREAD_MUTEX_LOCK(&data_req_mutex);
 
 				struct data_req *req_aux = data_req_list;
 				if (!req_aux)
@@ -266,7 +271,7 @@ static void *progress_func(void *arg)
 			}
 		}
 	}
-	_STARPU_PTHREAD_MUTEX_UNLOCK(&data_req_mutex);
+	STARPU_PTHREAD_MUTEX_UNLOCK(&data_req_mutex);
 
 	return NULL;
 }
@@ -278,7 +283,7 @@ static void *thread_func(void *arg)
 	unsigned index = thread_data->index;
 	int ret;
 
-	starpu_variable_data_register(&thread_data->handle, 0, (uintptr_t)&thread_data->val, sizeof(unsigned));
+	starpu_variable_data_register(&thread_data->handle, STARPU_MAIN_RAM, (uintptr_t)&thread_data->val, sizeof(unsigned));
 
 	for (iter = 0; iter < niter; iter++)
 	{
@@ -322,7 +327,7 @@ int main(int argc, char **argv)
 	nthreads /= 4;
 #endif
 
-	ret = starpu_init(NULL);
+	ret = starpu_initialize(NULL, &argc, &argv);
 	if (ret == -ENODEV) return STARPU_TEST_SKIPPED;
 	STARPU_CHECK_RETURN_VALUE(ret, "starpu_init");
 
@@ -333,9 +338,9 @@ int main(int argc, char **argv)
 #endif
 
 	/* Create a thread to perform blocking calls */
-	pthread_t progress_thread;
-	_STARPU_PTHREAD_MUTEX_INIT(&data_req_mutex, NULL);
-	_STARPU_PTHREAD_COND_INIT(&data_req_cond, NULL);
+	starpu_pthread_t progress_thread;
+	STARPU_PTHREAD_MUTEX_INIT(&data_req_mutex, NULL);
+	STARPU_PTHREAD_COND_INIT(&data_req_cond, NULL);
 	data_req_list = NULL;
 	progress_thread_running = 0;
 
@@ -344,37 +349,37 @@ int main(int argc, char **argv)
 	{
 		problem_data[t].index = t;
 		problem_data[t].val = 0;
-		_STARPU_PTHREAD_MUTEX_INIT(&problem_data[t].recv_mutex, NULL);
+		STARPU_PTHREAD_MUTEX_INIT(&problem_data[t].recv_mutex, NULL);
 		problem_data[t].recv_flag = 0;
 		problem_data[t].neighbour = &problem_data[(t+1)%nthreads];
 	}
 
-	pthread_create(&progress_thread, NULL, progress_func, NULL);
+	starpu_pthread_create(&progress_thread, NULL, progress_func, NULL);
 
-	_STARPU_PTHREAD_MUTEX_LOCK(&data_req_mutex);
+	STARPU_PTHREAD_MUTEX_LOCK(&data_req_mutex);
 	while (!progress_thread_running)
-		_STARPU_PTHREAD_COND_WAIT(&data_req_cond, &data_req_mutex);
-	_STARPU_PTHREAD_MUTEX_UNLOCK(&data_req_mutex);
+		STARPU_PTHREAD_COND_WAIT(&data_req_cond, &data_req_mutex);
+	STARPU_PTHREAD_MUTEX_UNLOCK(&data_req_mutex);
 
 	for (t = 0; t < nthreads; t++)
 	{
-		ret = pthread_create(&problem_data[t].thread, NULL, thread_func, &problem_data[t]);
+		ret = starpu_pthread_create(&problem_data[t].thread, NULL, thread_func, &problem_data[t]);
 		STARPU_ASSERT(!ret);
 	}
 
 	for (t = 0; t < nthreads; t++)
 	{
-		ret = pthread_join(problem_data[t].thread, &retval);
+		ret = starpu_pthread_join(problem_data[t].thread, &retval);
 		STARPU_ASSERT(!ret);
 		STARPU_ASSERT(retval == NULL);
 	}
 
-	_STARPU_PTHREAD_MUTEX_LOCK(&data_req_mutex);
+	STARPU_PTHREAD_MUTEX_LOCK(&data_req_mutex);
 	progress_thread_running = 0;
-	_STARPU_PTHREAD_COND_SIGNAL(&data_req_cond);
-	_STARPU_PTHREAD_MUTEX_UNLOCK(&data_req_mutex);
+	STARPU_PTHREAD_COND_SIGNAL(&data_req_cond);
+	STARPU_PTHREAD_MUTEX_UNLOCK(&data_req_mutex);
 
-	ret = pthread_join(progress_thread, &retval);
+	ret = starpu_pthread_join(progress_thread, &retval);
 	STARPU_ASSERT(!ret);
 	STARPU_ASSERT(retval == NULL);
 
@@ -390,7 +395,7 @@ int main(int argc, char **argv)
 	ret = EXIT_SUCCESS;
 	if (problem_data[nthreads - 1].val != (nthreads * niter))
 	{
-		FPRINTF(stderr, "Final value : %u should be %d\n", problem_data[nthreads - 1].val, (nthreads * niter));
+		FPRINTF(stderr, "Final value : %u should be %u\n", problem_data[nthreads - 1].val, (nthreads * niter));
 		ret = EXIT_FAILURE;
 	}
 	starpu_data_release(last_handle);
@@ -402,5 +407,5 @@ int main(int argc, char **argv)
 
 	starpu_shutdown();
 
-	STARPU_RETURN(ret);
+	return ret;
 }
