@@ -1,8 +1,8 @@
 /* StarPU --- Runtime system for heterogeneous multicore architectures.
  *
- * Copyright (C) 2010-2012  Université de Bordeaux 1
+ * Copyright (C) 2010-2012, 2016  Université de Bordeaux
  * Copyright (C) 2010  Mehdi Juhoor <mjuhoor@gmail.com>
- * Copyright (C) 2010, 2011, 2012  Centre National de la Recherche Scientifique
+ * Copyright (C) 2010, 2011, 2012, 2013  CNRS
  *
  * StarPU is free software; you can redistribute it and/or modify
  * it under the terms of the GNU Lesser General Public License as published by
@@ -23,7 +23,12 @@
 #include <stdlib.h>
 #include "../helper.h"
 
-#ifdef STARPU_SLOW_MACHINE
+/*
+ * Test running a task on a partitioned data, then on the unpartitioned
+ * data, etc. in a loop
+ */
+
+#ifdef STARPU_QUICK_CHECK
 #define NITER		100
 #else
 #define NITER		1000
@@ -34,24 +39,25 @@ float *buffer;
 
 starpu_data_handle_t v_handle;
 
-static void dummy_codelet(void *descr[], __attribute__ ((unused)) void *_args)
+void dummy_codelet(void *descr[], STARPU_ATTRIBUTE_UNUSED void *_args)
 {
 }
 
 static struct starpu_codelet cl =
 {
 	.modes = { STARPU_RW },
-	.where = STARPU_CPU|STARPU_CUDA|STARPU_OPENCL,
-	.cpu_funcs = {dummy_codelet, NULL},
+	.cpu_funcs = {dummy_codelet},
 #ifdef STARPU_USE_CUDA
-	.cuda_funcs = {dummy_codelet, NULL},
+	.cuda_funcs = {dummy_codelet},
 #endif
 #ifdef STARPU_USE_OPENCL
-	.opencl_funcs = {dummy_codelet, NULL},
+	.opencl_funcs = {dummy_codelet},
 #endif
+	.cpu_funcs_name = {"dummy_codelet"},
 	.nbuffers = 1
 };
 
+static
 struct starpu_task* create_task(starpu_data_handle_t handle)
 {
 	struct starpu_task *task;
@@ -68,18 +74,18 @@ int main(int argc, char **argv)
 {
 	int ret;
 
-	ret = starpu_init(NULL);
+	ret = starpu_initialize(NULL, &argc, &argv);
 	if (ret == -ENODEV) return STARPU_TEST_SKIPPED;
 	STARPU_CHECK_RETURN_VALUE(ret, "starpu_init");
 
 	ret = starpu_malloc((void **)&buffer, VECTORSIZE);
 	STARPU_CHECK_RETURN_VALUE(ret, "starpu_malloc");
 
-	starpu_vector_data_register(&v_handle, 0, (uintptr_t)buffer, VECTORSIZE, sizeof(char));
+	starpu_vector_data_register(&v_handle, STARPU_MAIN_RAM, (uintptr_t)buffer, VECTORSIZE, sizeof(char));
 
 	struct starpu_data_filter f =
 	{
-		.filter_func = starpu_vector_divide_in_2_filter_func,
+		.filter_func = starpu_vector_filter_divide_in_2,
 		/* there are only 2 children */
 		.nchildren = 2,
 		/* the length of the first part */
@@ -106,7 +112,7 @@ int main(int argc, char **argv)
 		ret = starpu_task_wait_for_all();
 		STARPU_CHECK_RETURN_VALUE(ret, "starpu_task_wait_for_all");
 
-		starpu_data_unpartition(v_handle, 0);
+		starpu_data_unpartition(v_handle, STARPU_MAIN_RAM);
 
 		tasks[2] = create_task(v_handle);
 		ret = starpu_task_submit(tasks[2]);
