@@ -1,6 +1,6 @@
 /* StarPU --- Runtime system for heterogeneous multicore architectures.
  *
- * Copyright (C) 2012  Inria
+ * Copyright (C) 2012  INRIA
  *
  * StarPU is free software; you can redistribute it and/or modify
  * it under the terms of the GNU Lesser General Public License as published by
@@ -18,13 +18,16 @@
 #include <starpu.h>
 #include "../helper.h"
 
+/*
+ * Test the bundle interface, putting tasks working on the same data in
+ * the same bundle
+ */
+
 #define NB_BUNDLE 10
 #define NB_ITERATION 5
 
 void func_cpu(void *descr[], void *args)
 {
-	STARPU_SKIP_IF_VALGRIND;
-
 	float *x = (float *) STARPU_VARIABLE_GET_PTR(descr[0]);
 	float factor;
 
@@ -35,8 +38,8 @@ void func_cpu(void *descr[], void *args)
 struct starpu_codelet codelet =
 {
 	.modes = {STARPU_RW},
-	.where = STARPU_CPU,
-	.cpu_funcs = {func_cpu, NULL},
+	.cpu_funcs = {func_cpu},
+	.cpu_funcs_name = {"func_cpu"},
         .nbuffers = 1
 };
 
@@ -44,7 +47,12 @@ int main(int argc, char **argv)
 {
         int i, j, ret;
 
-	float data[NB_BUNDLE];
+	ret = starpu_initialize(NULL, &argc, &argv);
+	if (ret == -ENODEV) return STARPU_TEST_SKIPPED;
+	STARPU_CHECK_RETURN_VALUE(ret, "starpu_initialize");
+
+	float *data;
+	starpu_malloc((void**)&data, sizeof(*data) * NB_BUNDLE);
 	float factors[NB_BUNDLE];
         starpu_data_handle_t handles[NB_BUNDLE];
 
@@ -58,12 +66,8 @@ int main(int argc, char **argv)
 		factors[i] = NB_BUNDLE - i;
 	}
 
-	ret = starpu_init(NULL);
-	if (ret == -ENODEV) return STARPU_TEST_SKIPPED;
-	STARPU_CHECK_RETURN_VALUE(ret, "starpu_init");
-
 	for (i = 0; i < NB_BUNDLE; i++)
-		starpu_variable_data_register(&handles[i], 0, (uintptr_t)&data[i], sizeof(float));
+		starpu_variable_data_register(&handles[i], STARPU_MAIN_RAM, (uintptr_t)&data[i], sizeof(float));
 
         FPRINTF(stderr, "VALUES:");
 	for (i = 0; i < NB_BUNDLE; i++)
@@ -89,8 +93,9 @@ int main(int argc, char **argv)
 			STARPU_CHECK_RETURN_VALUE(ret, "starpu_task_insert");
 		}
 
+		/* Put one aside, just for fun */
 		ret = starpu_task_bundle_remove(bundles[i], task[NB_ITERATION / 2]);
-		STARPU_CHECK_RETURN_VALUE(ret, "starpu_task_remove");
+		STARPU_CHECK_RETURN_VALUE(ret, "starpu_task_bundle_remove");
 
 		for (j = 0; j < NB_ITERATION; j++)
 		{
@@ -122,6 +127,8 @@ int main(int argc, char **argv)
 		starpu_data_unregister(handles[i]);
 	}
 
+		starpu_free(data);
+
 	starpu_shutdown();
 
 	return EXIT_SUCCESS;
