@@ -1,7 +1,7 @@
 /* StarPU --- Runtime system for heterogeneous multicore architectures.
  *
- * Copyright (C) 2010, 2012-2014  Université de Bordeaux 1
- * Copyright (C) 2010, 2011, 2012, 2013  Centre National de la Recherche Scientifique
+ * Copyright (C) 2010, 2012-2016  Université de Bordeaux
+ * Copyright (C) 2010, 2011, 2012, 2013  CNRS
  *
  * StarPU is free software; you can redistribute it and/or modify
  * it under the terms of the GNU Lesser General Public License as published by
@@ -20,6 +20,10 @@
 
 #include "../helper.h"
 
+/*
+ * Test that one can resubmit a whole task graph repeatedly
+ */
+
 #ifdef STARPU_QUICK_CHECK
 static unsigned niter = 64;
 #else
@@ -44,22 +48,31 @@ static unsigned niter = 16384;
 static struct starpu_task taskA, taskB, taskC, taskD;
 
 static unsigned loop_cnt = 0;
-static unsigned check_cnt = 0;
+static unsigned *check_cnt;
 static starpu_pthread_cond_t cond = STARPU_PTHREAD_COND_INITIALIZER;
 static starpu_pthread_mutex_t mutex = STARPU_PTHREAD_MUTEX_INITIALIZER;
 
-static void dummy_func(void *descr[] STARPU_ATTRIBUTE_UNUSED, void *arg STARPU_ATTRIBUTE_UNUSED)
+extern void cuda_host_increment(void *descr[], void *_args);
+
+void cpu_increment(void *descr[], void *arg STARPU_ATTRIBUTE_UNUSED)
 {
-	(void) STARPU_ATOMIC_ADD(&check_cnt, 1);
+	unsigned *var = (unsigned *)STARPU_VARIABLE_GET_PTR(descr[0]);
+	(*var)++;
 }
 
 static struct starpu_codelet dummy_codelet =
 {
-	.cpu_funcs = {dummy_func, NULL},
-	.cuda_funcs = {dummy_func, NULL},
-	.opencl_funcs = {dummy_func, NULL},
+	.cpu_funcs = {cpu_increment},
+#ifdef STARPU_USE_CUDA
+	.cuda_funcs = {cuda_host_increment},
+	.cuda_flags = {STARPU_CUDA_ASYNC},
+#endif
+	// TODO
+	//.opencl_funcs = {dummy_func},
+	.cpu_funcs_name = {"cpu_increment"},
 	.model = NULL,
-	.nbuffers = 0
+	.modes = { STARPU_RW },
+	.nbuffers = 1
 };
 
 static void callback_task_D(void *arg STARPU_ATTRIBUTE_UNUSED)
@@ -93,26 +106,34 @@ int main(int argc, char **argv)
 //	double end;
 	int ret;
 
-	ret = starpu_init(NULL);
+	ret = starpu_initialize(NULL, &argc, &argv);
 	if (ret == -ENODEV) return STARPU_TEST_SKIPPED;
 	STARPU_CHECK_RETURN_VALUE(ret, "starpu_init");
 
+	starpu_data_set_default_sequential_consistency_flag(0);
+
+	starpu_malloc((void**)&check_cnt, sizeof(*check_cnt));
+	*check_cnt = 0;
+
+	starpu_data_handle_t check_data;
+	starpu_variable_data_register(&check_data, STARPU_MAIN_RAM, (uintptr_t)check_cnt, sizeof(*check_cnt));
+
 	starpu_task_init(&taskA);
 	taskA.cl = &dummy_codelet;
-	taskA.cl_arg = &taskA;
+	taskA.handles[0] = check_data;
 
 	starpu_task_init(&taskB);
 	taskB.cl = &dummy_codelet;
-	taskB.cl_arg = &taskB;
+	taskB.handles[0] = check_data;
 
 	starpu_task_init(&taskC);
 	taskC.cl = &dummy_codelet;
-	taskC.cl_arg = &taskC;
+	taskC.handles[0] = check_data;
 
 	starpu_task_init(&taskD);
 	taskD.cl = &dummy_codelet;
-	taskD.cl_arg = &taskD;
 	taskD.callback_func = callback_task_D;
+	taskD.handles[0] = check_data;
 
 	struct starpu_task *depsBC_array[1] = {&taskA};
 	starpu_task_declare_deps_array(&taskB, 1, depsBC_array);
@@ -126,28 +147,36 @@ int main(int argc, char **argv)
 	ret = starpu_task_submit(&taskC); if (ret == -ENODEV) goto enodev; STARPU_CHECK_RETURN_VALUE(ret, "starpu_task_submit");
 	ret = starpu_task_submit(&taskD); if (ret == -ENODEV) goto enodev; STARPU_CHECK_RETURN_VALUE(ret, "starpu_task_submit");
 
+	starpu_do_schedule();
 	/* Wait for the termination of all loops */
 	STARPU_PTHREAD_MUTEX_LOCK(&mutex);
 	if (loop_cnt < niter)
 		STARPU_PTHREAD_COND_WAIT(&cond, &mutex);
 	STARPU_PTHREAD_MUTEX_UNLOCK(&mutex);
 
-	STARPU_ASSERT(check_cnt == (4*loop_cnt));
+	starpu_data_acquire(check_data, STARPU_R);
+	starpu_data_release(check_data);
 
-	starpu_shutdown();
+	STARPU_ASSERT(*check_cnt == (4*loop_cnt));
 
-	/* Cleanup the statically allocated tasks after shutdown, as StarPU is still working on it after the callback */
+	starpu_free(check_cnt);
+	starpu_data_unregister(check_data);
+
+	starpu_task_wait_for_all();
 	starpu_task_clean(&taskA);
 	starpu_task_clean(&taskB);
 	starpu_task_clean(&taskC);
 	starpu_task_clean(&taskD);
 
+	starpu_shutdown();
+
 	return EXIT_SUCCESS;
 
 enodev:
 	fprintf(stderr, "WARNING: No one can execute this task\n");
 	/* yes, we do not perform the computation but we did detect that no one
  	 * could perform the kernel, so this is not an error from StarPU */
+	starpu_data_unregister(check_data);
 	starpu_shutdown();
 	return STARPU_TEST_SKIPPED;
 }
